@@ -760,5 +760,14 @@ TextAttrsMgr::TextPosTextAttr::
     }
   }
 
+  const nsIContent* content = aFrame->GetContent();
+  if (content && content->IsHTML()) {
+    const nsIAtom* tagName = content->Tag();
+    if (tagName == nsGkAtoms::sup) 
+      return eTextPosSuper;
+    if (tagName == nsGkAtoms::sub) 
+      return eTextPosSub;
+  }
+
   return eTextPosNone;
 }
