@@ -22,9 +22,9 @@ function test() {
 }
 
 function testCleanExit() {
-  gPane.activeThread.addOneTimeListener("framesadded", function() {
+  gDebugger.DebuggerController.activeThread.addOneTimeListener("framesadded", function() {
     Services.tm.currentThread.dispatch({ run: function() {
-      is(gDebugger.StackFrames.activeThread.paused, true,
+      is(gDebugger.DebuggerController.activeThread.paused, true,
         "Should be paused after the debugger statement.");
 
       closeDebuggerAndFinish(gTab);
