@@ -19,7 +19,7 @@ registerCleanupFunction(function() {
 
 function test() {
   Services.prefs.setBoolPref("devtools.gcli.enable", false);
-  addTab("data:text/html,test for bug 592442");
+  addTab("data:text/html;charset=utf-8,test for bug 592442");
   browser.addEventListener("load", testExtraneousClosingBrackets, true);
 }
 
