@@ -86,8 +86,7 @@ class nsBaseContentList : public nsINodeList
 public:
   nsBaseContentList()
   {
-    // Mark ourselves as a proxy
-    SetIsProxy();
+    SetIsDOMBinding();
   }
   virtual ~nsBaseContentList();
 
@@ -107,6 +106,7 @@ public:
 
   void AppendElement(nsIContent *aContent)
   {
+    NS_ASSERTION(aContent, "Element to append must not be null");
     mElements.AppendElement(aContent);
   }
   void MaybeAppendElement(nsIContent* aContent)
@@ -138,7 +138,7 @@ public:
 
   virtual PRInt32 IndexOf(nsIContent *aContent, bool aDoFlush);
 
-  virtual JSObject* WrapObject(JSContext *cx, XPCWrappedNativeScope *scope,
+  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
                                bool *triedToWrap) = 0;
 
 protected:
@@ -162,7 +162,7 @@ public:
   {
     return mRoot;
   }
-  virtual JSObject* WrapObject(JSContext *cx, XPCWrappedNativeScope *scope,
+  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
                                bool *triedToWrap);
 
 private:
@@ -293,7 +293,7 @@ public:
   virtual ~nsContentList();
 
   // nsWrapperCache
-  virtual JSObject* WrapObject(JSContext *cx, XPCWrappedNativeScope *scope,
+  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
                                bool *triedToWrap);
 
   // nsIDOMHTMLCollection
