@@ -67,15 +67,37 @@
 #include "nsContentUtils.h"
 #include "nsDOMFile.h"
 #include "nsDOMBlobBuilder.h"
+#include "nsIPrincipal.h"
+#include "nsIScriptObjectPrincipal.h"
+#include "mozilla/dom/bindings/XMLHttpRequestBinding.h"
+#include "mozilla/dom/bindings/XMLHttpRequestUploadBinding.h"
+
+#include "mozilla/Assertions.h"
 
 class nsILoadGroup;
 class AsyncVerifyRedirectCallbackForwarder;
 class nsIUnicodeDecoder;
+class nsIDOMFormData;
+
+#define IMPL_EVENT_HANDLER(_lowercase, _capitalized)                    \
+  JSObject* GetOn##_lowercase()                                         \
+  {                                                                     \
+    return GetListenerAsJSObject(mOn##_capitalized##Listener);          \
+  }                                                                     \
+  void SetOn##_lowercase(JSContext* aCx, JSObject* aCallback, nsresult& aRv) \
+  {                                                                     \
+    aRv = SetJSObjectListener(aCx, NS_LITERAL_STRING(#_lowercase),      \
+                              mOn##_capitalized##Listener,              \
+                              aCallback);                               \
+  }
 
 class nsXHREventTarget : public nsDOMEventTargetHelper,
                          public nsIXMLHttpRequestEventTarget
 {
 public:
+  typedef mozilla::dom::bindings::prototypes::XMLHttpRequestResponseType::value
+          XMLHttpRequestResponseType;
+
   virtual ~nsXHREventTarget() {}
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsXHREventTarget,
@@ -83,8 +105,45 @@ public:
   NS_DECL_NSIXMLHTTPREQUESTEVENTTARGET
   NS_FORWARD_NSIDOMEVENTTARGET(nsDOMEventTargetHelper::)
 
+  IMPL_EVENT_HANDLER(loadstart, LoadStart)
+  IMPL_EVENT_HANDLER(progress, Progress)
+  IMPL_EVENT_HANDLER(abort, Abort)
+  IMPL_EVENT_HANDLER(error, Error)
+  IMPL_EVENT_HANDLER(load, Load)
+  IMPL_EVENT_HANDLER(timeout, Timeout)
+  IMPL_EVENT_HANDLER(loadend, Loadend)
+  
   virtual void DisconnectFromOwner();
 protected:
+  static inline JSObject* GetListenerAsJSObject(nsDOMEventListenerWrapper* aWrapper)
+  {
+    if (!aWrapper) {
+      return nsnull;
+    }
+
+    nsCOMPtr<nsIXPConnectJSObjectHolder> holder =
+        do_QueryInterface(aWrapper->GetInner());
+    JSObject* obj;
+    return holder && NS_SUCCEEDED(holder->GetJSObject(&obj)) ? obj : nsnull;
+  }
+  inline nsresult SetJSObjectListener(JSContext* aCx,
+                                      const nsAString& aType,
+                                      nsRefPtr<nsDOMEventListenerWrapper>& aWrapper,
+                                      JSObject* aCallback)
+  {
+    nsCOMPtr<nsIDOMEventListener> listener;
+    if (aCallback) {
+      nsresult rv =
+        nsContentUtils::XPConnect()->WrapJS(aCx,
+                                            aCallback,
+                                            NS_GET_IID(nsIDOMEventListener),
+                                            getter_AddRefs(listener));
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+
+    return RemoveAddEventListener(aType, aWrapper, listener);
+  }
+
   nsRefPtr<nsDOMEventListenerWrapper> mOnLoadListener;
   nsRefPtr<nsDOMEventListenerWrapper> mOnErrorListener;
   nsRefPtr<nsDOMEventListenerWrapper> mOnAbortListener;
@@ -101,12 +160,23 @@ public:
   nsXMLHttpRequestUpload(nsDOMEventTargetHelper* aOwner)
   {
     BindToOwner(aOwner);
+    SetIsDOMBinding();
   }                                         
   NS_DECL_ISUPPORTS_INHERITED
   NS_FORWARD_NSIXMLHTTPREQUESTEVENTTARGET(nsXHREventTarget::)
   NS_FORWARD_NSIDOMEVENTTARGET(nsXHREventTarget::)
   NS_DECL_NSIXMLHTTPREQUESTUPLOAD
 
+  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
+                               bool *triedToWrap)
+  {
+    return mozilla::dom::bindings::prototypes::XMLHttpRequestUpload::Wrap(cx, scope, this, triedToWrap);
+  }
+  nsISupports* GetParentObject()
+  {
+    return GetOwner();
+  }
+
   bool HasListeners()
   {
     return mListenerManager && mListenerManager->HasListeners();
@@ -129,6 +199,43 @@ public:
   nsXMLHttpRequest();
   virtual ~nsXMLHttpRequest();
 
+  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
+                               bool *triedToWrap)
+  {
+    return mozilla::dom::bindings::prototypes::XMLHttpRequest::Wrap(cx, scope, this, triedToWrap);
+  }
+  nsISupports* GetParentObject()
+  {
+    return GetOwner();
+  }
+
+  // The WebIDL parser converts constructors into methods called _Constructor.
+  static already_AddRefed<nsXMLHttpRequest>
+  _Constructor(nsISupports* aGlobal, nsresult& aRv)
+  {
+    nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(aGlobal);
+    nsCOMPtr<nsIScriptObjectPrincipal> principal = do_QueryInterface(aGlobal);
+    if (!window || ! principal) {
+      aRv = NS_ERROR_FAILURE;
+      return NULL;
+    }
+
+    nsRefPtr<nsXMLHttpRequest> req = new nsXMLHttpRequest();
+    req->Construct(principal->GetPrincipal(), window);
+    return req.forget();
+  }
+
+  void Construct(nsIPrincipal* aPrincipal,
+                 nsPIDOMWindow* aOwnerWindow,
+                 nsIURI* aBaseURI = NULL)
+  {
+    MOZ_ASSERT(aPrincipal);
+    MOZ_ASSERT_IF(aOwnerWindow, aOwnerWindow->IsInnerWindow());
+    mPrincipal = aPrincipal;
+    BindToOwner(aOwnerWindow);
+    mBaseURI = aBaseURI;
+  }
+
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIXMLHttpRequest
@@ -164,6 +271,235 @@ public:
 
   NS_FORWARD_NSIDOMEVENTTARGET(nsXHREventTarget::)
 
+#ifdef DEBUG
+  void StaticAssertions();
+#endif
+
+  // event handler
+  IMPL_EVENT_HANDLER(readystatechange, Readystatechange)
+  JSObject* GetOnuploadprogress()
+  {
+    nsIDOMDocument* domDoc =
+      GetOwner() ? GetOwner()->GetExtantDocument() : NULL;
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
+    if (doc) {
+      doc->WarnOnceAbout(nsIDocument::eOnuploadprogress);
+    }
+    return GetListenerAsJSObject(mOnUploadProgressListener);
+  }
+  void SetOnuploadprogress(JSContext* aCx, JSObject* aCallback,
+                           nsresult& aRv)
+  {
+    nsIDOMDocument* domDoc =
+      GetOwner() ? GetOwner()->GetExtantDocument() : NULL;
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
+    if (doc) {
+      doc->WarnOnceAbout(nsIDocument::eOnuploadprogress);
+    }
+    aRv = SetJSObjectListener(aCx, NS_LITERAL_STRING("uploadprogress"),
+                              mOnUploadProgressListener, aCallback);
+  }
+
+  // states
+  uint16_t GetReadyState();
+
+  // request
+  void Open(const nsAString& aMethod, const nsAString& aUrl, bool aAsync,
+            const nsAString& aUser, const nsAString& aPassword, nsresult& aRv)
+  {
+    aRv = Open(NS_ConvertUTF16toUTF8(aMethod), NS_ConvertUTF16toUTF8(aUrl),
+               aAsync, aUser, aPassword);
+  }
+  void SetRequestHeader(const nsAString& aHeader, const nsAString& aValue,
+                        nsresult& aRv)
+  {
+    aRv = SetRequestHeader(NS_ConvertUTF16toUTF8(aHeader),
+                           NS_ConvertUTF16toUTF8(aValue));
+  }
+  uint32_t GetTimeout()
+  {
+    return mTimeoutMilliseconds;
+  }
+  void SetTimeout(uint32_t aTimeout, nsresult& aRv);
+  bool GetWithCredentials();
+  void SetWithCredentials(bool aWithCredentials, nsresult& aRv);
+  nsXMLHttpRequestUpload* GetUpload();
+
+private:
+  class RequestBody
+  {
+  public:
+    RequestBody() : mType(Uninitialized)
+    {
+    }
+    RequestBody(JSObject* aArrayBuffer) : mType(ArrayBuffer)
+    {
+      mValue.mArrayBuffer = aArrayBuffer;
+    }
+    RequestBody(nsIDOMBlob* aBlob) : mType(Blob)
+    {
+      mValue.mBlob = aBlob;
+    }
+    RequestBody(nsIDocument* aDocument) : mType(Document)
+    {
+      mValue.mDocument = aDocument;
+    }
+    RequestBody(const nsAString& aString) : mType(DOMString)
+    {
+      mValue.mString = &aString;
+    }
+    RequestBody(nsIDOMFormData* aFormData) : mType(FormData)
+    {
+      mValue.mFormData = aFormData;
+    }
+    RequestBody(nsIInputStream* aStream) : mType(InputStream)
+    {
+      mValue.mStream = aStream;
+    }
+
+    enum Type {
+      Uninitialized,
+      ArrayBuffer,
+      Blob,
+      Document,
+      DOMString,
+      FormData,
+      InputStream
+    };
+    union Value {
+      JSObject* mArrayBuffer;
+      nsIDOMBlob* mBlob;
+      nsIDocument* mDocument;
+      const nsAString* mString;
+      nsIDOMFormData* mFormData;
+      nsIInputStream* mStream;
+    };
+
+    Type GetType() const
+    {
+      MOZ_ASSERT(mType != Uninitialized);
+      return mType;
+    }
+    Value GetValue() const
+    {
+      MOZ_ASSERT(mType != Uninitialized);
+      return mValue;
+    }
+
+  private:
+    Type mType;
+    Value mValue;
+  };
+
+  static nsresult GetRequestBody(nsIVariant* aVariant,
+                                 JSContext* aCx,
+                                 const Nullable<RequestBody>& aBody,
+                                 nsIInputStream** aResult,
+                                 nsACString& aContentType,
+                                 nsACString& aCharset);
+
+  nsresult Send(JSContext *aCx, nsIVariant* aVariant, const Nullable<RequestBody>& aBody);
+  nsresult Send(JSContext *aCx, const Nullable<RequestBody>& aBody)
+  {
+    return Send(aCx, nsnull, aBody);
+  }
+  nsresult Send(JSContext *aCx, const RequestBody& aBody)
+  {
+    return Send(aCx, Nullable<RequestBody>(aBody));
+  }
+
+public:
+  void Send(JSContext *aCx, nsresult& aRv)
+  {
+    aRv = Send(aCx, Nullable<RequestBody>());
+  }
+  void Send(JSContext *aCx, JSObject* aArrayBuffer, nsresult& aRv)
+  {
+    NS_ASSERTION(aArrayBuffer, "Null should go to string version");
+    aRv = Send(aCx, RequestBody(aArrayBuffer));
+  }
+  void Send(JSContext *aCx, nsIDOMBlob* aBlob, nsresult& aRv)
+  {
+    NS_ASSERTION(aBlob, "Null should go to string version");
+    aRv = Send(aCx, RequestBody(aBlob));
+  }
+  void Send(JSContext *aCx, nsIDocument* aDoc, nsresult& aRv)
+  {
+    NS_ASSERTION(aDoc, "Null should go to string version");
+    aRv = Send(aCx, RequestBody(aDoc));
+  }
+  void Send(JSContext *aCx, const nsAString& aString, nsresult& aRv)
+  {
+    if (DOMStringIsNull(aString)) {
+      Send(aCx, aRv);
+    }
+    else {
+      aRv = Send(aCx, RequestBody(aString));
+    }
+  }
+  void Send(JSContext *aCx, nsIDOMFormData* aFormData, nsresult& aRv)
+  {
+    NS_ASSERTION(aFormData, "Null should go to string version");
+    aRv = Send(aCx, RequestBody(aFormData));
+  }
+  void Send(JSContext *aCx, nsIInputStream* aStream, nsresult& aRv)
+  {
+    NS_ASSERTION(aStream, "Null should go to string version");
+    aRv = Send(aCx, RequestBody(aStream));
+  }
+  void SendAsBinary(JSContext *aCx, const nsAString& aBody, nsresult& aRv);
+
+  void Abort();
+
+  // response
+  uint32_t GetStatus();
+  void GetStatusText(nsString& aStatusText);
+  void GetResponseHeader(const nsACString& aHeader, nsACString& aResult,
+                         nsresult& aRv);
+  void GetResponseHeader(const nsAString& aHeader, nsString& aResult,
+                         nsresult& aRv)
+  {
+    nsCString result;
+    GetResponseHeader(NS_ConvertUTF16toUTF8(aHeader), result, aRv);
+    if (result.IsVoid()) {
+      aResult.SetIsVoid(true);
+    }
+    else {
+      // We use UTF8ToNewUnicode here because it truncates after invalid UTF-8
+      // characters, CopyUTF8toUTF16 just doesn't copy in that case.
+      PRUint32 length;
+      PRUnichar* chars = UTF8ToNewUnicode(result, &length);
+      aResult.Adopt(chars, length);
+    }
+  }
+  void GetAllResponseHeaders(nsString& aResponseHeaders);
+  void OverrideMimeType(const nsAString& aMimeType)
+  {
+    // XXX Should we do some validation here?
+    mOverrideMimeType = aMimeType;
+  }
+  XMLHttpRequestResponseType GetResponseType()
+  {
+    return XMLHttpRequestResponseType(mResponseType);
+  }
+  void SetResponseType(XMLHttpRequestResponseType aType, nsresult& aRv);
+  JS::Value GetResponse(JSContext* aCx, nsresult& aRv);
+  void GetResponseText(nsString& aResponseText, nsresult& aRv);
+  nsIDocument* GetResponseXML(nsresult& aRv);
+
+  bool GetMozBackgroundRequest();
+  void SetMozBackgroundRequest(bool aMozBackgroundRequest, nsresult& aRv);
+  bool GetMultipart();
+  void SetMultipart(bool aMultipart, nsresult& aRv);
+
+  nsIChannel* GetChannel()
+  {
+    return mChannel;
+  }
+
+  // We need a GetInterface callable from JS for chrome JS
+  JS::Value GetInterface(JSContext* aCx, nsIJSIID* aIID, nsresult& aRv);
+
   // This creates a trusted readystatechange event, which is not cancelable and
   // doesn't bubble.
   static nsresult CreateReadystatechangeEvent(nsIDOMEvent** aDOMEvent);
@@ -253,12 +589,15 @@ protected:
   friend class AsyncVerifyRedirectCallbackForwarder;
   void OnRedirectVerifyCallback(nsresult result);
 
+  nsresult Open(const nsACString& method, const nsACString& url, bool async,
+                const nsAString& user, const nsAString& password);
+
   nsCOMPtr<nsISupports> mContext;
   nsCOMPtr<nsIPrincipal> mPrincipal;
   nsCOMPtr<nsIChannel> mChannel;
   // mReadRequest is different from mChannel for multipart requests
   nsCOMPtr<nsIRequest> mReadRequest;
-  nsCOMPtr<nsIDOMDocument> mResponseXML;
+  nsCOMPtr<nsIDocument> mResponseXML;
   nsCOMPtr<nsIChannel> mCORSPreflightChannel;
   nsTArray<nsCString> mCORSUnsafeHeaders;
 
@@ -304,17 +643,21 @@ protected:
 
   nsCString mResponseCharset;
 
-  enum {
+  enum ResponseType {
     XML_HTTP_RESPONSE_TYPE_DEFAULT,
     XML_HTTP_RESPONSE_TYPE_ARRAYBUFFER,
     XML_HTTP_RESPONSE_TYPE_BLOB,
     XML_HTTP_RESPONSE_TYPE_DOCUMENT,
-    XML_HTTP_RESPONSE_TYPE_TEXT,
     XML_HTTP_RESPONSE_TYPE_JSON,
+    XML_HTTP_RESPONSE_TYPE_TEXT,
     XML_HTTP_RESPONSE_TYPE_CHUNKED_TEXT,
     XML_HTTP_RESPONSE_TYPE_CHUNKED_ARRAYBUFFER,
     XML_HTTP_RESPONSE_TYPE_MOZ_BLOB
-  } mResponseType;
+  };
+
+  void SetResponseType(nsXMLHttpRequest::ResponseType aType, nsresult& aRv);
+
+  ResponseType mResponseType;
 
   // It is either a cached blob-response from the last call to GetResponse,
   // but is also explicitly set in OnStopRequest.
@@ -327,7 +670,7 @@ protected:
   // and mDOMFile is null.
   nsRefPtr<nsDOMBlobBuilder> mBuilder;
 
-  nsCString mOverrideMimeType;
+  nsString mOverrideMimeType;
 
   /**
    * The notification callbacks the channel had when Send() was
@@ -406,6 +749,8 @@ protected:
   nsTArray<RequestHeader> mModifiedRequestHeaders;
 };
 
+#undef IMPL_EVENT_HANDLER
+
 // helper class to expose a progress DOM Event
 
 class nsXMLHttpProgressEvent : public nsIDOMProgressEvent,
