@@ -113,8 +113,7 @@ DOMSVGPathSegList::~DOMSVGPathSegList()
 }
 
 JSObject*
-DOMSVGPathSegList::WrapObject(JSContext *cx, XPCWrappedNativeScope *scope,
-                              bool *triedToWrap)
+DOMSVGPathSegList::WrapObject(JSContext *cx, JSObject *scope, bool *triedToWrap)
 {
   return mozilla::dom::binding::SVGPathSegList::create(cx, scope, this,
                                                        triedToWrap);
