@@ -36,6 +36,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/FloatingPoint.h"
 #include "mozilla/Util.h"
 
 #include "txExpr.h"
@@ -396,8 +397,8 @@ txCoreFunctionCall::evaluate(txIEvalContext* aContext, txAExprResult** aResult)
             NS_ENSURE_SUCCESS(rv, rv);
 
             // check for NaN or +/-Inf
-            if (txDouble::isNaN(start) ||
-                txDouble::isInfinite(start) ||
+            if (MOZ_DOUBLE_IS_NaN(start) ||
+                MOZ_DOUBLE_IS_INFINITE(start) ||
                 start >= src.Length() + 0.5) {
                 aContext->recycler()->getEmptyStringResult(aResult);
 
@@ -412,7 +413,7 @@ txCoreFunctionCall::evaluate(txIEvalContext* aContext, txAExprResult** aResult)
                 NS_ENSURE_SUCCESS(rv, rv);
 
                 end += start;
-                if (txDouble::isNaN(end) || end < 0) {
+                if (MOZ_DOUBLE_IS_NaN(end) || end < 0) {
                     aContext->recycler()->getEmptyStringResult(aResult);
 
                     return NS_OK;
@@ -556,8 +557,8 @@ txCoreFunctionCall::evaluate(txIEvalContext* aContext, txAExprResult** aResult)
             rv = evaluateToNumber(mParams[0], aContext, &dbl);
             NS_ENSURE_SUCCESS(rv, rv);
 
-            if (!txDouble::isNaN(dbl) && !txDouble::isInfinite(dbl)) {
-                if (txDouble::isNeg(dbl) && dbl >= -0.5) {
+            if (!MOZ_DOUBLE_IS_NaN(dbl) && !MOZ_DOUBLE_IS_INFINITE(dbl)) {
+                if (MOZ_DOUBLE_IS_NEGATIVE(dbl) && dbl >= -0.5) {
                     dbl *= 0;
                 }
                 else {
@@ -573,9 +574,9 @@ txCoreFunctionCall::evaluate(txIEvalContext* aContext, txAExprResult** aResult)
             rv = evaluateToNumber(mParams[0], aContext, &dbl);
             NS_ENSURE_SUCCESS(rv, rv);
 
-            if (!txDouble::isNaN(dbl) &&
-                !txDouble::isInfinite(dbl) &&
-                !(dbl == 0 && txDouble::isNeg(dbl))) {
+            if (!MOZ_DOUBLE_IS_NaN(dbl) &&
+                !MOZ_DOUBLE_IS_INFINITE(dbl) &&
+                !(dbl == 0 && MOZ_DOUBLE_IS_NEGATIVE(dbl))) {
                 dbl = floor(dbl);
             }
 
@@ -587,8 +588,8 @@ txCoreFunctionCall::evaluate(txIEvalContext* aContext, txAExprResult** aResult)
             rv = evaluateToNumber(mParams[0], aContext, &dbl);
             NS_ENSURE_SUCCESS(rv, rv);
 
-            if (!txDouble::isNaN(dbl) && !txDouble::isInfinite(dbl)) {
-                if (txDouble::isNeg(dbl) && dbl > -1) {
+            if (!MOZ_DOUBLE_IS_NaN(dbl) && !MOZ_DOUBLE_IS_INFINITE(dbl)) {
+                if (MOZ_DOUBLE_IS_NEGATIVE(dbl) && dbl > -1) {
                     dbl *= 0;
                 }
                 else {
