@@ -55,6 +55,7 @@
 #include "nsAttrName.h"
 #include "nsTArray.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/StandardInteger.h"
 
 const PRUint32 kUnknownIndex = PRUint32(-1);
 
@@ -587,7 +588,7 @@ txXPathNodeUtils::getXSLTId(const txXPathNode& aNode,
                             const txXPathNode& aBase,
                             nsAString& aResult)
 {
-    PRUword nodeid = ((PRUword)aNode.mNode) - ((PRUword)aBase.mNode);
+    uintptr_t nodeid = ((uintptr_t)aNode.mNode) - ((uintptr_t)aBase.mNode);
     if (!aNode.isAttribute()) {
         CopyASCIItoUTF16(nsPrintfCString(kFmtSize, gPrintfFmt, nodeid),
                          aResult);
