@@ -36,6 +36,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/FloatingPoint.h"
+
 #include "txXSLTNumber.h"
 #include "nsGkAtoms.h"
 #include "txCore.h"
@@ -122,7 +124,7 @@ txXSLTNumber::getValueList(Expr* aValueExpr, txPattern* aCountPattern,
 
         double value = result->numberValue();
 
-        if (txDouble::isInfinite(value) || txDouble::isNaN(value) ||
+        if (MOZ_DOUBLE_IS_INFINITE(value) || MOZ_DOUBLE_IS_NaN(value) ||
             value < 0.5) {
             txDouble::toString(value, aValueString);
             return NS_OK;
