@@ -78,6 +78,7 @@
 #include "nsIWebNavigation.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "Connection.h"
+#include "MobileConnection.h"
 
 #ifdef MOZ_B2G_RIL
 #include "TelephonyFactory.h"
@@ -195,6 +196,11 @@ Navigator::Invalidate()
     mConnection = nsnull;
   }
 
+  if (mMobileConnection) {
+    mMobileConnection->Shutdown();
+    mMobileConnection = nsnull;
+  }
+
 #ifdef MOZ_B2G_BT
   if (mBluetooth) {
     mBluetooth = nsnull;
@@ -948,7 +954,7 @@ Navigator::GetMozBattery(nsIDOMMozBatteryManager** aBattery)
     *aBattery = nsnull;
 
     nsCOMPtr<nsPIDOMWindow> win(do_QueryReferent(mWindow));
-    NS_ENSURE_TRUE(win->GetDocShell(), NS_OK);
+    NS_ENSURE_TRUE(win && win->GetDocShell(), NS_OK);
 
     mBatteryManager = new battery::BatteryManager();
     mBatteryManager->Init(win);
@@ -972,8 +978,7 @@ Navigator::GetMozPower(nsIDOMMozPowerManager** aPower)
     mPowerManager->Init(win);
   }
 
-  nsCOMPtr<nsIDOMMozPowerManager> power =
-    do_QueryInterface(NS_ISUPPORTS_CAST(nsIDOMMozPowerManager*, mPowerManager));
+  nsCOMPtr<nsIDOMMozPowerManager> power(mPowerManager);
   power.forget(aPower);
 
   return NS_OK;
@@ -1140,6 +1145,37 @@ Navigator::GetMozConnection(nsIDOMMozConnection** aConnection)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+Navigator::GetMozMobileConnection(nsIDOMMozMobileConnection** aMobileConnection)
+{
+  *aMobileConnection = nsnull;
+
+  if (!mMobileConnection) {
+    nsCOMPtr<nsPIDOMWindow> window = do_QueryReferent(mWindow);
+    NS_ENSURE_TRUE(window && window->GetDocShell(), NS_OK);
+
+    // Chrome is always allowed access, so do the permission check only
+    // for non-chrome pages.
+    if (!nsContentUtils::IsCallerChrome()) {
+      nsCOMPtr<nsIDocument> doc = do_QueryInterface(window->GetExtantDocument());
+      NS_ENSURE_TRUE(doc, NS_OK);
+
+      nsCOMPtr<nsIURI> uri;
+      doc->NodePrincipal()->GetURI(getter_AddRefs(uri));
+
+      if (!nsContentUtils::URIIsChromeOrInPref(uri, "dom.mobileconnection.whitelist")) {
+        return NS_OK;
+      }
+    }
+
+    mMobileConnection = new network::MobileConnection();
+    mMobileConnection->Init(window);
+  }
+
+  NS_ADDREF(*aMobileConnection = mMobileConnection);
+  return NS_OK;
+}
+
 #ifdef MOZ_B2G_BT
 //*****************************************************************************
 //    nsNavigator::nsIDOMNavigatorBluetooth
@@ -1154,7 +1190,7 @@ Navigator::GetMozBluetooth(nsIDOMBluetoothAdapter** aBluetooth)
     nsCOMPtr<nsPIDOMWindow> window = do_QueryReferent(mWindow);
     NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
 
-    mBluetooth = new bluetooth::BluetoothAdapter();
+    mBluetooth = new bluetooth::BluetoothAdapter(window);
 
     bluetooth = mBluetooth;
   }
