@@ -37,33 +37,38 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "mozilla/Util.h"
-
 #include "WorkerScope.h"
 
 #include "jsapi.h"
 #include "jsdbgapi.h"
-
+#include "mozilla/Util.h"
+#include "mozilla/dom/bindings/DOMJSClass.h"
+#include "mozilla/dom/bindings/EventTargetBinding.h"
+#include "mozilla/dom/bindings/Utils.h"
+#include "mozilla/dom/bindings/XMLHttpRequestBinding.h"
+#include "mozilla/dom/bindings/XMLHttpRequestUploadBinding.h"
 #include "nsTraceRefcnt.h"
 #include "xpcpublic.h"
 
+#ifdef ANDROID
+#include <android/log.h>
+#endif
+
 #include "ChromeWorkerScope.h"
 #include "Events.h"
+#include "EventListenerManager.h"
 #include "EventTarget.h"
 #include "Exceptions.h"
 #include "File.h"
 #include "FileReaderSync.h"
-#include "ListenerManager.h"
 #include "Location.h"
+#include "ImageData.h"
 #include "Navigator.h"
 #include "Principal.h"
 #include "ScriptLoader.h"
 #include "Worker.h"
 #include "WorkerPrivate.h"
 #include "XMLHttpRequest.h"
-#ifdef ANDROID
-#include <android/log.h>
-#endif
 
 #include "WorkerInlines.h"
 
@@ -74,11 +79,12 @@
   JSPROP_ENUMERATE
 
 using namespace mozilla;
+using namespace mozilla::dom::bindings;
 USING_WORKERS_NAMESPACE
 
 namespace {
 
-class WorkerGlobalScope : public events::EventTarget
+class WorkerGlobalScope : public EventTarget
 {
   static JSClass sClass;
   static JSPropertySpec sProperties[];
@@ -124,13 +130,13 @@ public:
   static JSObject*
   InitClass(JSContext* aCx, JSObject* aObj, JSObject* aParentProto)
   {
-    return JS_InitClass(aCx, aObj, aParentProto, &sClass, Construct, 0,
+    return JS_InitClass(aCx, aObj, aParentProto, Class(), Construct, 0,
                         sProperties, sFunctions, NULL, NULL);
   }
 
 protected:
-  WorkerGlobalScope(WorkerPrivate* aWorker)
-  : mWorker(aWorker)
+  WorkerGlobalScope(JSContext* aCx, WorkerPrivate* aWorker)
+  : EventTarget(aCx), mWorker(aWorker)
   {
     MOZ_COUNT_CTOR(mozilla::dom::workers::WorkerGlobalScope);
     for (int32 i = 0; i < SLOT_COUNT; i++) {
@@ -143,25 +149,25 @@ protected:
     MOZ_COUNT_DTOR(mozilla::dom::workers::WorkerGlobalScope);
   }
 
-  void
-  TraceInstance(JSTracer* aTrc)
+  virtual void
+  _Trace(JSTracer* aTrc) MOZ_OVERRIDE
   {
     for (int32 i = 0; i < SLOT_COUNT; i++) {
       JS_CALL_VALUE_TRACER(aTrc, mSlots[i], "WorkerGlobalScope instance slot");
     }
     mWorker->TraceInternal(aTrc);
-    events::EventTarget::TraceInstance(aTrc);
+    EventTarget::_Trace(aTrc);
   }
 
-  void
-  FinalizeInstance(JSContext* aCx)
+  virtual void
+  _Finalize(JSFreeOp* aFop) MOZ_OVERRIDE
   {
-    events::EventTarget::FinalizeInstance(aCx);
+    EventTarget::_Finalize(aFop);
   }
 
 private:
   static JSBool
-  GetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  _GetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
     JS_ASSERT(JSID_TO_INT(aIdval) >= 0 && JSID_TO_INT(aIdval) < STRING_COUNT);
@@ -172,11 +178,22 @@ private:
       return false;
     }
 
-    return scope->GetEventListenerOnEventTarget(aCx, name + 2, aVp);
+    nsresult rv = NS_OK;
+
+    JSObject* listener =
+      scope->GetEventListener(NS_ConvertASCIItoUTF16(name + 2), rv);
+
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to get event listener!");
+      return false;
+    }
+
+    *aVp = listener ? OBJECT_TO_JSVAL(listener) : JSVAL_NULL;
+    return true;
   }
 
   static JSBool
-  SetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, JSBool aStrict,
+  _SetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, JSBool aStrict,
                    jsval* aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
@@ -188,7 +205,20 @@ private:
       return false;
     }
 
-    return scope->SetEventListenerOnEventTarget(aCx, name + 2, aVp);
+    if (JSVAL_IS_PRIMITIVE(*aVp)) {
+      JS_ReportError(aCx, "Not an event listener!");
+      return false;
+    }
+
+    nsresult rv = NS_OK;
+    scope->SetEventListener(NS_ConvertASCIItoUTF16(name + 2),
+                            JSVAL_TO_OBJECT(*aVp), rv);
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to set event listener!");
+      return false;
+    }
+
+    return true;
   }
 
   static WorkerGlobalScope*
@@ -312,22 +342,25 @@ private:
       return false;
     }
 
-    jsval adaptor;
-    if (!scope->GetEventListenerOnEventTarget(aCx, name + 2, &adaptor)) {
+    nsresult rv = NS_OK;
+
+    JSObject* adaptor =
+      scope->GetEventListener(NS_ConvertASCIItoUTF16(name + 2), rv);
+
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to get event listener!");
       return false;
     }
 
-    if (JSVAL_IS_VOID(adaptor)) {
+    if (!adaptor) {
       *aVp = JSVAL_NULL;
       return true;
     }
 
-    JS_ASSERT(JSVAL_IS_OBJECT(adaptor));
+    *aVp = js::GetFunctionNativeReserved(adaptor, SLOT_wrappedFunction);
 
-    jsval listener = js::GetFunctionNativeReserved(JSVAL_TO_OBJECT(adaptor),
-                                                   SLOT_wrappedFunction);
+    JS_ASSERT(!JSVAL_IS_PRIMITIVE(*aVp));
 
-    *aVp = listener;
     return true;
   }
 
@@ -341,8 +374,14 @@ private:
       return false;
     }
 
-    JSFunction* adaptor = js::NewFunctionWithReserved(aCx, UnwrapErrorEvent, 1, 0,
-                                                      JS_GetGlobalObject(aCx), "unwrap");
+    if (JSVAL_IS_PRIMITIVE(*aVp)) {
+      JS_ReportError(aCx, "Not an event listener!");
+      return false;
+    }
+
+    JSFunction* adaptor =
+      js::NewFunctionWithReserved(aCx, UnwrapErrorEvent, 1, 0,
+                                  JS_GetGlobalObject(aCx), "unwrap");
     if (!adaptor) {
       return false;
     }
@@ -356,8 +395,16 @@ private:
                                   OBJECT_TO_JSVAL(aObj));
     js::SetFunctionNativeReserved(listener, SLOT_wrappedFunction, *aVp);
 
-    jsval val = OBJECT_TO_JSVAL(listener);
-    return scope->SetEventListenerOnEventTarget(aCx, name + 2, &val);
+    nsresult rv = NS_OK;
+
+    scope->SetEventListener(NS_ConvertASCIItoUTF16(name + 2), listener, rv);
+
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to set event listener!");
+      return false;
+    }
+
+    return true;
   }
 
   static JSBool
@@ -592,8 +639,7 @@ JSClass WorkerGlobalScope::sClass = {
   "WorkerGlobalScope",
   0,
   JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-  JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
-  JSCLASS_NO_OPTIONAL_MEMBERS
+  JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub
 };
 
 JSPropertySpec WorkerGlobalScope::sProperties[] = {
@@ -602,7 +648,7 @@ JSPropertySpec WorkerGlobalScope::sProperties[] = {
   { sEventStrings[STRING_onerror], STRING_onerror, PROPERTY_FLAGS,
     GetOnErrorListener, SetOnErrorListener },
   { sEventStrings[STRING_onclose], STRING_onclose, PROPERTY_FLAGS,
-    GetEventListener, SetEventListener },
+    _GetEventListener, _SetEventListener },
   { "navigator", SLOT_navigator, PROPERTY_FLAGS, GetNavigator, 
     js_GetterOnlyPropertyStub },
   { "self", 0, PROPERTY_FLAGS, GetSelf, js_GetterOnlyPropertyStub },
@@ -629,7 +675,7 @@ const char* const WorkerGlobalScope::sEventStrings[STRING_COUNT] = {
 
 class DedicatedWorkerGlobalScope : public WorkerGlobalScope
 {
-  static JSClass sClass;
+  static DOMJSClass sClass;
   static JSPropertySpec sProperties[];
   static JSFunctionSpec sFunctions[];
 
@@ -646,32 +692,38 @@ public:
   static JSClass*
   Class()
   {
-    return &sClass;
+    return sClass.ToJSClass();
   }
 
   static JSObject*
   InitClass(JSContext* aCx, JSObject* aObj, JSObject* aParentProto)
   {
-    return JS_InitClass(aCx, aObj, aParentProto, &sClass, Construct, 0,
+    return JS_InitClass(aCx, aObj, aParentProto, Class(), Construct, 0,
                         sProperties, sFunctions, NULL, NULL);
   }
 
   static JSBool
   InitPrivate(JSContext* aCx, JSObject* aObj, WorkerPrivate* aWorkerPrivate)
   {
-    JS_ASSERT(JS_GetClass(aObj) == &sClass);
-    JS_ASSERT(!GetJSPrivateSafeish<DedicatedWorkerGlobalScope>(aObj));
+    JS_ASSERT(JS_GetClass(aObj) == Class());
 
-    DedicatedWorkerGlobalScope* priv =
-      new DedicatedWorkerGlobalScope(aWorkerPrivate);
-    SetJSPrivateSafeish(aObj, priv);
+    mozilla::dom::bindings::AllocateProtoOrIfaceCache(aObj);
 
+    nsRefPtr<DedicatedWorkerGlobalScope> scope =
+      new DedicatedWorkerGlobalScope(aCx, aWorkerPrivate);
+
+    js::SetReservedSlot(aObj, DOM_GLOBAL_OBJECT_SLOT, PRIVATE_TO_JSVAL(scope));
+
+    scope->SetIsDOMBinding();
+    scope->SetWrapper(aObj);
+
+    scope.forget();
     return true;
   }
 
 protected:
-  DedicatedWorkerGlobalScope(WorkerPrivate* aWorker)
-  : WorkerGlobalScope(aWorker)
+  DedicatedWorkerGlobalScope(JSContext* aCx, WorkerPrivate* aWorker)
+  : WorkerGlobalScope(aCx, aWorker)
   {
     MOZ_COUNT_CTOR(mozilla::dom::workers::DedicatedWorkerGlobalScope);
   }
@@ -681,12 +733,9 @@ protected:
     MOZ_COUNT_DTOR(mozilla::dom::workers::DedicatedWorkerGlobalScope);
   }
 
-  using WorkerGlobalScope::TraceInstance;
-  using WorkerGlobalScope::FinalizeInstance;
-
 private:
   static JSBool
-  GetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
+  _GetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, jsval* aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
     JS_ASSERT(JSID_TO_INT(aIdval) >= 0 && JSID_TO_INT(aIdval) < STRING_COUNT);
@@ -697,12 +746,23 @@ private:
       return false;
     }
 
-    return scope->GetEventListenerOnEventTarget(aCx, name + 2, aVp);
+    nsresult rv = NS_OK;
+
+    JSObject* listener =
+      scope->GetEventListener(NS_ConvertASCIItoUTF16(name + 2), rv);
+
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to get event listener!");
+      return false;
+    }
+
+    *aVp = listener ? OBJECT_TO_JSVAL(listener) : JSVAL_NULL;
+    return true;
   }
 
   static JSBool
-  SetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, JSBool aStrict,
-                   jsval* aVp)
+  _SetEventListener(JSContext* aCx, JSObject* aObj, jsid aIdval, JSBool aStrict,
+                    jsval* aVp)
   {
     JS_ASSERT(JSID_IS_INT(aIdval));
     JS_ASSERT(JSID_TO_INT(aIdval) >= 0 && JSID_TO_INT(aIdval) < STRING_COUNT);
@@ -713,19 +773,34 @@ private:
       return false;
     }
 
-    return scope->SetEventListenerOnEventTarget(aCx, name + 2, aVp);
+    if (JSVAL_IS_PRIMITIVE(*aVp)) {
+      JS_ReportError(aCx, "Not an event listener!");
+      return false;
+    }
+
+    nsresult rv = NS_OK;
+
+    scope->SetEventListener(NS_ConvertASCIItoUTF16(name + 2),
+                            JSVAL_TO_OBJECT(*aVp), rv);
+
+    if (NS_FAILED(rv)) {
+      JS_ReportError(aCx, "Failed to set event listener!");
+      return false;
+    }
+
+    return true;
   }
 
   static DedicatedWorkerGlobalScope*
   GetInstancePrivate(JSContext* aCx, JSObject* aObj, const char* aFunctionName)
   {
     JSClass* classPtr = JS_GetClass(aObj);
-    if (classPtr == &sClass) {
-      return GetJSPrivateSafeish<DedicatedWorkerGlobalScope>(aObj);
+    if (classPtr == Class()) {
+      return UnwrapDOMObject<DedicatedWorkerGlobalScope>(aObj, classPtr);
     }
 
     JS_ReportErrorNumber(aCx, js_GetErrorMessage, NULL,
-                         JSMSG_INCOMPATIBLE_PROTO, sClass.name, aFunctionName,
+                         JSMSG_INCOMPATIBLE_PROTO, Class()->name, aFunctionName,
                          classPtr->name);
     return NULL;
   }
@@ -734,7 +809,7 @@ private:
   Construct(JSContext* aCx, unsigned aArgc, jsval* aVp)
   {
     JS_ReportErrorNumber(aCx, js_GetErrorMessage, NULL, JSMSG_WRONG_CONSTRUCTOR,
-                         sClass.name);
+                         Class()->name);
     return false;
   }
 
@@ -752,25 +827,26 @@ private:
   }
 
   static void
-  Finalize(JSContext* aCx, JSObject* aObj)
+  Finalize(JSFreeOp* aFop, JSObject* aObj)
   {
-    JS_ASSERT(JS_GetClass(aObj) == &sClass);
+    JS_ASSERT(JS_GetClass(aObj) == Class());
     DedicatedWorkerGlobalScope* scope =
-      GetJSPrivateSafeish<DedicatedWorkerGlobalScope>(aObj);
+      UnwrapDOMObject<DedicatedWorkerGlobalScope>(aObj, Class());
     if (scope) {
-      scope->FinalizeInstance(aCx);
-      delete scope;
+      DestroyProtoOrIfaceCache(aObj);
+      scope->_Finalize(aFop);
     }
   }
 
   static void
   Trace(JSTracer* aTrc, JSObject* aObj)
   {
-    JS_ASSERT(JS_GetClass(aObj) == &sClass);
+    JS_ASSERT(JS_GetClass(aObj) == Class());
     DedicatedWorkerGlobalScope* scope =
-      GetJSPrivateSafeish<DedicatedWorkerGlobalScope>(aObj);
+      UnwrapDOMObject<DedicatedWorkerGlobalScope>(aObj, Class());
     if (scope) {
-      scope->TraceInstance(aTrc);
+      mozilla::dom::bindings::TraceProtoOrIfaceCache(aTrc, aObj);
+      scope->_Trace(aTrc);
     }
   }
 
@@ -797,17 +873,26 @@ private:
   }
 };
 
-JSClass DedicatedWorkerGlobalScope::sClass = {
-  "DedicatedWorkerGlobalScope",
-  JSCLASS_GLOBAL_FLAGS | JSCLASS_HAS_PRIVATE | JSCLASS_IMPLEMENTS_BARRIERS | JSCLASS_NEW_RESOLVE,
-  JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-  JS_EnumerateStub, reinterpret_cast<JSResolveOp>(Resolve), JS_ConvertStub,
-  Finalize, NULL, NULL, NULL, NULL, Trace
+MOZ_STATIC_ASSERT(prototypes::MaxProtoChainLength == 3,
+                  "The MaxProtoChainLength must match our manual DOMJSClasses");
+
+DOMJSClass DedicatedWorkerGlobalScope::sClass = {
+  {
+    "DedicatedWorkerGlobalScope",
+    JSCLASS_DOM_GLOBAL | JSCLASS_IS_DOMJSCLASS | JSCLASS_IMPLEMENTS_BARRIERS |
+    JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(3) | JSCLASS_NEW_RESOLVE,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
+    JS_EnumerateStub, reinterpret_cast<JSResolveOp>(Resolve), JS_ConvertStub,
+    Finalize, NULL, NULL, NULL, NULL, Trace
+  },
+  { prototypes::id::EventTarget_workers, prototypes::id::_ID_Count,
+    prototypes::id::_ID_Count },
+  -1, false, DOM_GLOBAL_OBJECT_SLOT
 };
 
 JSPropertySpec DedicatedWorkerGlobalScope::sProperties[] = {
   { sEventStrings[STRING_onmessage], STRING_onmessage, PROPERTY_FLAGS,
-    GetEventListener, SetEventListener },
+    _GetEventListener, _SetEventListener },
   { 0, 0, 0, NULL, NULL }
 };
 
@@ -825,8 +910,13 @@ WorkerGlobalScope::GetInstancePrivate(JSContext* aCx, JSObject* aObj,
                                       const char* aFunctionName)
 {
   JSClass* classPtr = JS_GetClass(aObj);
-  if (classPtr == &sClass || classPtr == DedicatedWorkerGlobalScope::Class()) {
-    return GetJSPrivateSafeish<WorkerGlobalScope>(aObj);
+
+  // We can only make DedicatedWorkerGlobalScope, not WorkerGlobalScope, so this
+  // should never happen.
+  JS_ASSERT(classPtr != Class());
+
+  if (classPtr == DedicatedWorkerGlobalScope::Class()) {
+    return UnwrapDOMObject<DedicatedWorkerGlobalScope>(aObj, classPtr);
   }
 
   JS_ReportErrorNumber(aCx, js_GetErrorMessage, NULL, JSMSG_INCOMPATIBLE_PROTO,
@@ -841,6 +931,8 @@ BEGIN_WORKERS_NAMESPACE
 JSObject*
 CreateDedicatedWorkerGlobalScope(JSContext* aCx)
 {
+  using namespace mozilla::dom::bindings::prototypes;
+
   WorkerPrivate* worker = GetWorkerPrivateFromContext(aCx);
   JS_ASSERT(worker);
 
@@ -868,7 +960,7 @@ CreateDedicatedWorkerGlobalScope(JSContext* aCx)
   //          -> Object
 
   JSObject* eventTargetProto =
-    events::InitEventTargetClass(aCx, global, nsnull);
+    EventTarget_workers::GetProtoObject(aCx, global, global);
   if (!eventTargetProto) {
     return NULL;
   }
@@ -895,11 +987,10 @@ CreateDedicatedWorkerGlobalScope(JSContext* aCx)
     return NULL;
   }
 
-  if (worker->IsChromeWorker()) {
-    if (!chromeworker::InitClass(aCx, global, workerProto, false) ||
-        !chromeworker::DefineChromeWorkerFunctions(aCx, global)) {
-      return NULL;
-    }
+  if (worker->IsChromeWorker() &&
+      (!chromeworker::InitClass(aCx, global, workerProto, false) ||
+       !DefineChromeWorkerFunctions(aCx, global))) {
+    return NULL;
   }
 
   // Init other classes we care about.
@@ -907,12 +998,19 @@ CreateDedicatedWorkerGlobalScope(JSContext* aCx)
       !file::InitClasses(aCx, global) ||
       !filereadersync::InitClass(aCx, global) ||
       !exceptions::InitClasses(aCx, global) ||
-      !xhr::InitClasses(aCx, global, eventTargetProto) ||
       !location::InitClass(aCx, global) ||
+      !imagedata::InitClass(aCx, global) ||
       !navigator::InitClass(aCx, global)) {
     return NULL;
   }
 
+  // Init other paris-bindings.
+  if (!XMLHttpRequest_workers::CreateInterfaceObjects(aCx, global, global) ||
+      !XMLHttpRequestUpload_workers::CreateInterfaceObjects(aCx, global,
+                                                            global)) {
+    return NULL;
+  }
+
   if (!JS_DefineProfilingFunctions(aCx, global)) {
     return NULL;
   }
