@@ -26,14 +26,15 @@
 
 #ifndef HB_OT_H
 #define HB_OT_H
+#define HB_OT_H_IN
 
 #include "hb.h"
 
 #include "hb-ot-layout.h"
-#include "hb-ot-shape.h"
 #include "hb-ot-tag.h"
 
 HB_BEGIN_DECLS
 HB_END_DECLS
 
+#undef HB_OT_H_IN
 #endif /* HB_OT_H */
