@@ -136,7 +136,7 @@ FT2FontEntry::CreateScaledFont(const gfxFontStyle *aStyle)
 
     // synthetic oblique by skewing via the font matrix
     bool needsOblique = !IsItalic() &&
-            (aStyle->style & (FONT_STYLE_ITALIC | FONT_STYLE_OBLIQUE));
+            (aStyle->style & (NS_FONT_STYLE_ITALIC | NS_FONT_STYLE_OBLIQUE));
 
     if (needsOblique) {
         const double kSkewFactor = 0.25;
@@ -343,12 +343,11 @@ FT2FontEntry::CairoFontFace()
 nsresult
 FT2FontEntry::ReadCMAP()
 {
-    if (mCmapInitialized) {
+    if (mCharacterMap) {
         return NS_OK;
     }
 
-    // attempt this once, if errors occur leave a blank cmap
-    mCmapInitialized = true;
+    nsRefPtr<gfxCharacterMap> charmap = new gfxCharacterMap();
 
     AutoFallibleTArray<PRUint8,16384> buffer;
     nsresult rv = GetFontTable(TTAG_cmap, buffer);
@@ -357,11 +356,18 @@ FT2FontEntry::ReadCMAP()
         bool unicodeFont;
         bool symbolFont;
         rv = gfxFontUtils::ReadCMAP(buffer.Elements(), buffer.Length(),
-                                    mCharacterMap, mUVSOffset,
+                                    *charmap, mUVSOffset,
                                     unicodeFont, symbolFont);
     }
 
     mHasCmapTable = NS_SUCCEEDED(rv);
+    if (mHasCmapTable) {
+        gfxPlatformFontList *pfl = gfxPlatformFontList::PlatformFontList();
+        mCharacterMap = pfl->FindCharMap(charmap);
+    } else {
+        // if error occurred, initialize to null cmap
+        mCharacterMap = new gfxCharacterMap();
+    }
     return rv;
 }
 
@@ -389,6 +395,23 @@ FT2FontEntry::GetFontTable(PRUint32 aTableTag,
     return NS_OK;
 }
 
+void
+FT2FontEntry::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf,
+                                  FontListSizes*    aSizes) const
+{
+    gfxFontEntry::SizeOfExcludingThis(aMallocSizeOf, aSizes);
+    aSizes->mFontListSize +=
+        mFilename.SizeOfExcludingThisIfUnshared(aMallocSizeOf);
+}
+
+void
+FT2FontEntry::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf,
+                                  FontListSizes*    aSizes) const
+{
+    aSizes->mFontListSize += aMallocSizeOf(this);
+    SizeOfExcludingThis(aMallocSizeOf, aSizes);
+}
+
 /*
  * FT2FontFamily
  * A standard gfxFontFamily; just adds a method used to support sending
@@ -710,6 +733,35 @@ AppendToFaceList(nsCString& aFaceList,
     aFaceList.Append(',');
 }
 
+void
+FT2FontEntry::CheckForBrokenFont()
+{
+    NS_ASSERTION(mFamily != nsnull, "font entry must belong to a family");
+
+    // note if the family is in the "bad underline" blacklist
+    if (mFamily->IsBadUnderlineFamily()) {
+        mIsBadUnderlineFont = true;
+    }
+
+    // bug 721719 - set the IgnoreGSUB flag on entries for Roboto
+    // because of unwanted on-by-default "ae" ligature.
+    // (See also AppendFaceFromFontListEntry.)
+    if (mFamily->Name().EqualsLiteral("roboto")) {
+        mIgnoreGSUB = true;
+    }
+
+    // bug 706888 - set the IgnoreGSUB flag on the broken version of
+    // Droid Sans Arabic from certain phones, as identified by the
+    // font checksum in the 'head' table
+    else if (mFamily->Name().EqualsLiteral("droid sans arabic")) {
+        const TT_Header *head = static_cast<const TT_Header*>
+            (FT_Get_Sfnt_Table(mFTFace, ft_sfnt_head));
+        if (head && head->CheckSum_Adjust == 0xe445242) {
+            mIgnoreGSUB = true;
+        }
+    }
+}
+
 void
 gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
                                         bool aStdFile,
@@ -763,16 +815,9 @@ gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
                 }
                 fe->mStandardFace = aStdFile;
                 family->AddFontEntry(fe);
-                if (family->IsBadUnderlineFamily()) {
-                    fe->mIsBadUnderlineFont = true;
-                }
 
-                // bug 721719 - set the IgnoreGSUB flag on entries for Roboto
-                // because of unwanted on-by-default "ae" ligature.
-                // (See also AppendFaceFromFontListEntry.)
-                if (name.EqualsLiteral("roboto")) {
-                    fe->mIgnoreGSUB = true;
-                }
+                // this depends on the entry having been added to its family
+                fe->CheckForBrokenFont();
 
                 AppendToFaceList(faceList, name, fe);
 #ifdef PR_LOGGING
@@ -980,20 +1025,9 @@ gfxFT2FontList::AppendFaceFromFontListEntry(const FontListEntry& aFLE,
             }
         }
         family->AddFontEntry(fe);
-        if (family->IsBadUnderlineFamily()) {
-            fe->mIsBadUnderlineFont = true;
-        }
 
-        // bug 721719 - set the IgnoreGSUB flag on entries for Roboto
-        // because of unwanted on-by-default "ae" ligature.
-        // This totally sucks, but if we don't hack around these broken fonts
-        // we get really bad text rendering, which we can't inflict on users. :(
-        // If we accumulate a few more examples of this stuff, it'll be time
-        // to create some prefs for the list of fonts where we need to ignore
-        // layout tables. Sigh.
-        if (name.EqualsLiteral("roboto")) {
-            fe->mIgnoreGSUB = true;
-        }
+        // this depends on the entry having been added to its family
+        fe->CheckForBrokenFont();
     }
 }
 
@@ -1093,8 +1127,10 @@ gfxFT2FontList::GetDefaultFont(const gfxFontStyle* aStyle, bool& aNeedsBold)
     }
 #elif defined(ANDROID)
     nsAutoString resolvedName;
-    if (ResolveFontName(NS_LITERAL_STRING("Droid Sans"), resolvedName))
+    if (ResolveFontName(NS_LITERAL_STRING("Roboto"), resolvedName) ||
+        ResolveFontName(NS_LITERAL_STRING("Droid Sans"), resolvedName)) {
         return FindFontForFamily(resolvedName, aStyle, aNeedsBold);
+    }
 #endif
     /* TODO: what about Qt or other platforms that may use this? */
     return nsnull;
