@@ -244,6 +244,7 @@ public:
   nsresult SetFrameTimeout(PRUint32 aFrameNum, PRInt32 aTimeout);
   nsresult SetFrameBlendMethod(PRUint32 aFrameNum, PRInt32 aBlendMethod);
   nsresult SetFrameHasNoAlpha(PRUint32 aFrameNum);
+  nsresult SetFrameAsNonPremult(PRUint32 aFrameNum, bool aIsNonPremult);
 
   /**
    * Sets the size of the container. This should only be called by the
@@ -635,7 +636,7 @@ private: // data
 
   // Discard members
   PRUint32                   mLockCount;
-  DiscardTrackerNode         mDiscardTrackerNode;
+  DiscardTracker::Node       mDiscardTrackerNode;
 
   // Source data members
   FallibleTArray<char>       mSourceData;
