@@ -37,6 +37,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "imgFrame.h"
+#include "DiscardTracker.h"
 
 #include <limits.h>
 
@@ -67,6 +68,8 @@ static PRUint32 gTotalDDBSize = 0;
 
 #endif
 
+using namespace mozilla::image;
+
 // Returns true if an image of aWidth x aHeight is allowed and legal.
 static bool AllowedImageSize(PRInt32 aWidth, PRInt32 aHeight)
 {
@@ -142,11 +145,13 @@ imgFrame::imgFrame() :
   mSinglePixel(false),
   mNeverUseDeviceSurface(false),
   mFormatChanged(false),
-  mCompositingFailed(false)
+  mCompositingFailed(false),
+  mNonPremult(false),
 #ifdef USE_WIN_SURFACE
-  , mIsDDBSurface(false)
+  mIsDDBSurface(false),
 #endif
-  , mLocked(false)
+  mLocked(false),
+  mInformedDiscardTracker(false)
 {
   static bool hasCheckedOptimize = false;
   if (!hasCheckedOptimize) {
@@ -166,6 +171,10 @@ imgFrame::~imgFrame()
       gTotalDDBSize -= mSize.width * mSize.height * 4;
   }
 #endif
+
+  if (mInformedDiscardTracker) {
+    DiscardTracker::InformAllocation(-4 * mSize.height * mSize.width);
+  }
 }
 
 nsresult imgFrame::Init(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, 
@@ -227,6 +236,14 @@ nsresult imgFrame::Init(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight,
 #endif
   }
 
+  // Inform the discard tracker that we've allocated some memory, but only if
+  // we're not a paletted image (paletted images are not usually large and are
+  // used only for animated frames, which we don't discard).
+  if (!mPalettedImageData) {
+    DiscardTracker::InformAllocation(4 * mSize.width * mSize.height);
+    mInformedDiscardTracker = true;
+  }
+
   return NS_OK;
 }
 
@@ -238,6 +255,11 @@ nsresult imgFrame::Optimize()
   if (mPalettedImageData || mOptSurface || mSinglePixel)
     return NS_OK;
 
+  // Don't do single-color opts on non-premult data.
+  // Cairo doesn't support non-premult single-colors.
+  if (mNonPremult)
+    return NS_OK;
+
   /* Figure out if the entire image is a constant color */
 
   // this should always be true
@@ -254,11 +276,12 @@ nsresult imgFrame::Optimize()
       if (mFormat == gfxASurface::ImageFormatARGB32 ||
           mFormat == gfxASurface::ImageFormatRGB24)
       {
-        mSinglePixelColor = gfxRGBA
-          (firstPixel,
-           (mFormat == gfxImageSurface::ImageFormatRGB24 ?
-            gfxRGBA::PACKED_XRGB :
-            gfxRGBA::PACKED_ARGB_PREMULTIPLIED));
+        // Should already be premult if desired.
+        gfxRGBA::PackedColorType inputType = gfxRGBA::PACKED_XRGB;
+        if (mFormat == gfxASurface::ImageFormatARGB32)
+          inputType = gfxRGBA::PACKED_ARGB_PREMULTIPLIED;
+
+        mSinglePixelColor = gfxRGBA(firstPixel, inputType);
 
         mSinglePixel = true;
 
@@ -271,6 +294,14 @@ nsresult imgFrame::Optimize()
 #ifdef XP_MACOSX
         mQuartzSurface = nsnull;
 #endif
+
+        // We just dumped most of our allocated memory, so tell the discard
+        // tracker that we're not using any at all.
+        if (mInformedDiscardTracker) {
+          DiscardTracker::InformAllocation(-4 * mSize.width * mSize.height);
+          mInformedDiscardTracker = false;
+        }
+
         return NS_OK;
       }
     }
@@ -410,6 +441,7 @@ imgFrame::SurfaceForDrawing(bool               aDoPadding,
     }
     tmpCtx.Rectangle(available);
     tmpCtx.Fill();
+
     return SurfaceWithFormat(new gfxSurfaceDrawable(surface, size), format);
   }
 
@@ -434,7 +466,8 @@ imgFrame::SurfaceForDrawing(bool               aDoPadding,
 
 void imgFrame::Draw(gfxContext *aContext, gfxPattern::GraphicsFilter aFilter,
                     const gfxMatrix &aUserSpaceToImageSpace, const gfxRect& aFill,
-                    const nsIntMargin &aPadding, const nsIntRect &aSubimage)
+                    const nsIntMargin &aPadding, const nsIntRect &aSubimage,
+                    PRUint32 aImageFlags)
 {
   SAMPLE_LABEL("image", "imgFrame::Draw");
   NS_ASSERTION(!aFill.IsEmpty(), "zero dest size --- fix caller");
@@ -459,7 +492,8 @@ void imgFrame::Draw(gfxContext *aContext, gfxPattern::GraphicsFilter aFilter,
   NS_ASSERTION(!sourceRect.Intersect(subimage).IsEmpty(),
                "We must be allowed to sample *some* source pixels!");
 
-  bool doTile = !imageRect.Contains(sourceRect);
+  bool doTile = !imageRect.Contains(sourceRect) &&
+                !(aImageFlags & imgIContainer::FLAG_CLAMP);
   SurfaceWithFormat surfaceResult =
     SurfaceForDrawing(doPadding, doPartialDecode, doTile, aPadding,
                       userSpaceToImageSpace, fill, subimage, sourceRect,
@@ -469,7 +503,7 @@ void imgFrame::Draw(gfxContext *aContext, gfxPattern::GraphicsFilter aFilter,
     gfxUtils::DrawPixelSnapped(aContext, surfaceResult.mDrawable,
                                userSpaceToImageSpace,
                                subimage, sourceRect, imageRect, fill,
-                               surfaceResult.mFormat, aFilter);
+                               surfaceResult.mFormat, aFilter, aImageFlags);
   }
 }
 
@@ -493,6 +527,8 @@ nsresult imgFrame::Extract(const nsIntRect& aRegion, imgFrame** aResult)
                                mFormat, mPaletteDepth);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  subImage->SetAsNonPremult(mNonPremult);
+
   // scope to destroy ctx
   {
     gfxContext ctx(subImage->ThebesSurface());
@@ -748,6 +784,11 @@ void imgFrame::SetHasNoAlpha()
   }
 }
 
+void imgFrame::SetAsNonPremult(bool aIsNonPremult)
+{
+  mNonPremult = aIsNonPremult;
+}
+
 bool imgFrame::GetCompositingFailed() const
 {
   return mCompositingFailed;
