@@ -188,6 +188,12 @@ JSD_GetMinorVersion(void);
 extern JSD_PUBLIC_API(JSContext*)
 JSD_GetDefaultJSContext(JSDContext* jsdc);
 
+/*
+* Returns a JSRuntime this context is associated with
+*/
+extern JSD_PUBLIC_API(JSRuntime*)
+JSD_GetJSRuntime(JSDContext* jsdc);
+
 /*
 * Set the private data for this context, returns previous value
 */
