@@ -108,15 +108,6 @@ ParseNode::clear()
     pn_parens = false;
 }
 
-bool
-FunctionBox::joinable() const
-{
-    return function()->isNullClosure() &&
-           (tcflags & (TCF_FUN_USES_ARGUMENTS |
-                       TCF_FUN_USES_OWN_NAME |
-                       TCF_COMPILE_N_GO)) == TCF_COMPILE_N_GO;
-}
-
 bool
 FunctionBox::inAnyDynamicScope() const
 {
