@@ -50,6 +50,50 @@
 
 namespace js {
 
+/*
+ * Indicates a location in the stack that an upvar value can be retrieved from
+ * as a two tuple of (level, slot).
+ *
+ * Some existing client code uses the level value as a delta, or level "skip"
+ * quantity. We could probably document that through use of more types at some
+ * point in the future.
+ */
+class UpvarCookie
+{
+    uint32_t value;
+
+    static const uint32_t FREE_VALUE = 0xfffffffful;
+
+    void checkInvariants() {
+        JS_STATIC_ASSERT(sizeof(UpvarCookie) == sizeof(uint32_t));
+        JS_STATIC_ASSERT(UPVAR_LEVEL_LIMIT < FREE_LEVEL);
+    }
+
+  public:
+    /*
+     * All levels above-and-including FREE_LEVEL are reserved so that
+     * FREE_VALUE can be used as a special value.
+     */
+    static const uint16_t FREE_LEVEL = 0x3fff;
+
+    /*
+     * If a function has a higher static level than this limit, we will not
+     * optimize it using UPVAR opcodes.
+     */
+    static const uint16_t UPVAR_LEVEL_LIMIT = 16;
+    static const uint16_t CALLEE_SLOT = 0xffff;
+    static bool isLevelReserved(uint16_t level) { return level >= FREE_LEVEL; }
+
+    bool isFree() const { return value == FREE_VALUE; }
+    /* isFree check should be performed before using these accessors. */
+    uint16_t level() const { JS_ASSERT(!isFree()); return uint16_t(value >> 16); }
+    uint16_t slot() const { JS_ASSERT(!isFree()); return uint16_t(value); }
+
+    void set(const UpvarCookie &other) { set(other.level(), other.slot()); }
+    void set(uint16_t newLevel, uint16_t newSlot) { value = (uint32_t(newLevel) << 16) | newSlot; }
+    void makeFree() { set(0xffff, 0xffff); JS_ASSERT(isFree()); }
+};
+
 /*
  * Parsing builds a tree of nodes that directs code generation.  This tree is
  * not a concrete syntax tree in all respects (for example, || and && are left
@@ -690,6 +734,8 @@ struct ParseNode {
     newBinaryOrAppend(ParseNodeKind kind, JSOp op, ParseNode *left, ParseNode *right,
                       TreeContext *tc);
 
+    inline PropertyName *atom() const;
+
     /*
      * The pn_expr and lexdef members are arms of an unsafe union. Unless you
      * know exactly what you're doing, use only the following methods to access
@@ -711,6 +757,8 @@ struct ParseNode {
     ParseNode  *maybeExpr()   { return pn_used ? NULL : expr(); }
     Definition *maybeLexDef() { return pn_used ? lexdef() : NULL; }
 
+    Definition *resolve();
+
 /* PN_FUNC and PN_NAME pn_dflags bits. */
 #define PND_LET         0x01            /* let (block-scoped) binding */
 #define PND_CONST       0x02            /* const binding (orthogonal to let) */
@@ -721,15 +769,14 @@ struct ParseNode {
 #define PND_GVAR        0x40            /* gvar binding, can't close over
                                            because it could be deleted */
 #define PND_PLACEHOLDER 0x80            /* placeholder definition for lexdep */
-#define PND_FUNARG     0x100            /* downward or upward funarg usage */
-#define PND_BOUND      0x200            /* bound to a stack or global slot */
-#define PND_DEOPTIMIZED 0x400           /* former pn_used name node, pn_lexdef
+#define PND_BOUND      0x100            /* bound to a stack or global slot */
+#define PND_DEOPTIMIZED 0x200           /* former pn_used name node, pn_lexdef
                                            still valid, but this use no longer
                                            optimizable via an upvar opcode */
-#define PND_CLOSED      0x800           /* variable is closed over */
+#define PND_CLOSED      0x400           /* variable is closed over */
 
 /* Flags to propagate from uses to definition. */
-#define PND_USE2DEF_FLAGS (PND_ASSIGNED | PND_FUNARG | PND_CLOSED)
+#define PND_USE2DEF_FLAGS (PND_ASSIGNED | PND_CLOSED)
 
 /* PN_LIST pn_xflags bits. */
 #define PNX_STRCAT      0x01            /* PNK_ADD list has string term */
@@ -772,7 +819,6 @@ struct ParseNode {
     bool isPlaceholder() const  { return test(PND_PLACEHOLDER); }
     bool isDeoptimized() const  { return test(PND_DEOPTIMIZED); }
     bool isAssigned() const     { return test(PND_ASSIGNED); }
-    bool isFunArg() const       { return test(PND_FUNARG); }
     bool isClosed() const       { return test(PND_CLOSED); }
 
     /*
@@ -789,9 +835,6 @@ struct ParseNode {
      */
     bool isTopLevel() const     { return test(PND_TOPLEVEL); }
 
-    /* Defined below, see after struct Definition. */
-    void setFunArg();
-
     void become(ParseNode *pn2);
     void clear();
 
@@ -1387,52 +1430,11 @@ void DumpParseTree(ParseNode *pn, int indent = 0);
  * When the compiler unwinds from the outermost tc, tc->lexdeps contains the
  * definition nodes with use chains for all free variables. These are either
  * global variables or reference errors.
- *
- * We analyze whether a binding is initialized, whether the bound names is ever
- * assigned apart from its initializer, and if the bound name definition or use
- * is in a direct child of a block. These PND_* flags allow a subset dominance
- * computation telling whether an initialized var dominates its uses. An inner
- * function using only such outer vars (and formal parameters) can be optimized
- * into a flat closure. See JSOP_{GET,CALL}DSLOT.
- *
- * Another important subset dominance relation: ... { var x = ...; ... x ... }
- * where x is not assigned after initialization and not used outside the block.
- * This style is common in the absence of 'let'. Even though the var x is not
- * at top level, we can tell its initialization dominates all uses cheaply,
- * because the above one-pass algorithm sees the definition before any uses,
- * and because all uses are contained in the same block as the definition.
- *
- * We also analyze function uses to flag upward/downward funargs.  If a lambda
- * post-dominates each of its upvars' sole, inevitable (i.e. not hidden behind
- * conditions or within loops or the like) initialization or assignment; then
- * we can optimize the lambda as a flat closure (after Chez Scheme's display
- * closures).
  */
 #define dn_uses         pn_link
 
 struct Definition : public ParseNode
 {
-    /*
-     * We store definition pointers in PN_NAMESET AtomDefnMapPtrs in the AST,
-     * but due to redefinition these nodes may become uses of other
-     * definitions.  This is unusual, so we simply chase the pn_lexdef link to
-     * find the final definition node. See functions called from
-     * js::frontend::AnalyzeFunctions.
-     *
-     * FIXME: MakeAssignment mutates for want of a parent link...
-     */
-    Definition *resolve() {
-        ParseNode *pn = this;
-        while (!pn->isDefn()) {
-            if (pn->isAssignment()) {
-                pn = pn->pn_left;
-                continue;
-            }
-            pn = pn->lexdef();
-        }
-        return (Definition *) pn;
-    }
-
     bool isFreeVar() const {
         JS_ASSERT(isDefn());
         return pn_cookie.isFree() || test(PND_GVAR);
@@ -1484,7 +1486,7 @@ ParseNode::test(unsigned flag) const
 {
     JS_ASSERT(pn_defn || pn_arity == PN_FUNC || pn_arity == PN_NAME);
 #ifdef DEBUG
-    if ((flag & (PND_ASSIGNED | PND_FUNARG)) && pn_defn && !(pn_dflags & flag)) {
+    if ((flag & PND_ASSIGNED) && pn_defn && !(pn_dflags & flag)) {
         for (ParseNode *pn = ((Definition *) this)->dn_uses; pn; pn = pn->pn_link) {
             JS_ASSERT(!pn->pn_defn);
             JS_ASSERT(!(pn->pn_dflags & flag));
@@ -1494,23 +1496,27 @@ ParseNode::test(unsigned flag) const
     return !!(pn_dflags & flag);
 }
 
-inline void
-ParseNode::setFunArg()
+/*
+ * We store definition pointers in PN_NAMESET AtomDefnMapPtrs in the AST,
+ * but due to redefinition these nodes may become uses of other
+ * definitions.  This is unusual, so we simply chase the pn_lexdef link to
+ * find the final definition node. See functions called from
+ * js::frontend::AnalyzeFunctions.
+ *
+ * FIXME: MakeAssignment mutates for want of a parent link...
+ */
+inline Definition *
+ParseNode::resolve()
 {
-    /*
-     * pn_defn NAND pn_used must be true, per this chart:
-     *
-     *   pn_defn pn_used
-     *         0       0        anonymous function used implicitly, e.g. by
-     *                          hidden yield in a genexp
-     *         0       1        a use of a definition or placeholder
-     *         1       0        a definition or placeholder
-     *         1       1        error: this case must not be possible
-     */
-    JS_ASSERT(!(pn_defn & pn_used));
-    if (pn_used)
-        pn_lexdef->pn_dflags |= PND_FUNARG;
-    pn_dflags |= PND_FUNARG;
+    ParseNode *pn = this;
+    while (!pn->isDefn()) {
+        if (pn->isAssignment()) {
+            pn = pn->pn_left;
+            continue;
+        }
+        pn = pn->lexdef();
+    }
+    return (Definition *) pn;
 }
 
 inline void
@@ -1554,8 +1560,6 @@ struct FunctionBox : public ObjectBox
 
     JSFunction *function() const { return (JSFunction *) object; }
 
-    bool joinable() const;
-
     /*
      * True if this function is inside the scope of a with-statement, an E4X
      * filter-expression, or a function that uses direct eval.
