@@ -24,7 +24,7 @@ struct StringWrapper
 } sw;
 
 void
-FinalizeCallback(JSContext *cx, JSFinalizeStatus status)
+FinalizeCallback(JSFreeOp *fop, JSFinalizeStatus status, JSBool isCompartmentGC)
 {
     if (status == JSFINALIZE_START)
         sw.strOk = !JS_IsAboutToBeFinalized(sw.str);
@@ -36,7 +36,7 @@ BEGIN_TEST(testInternAcrossGC)
     sw.strOk = false;
     CHECK(sw.str);
     JS_SetFinalizeCallback(rt, FinalizeCallback);
-    JS_GC(cx);
+    JS_GC(rt);
     CHECK(sw.strOk);
     return true;
 }
