@@ -132,30 +132,34 @@ JS_NewObjectWithUniqueType(JSContext *cx, JSClass *clasp, JSObject *proto, JSObj
 }
 
 JS_FRIEND_API(void)
-js::GCForReason(JSContext *cx, gcreason::Reason reason)
+js::PrepareCompartmentForGC(JSCompartment *comp)
 {
-    GC(cx, NULL, GC_NORMAL, reason);
+    comp->scheduleGC();
 }
 
 JS_FRIEND_API(void)
-js::CompartmentGCForReason(JSContext *cx, JSCompartment *comp, gcreason::Reason reason)
+js::PrepareForFullGC(JSRuntime *rt)
 {
-    /* We cannot GC the atoms compartment alone; use a full GC instead. */
-    JS_ASSERT(comp != cx->runtime->atomsCompartment);
-
-    GC(cx, comp, GC_NORMAL, reason);
+    for (CompartmentsIter c(rt); !c.done(); c.next())
+        c->scheduleGC();
 }
 
 JS_FRIEND_API(void)
-js::ShrinkingGC(JSContext *cx, gcreason::Reason reason)
+js::GCForReason(JSRuntime *rt, gcreason::Reason reason)
 {
-    GC(cx, NULL, GC_SHRINK, reason);
+    GC(rt, GC_NORMAL, reason);
 }
 
 JS_FRIEND_API(void)
-js::IncrementalGC(JSContext *cx, gcreason::Reason reason)
+js::ShrinkingGC(JSRuntime *rt, gcreason::Reason reason)
 {
-    GCSlice(cx, NULL, GC_NORMAL, reason);
+    GC(rt, GC_SHRINK, reason);
+}
+
+JS_FRIEND_API(void)
+js::IncrementalGC(JSRuntime *rt, gcreason::Reason reason)
+{
+    GCSlice(rt, GC_NORMAL, reason);
 }
 
 JS_FRIEND_API(void)
@@ -650,26 +654,6 @@ GetContextOutstandingRequests(const JSContext *cx)
 {
     return cx->outstandingRequests;
 }
-
-AutoSkipConservativeScan::AutoSkipConservativeScan(JSContext *cx
-                                                   MOZ_GUARD_OBJECT_NOTIFIER_PARAM_IN_IMPL)
-  : context(cx)
-{
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
-
-    JSRuntime *rt = context->runtime;
-    JS_ASSERT(rt->requestDepth >= 1);
-    JS_ASSERT(!rt->conservativeGC.requestThreshold);
-    if (rt->requestDepth == 1)
-        rt->conservativeGC.requestThreshold = 1;
-}
-
-AutoSkipConservativeScan::~AutoSkipConservativeScan()
-{
-    JSRuntime *rt = context->runtime;
-    if (rt->requestDepth == 1)
-        rt->conservativeGC.requestThreshold = 0;
-}
 #endif
 
 JS_FRIEND_API(JSCompartment *)
@@ -717,35 +701,39 @@ SetGCSliceCallback(JSRuntime *rt, GCSliceCallback callback)
     return old;
 }
 
-JS_FRIEND_API(bool)
-WantGCSlice(JSRuntime *rt)
+jschar *
+GCDescription::formatMessage(JSRuntime *rt) const
 {
-    if (rt->gcZeal() == gc::ZealFrameVerifierValue || rt->gcZeal() == gc::ZealFrameGCValue)
-        return true;
+    return rt->gcStats.formatMessage();
+}
 
-    if (rt->gcIncrementalState != gc::NO_INCREMENTAL)
-        return true;
-
-    return false;
+jschar *
+GCDescription::formatJSON(JSRuntime *rt, uint64_t timestamp) const
+{
+    return rt->gcStats.formatJSON(timestamp);
 }
 
 JS_FRIEND_API(void)
-NotifyDidPaint(JSContext *cx)
+NotifyDidPaint(JSRuntime *rt)
 {
-    JSRuntime *rt = cx->runtime;
-
     if (rt->gcZeal() == gc::ZealFrameVerifierValue) {
-        gc::VerifyBarriers(cx);
+        gc::VerifyBarriers(rt);
         return;
     }
 
     if (rt->gcZeal() == gc::ZealFrameGCValue) {
-        GCSlice(cx, NULL, GC_NORMAL, gcreason::REFRESH_FRAME);
+        PrepareForFullGC(rt);
+        GCSlice(rt, GC_NORMAL, gcreason::REFRESH_FRAME);
         return;
     }
 
-    if (rt->gcIncrementalState != gc::NO_INCREMENTAL && !rt->gcInterFrameGC)
-        GCSlice(cx, rt->gcIncrementalCompartment, GC_NORMAL, gcreason::REFRESH_FRAME);
+    if (rt->gcIncrementalState != gc::NO_INCREMENTAL && !rt->gcInterFrameGC) {
+        for (CompartmentsIter c(rt); !c.done(); c.next()) {
+            if (c->needsBarrier())
+                PrepareCompartmentForGC(c);
+        }
+        GCSlice(rt, GC_NORMAL, gcreason::REFRESH_FRAME);
+    }
 
     rt->gcInterFrameGC = false;
 }
