@@ -40,6 +40,9 @@
 /*
  * JS math package.
  */
+
+#include "mozilla/FloatingPoint.h"
+
 #include <stdlib.h>
 #include "jstypes.h"
 #include "prmjtime.h"
@@ -99,8 +102,8 @@ MathCache::MathCache() {
     memset(table, 0, sizeof(table));
 
     /* See comments in lookup(). */
-    JS_ASSERT(JSDOUBLE_IS_NEGZERO(-0.0));
-    JS_ASSERT(!JSDOUBLE_IS_NEGZERO(+0.0));
+    JS_ASSERT(MOZ_DOUBLE_IS_NEGATIVE_ZERO(-0.0));
+    JS_ASSERT(!MOZ_DOUBLE_IS_NEGATIVE_ZERO(+0.0));
     JS_ASSERT(hash(-0.0) != hash(+0.0));
 }
 
@@ -212,7 +215,7 @@ math_atan2_kernel(double x, double y)
      * - The sign of x determines the sign of the result.
      * - The sign of y determines the multiplicator, 1 or 3.
      */
-    if (JSDOUBLE_IS_INFINITE(x) && JSDOUBLE_IS_INFINITE(y)) {
+    if (MOZ_DOUBLE_IS_INFINITE(x) && MOZ_DOUBLE_IS_INFINITE(y)) {
         double z = js_copysign(M_PI / 4, x);
         if (y < 0)
             z *= 3;
@@ -222,7 +225,7 @@ math_atan2_kernel(double x, double y)
 
 #if defined(SOLARIS) && defined(__GNUC__)
     if (x == 0) {
-        if (JSDOUBLE_IS_NEGZERO(y))
+        if (MOZ_DOUBLE_IS_NEGZERO(y))
             return js_copysign(M_PI, x);
         if (y == 0)
             return x;
@@ -296,7 +299,7 @@ static double
 math_exp_body(double d)
 {
 #ifdef _WIN32
-    if (!JSDOUBLE_IS_NaN(d)) {
+    if (!MOZ_DOUBLE_IS_NaN(d)) {
         if (d == js_PositiveInfinity)
             return js_PositiveInfinity;
         if (d == js_NegativeInfinity)
@@ -387,7 +390,7 @@ js_math_max(JSContext *cx, unsigned argc, Value *vp)
     for (i = 0; i < argc; i++) {
         if (!ToNumber(cx, argv[i], &x))
             return JS_FALSE;
-        if (JSDOUBLE_IS_NaN(x)) {
+        if (MOZ_DOUBLE_IS_NaN(x)) {
             vp->setDouble(js_NaN);
             return JS_TRUE;
         }
@@ -417,7 +420,7 @@ js_math_min(JSContext *cx, unsigned argc, Value *vp)
     for (i = 0; i < argc; i++) {
         if (!ToNumber(cx, argv[i], &x))
             return JS_FALSE;
-        if (JSDOUBLE_IS_NaN(x)) {
+        if (MOZ_DOUBLE_IS_NaN(x)) {
             vp->setDouble(js_NaN);
             return JS_TRUE;
         }
@@ -449,7 +452,7 @@ powi(double x, int y)
                 // given us a finite p. This happens very rarely.
                 
                 double result = 1.0 / p;
-                return (result == 0 && JSDOUBLE_IS_INFINITE(p))
+                return (result == 0 && MOZ_DOUBLE_IS_INFINITE(p))
                        ? pow(x, static_cast<double>(y))  // Avoid pow(double, int).
                        : result;
             }
@@ -475,7 +478,7 @@ js_math_pow(JSContext *cx, unsigned argc, Value *vp)
      * Special case for square roots. Note that pow(x, 0.5) != sqrt(x)
      * when x = -0.0, so we have to guard for this.
      */
-    if (JSDOUBLE_IS_FINITE(x) && x != 0.0) {
+    if (MOZ_DOUBLE_IS_FINITE(x) && x != 0.0) {
         if (y == 0.5) {
             vp->setNumber(sqrt(x));
             return JS_TRUE;
@@ -489,7 +492,7 @@ js_math_pow(JSContext *cx, unsigned argc, Value *vp)
      * Because C99 and ECMA specify different behavior for pow(),
      * we need to wrap the libm call to make it ECMA compliant.
      */
-    if (!JSDOUBLE_IS_FINITE(y) && (x == 1.0 || x == -1.0)) {
+    if (!MOZ_DOUBLE_IS_FINITE(y) && (x == 1.0 || x == -1.0)) {
         vp->setDouble(js_NaN);
         return JS_TRUE;
     }
@@ -559,22 +562,6 @@ math_random(JSContext *cx, unsigned argc, Value *vp)
     return JS_TRUE;
 }
 
-#if defined _WIN32 && _MSC_VER < 1400
-/* Try to work around apparent _copysign bustage in VC7.x. */
-double
-js_copysign(double x, double y)
-{
-    jsdpun xu, yu;
-
-    xu.d = x;
-    yu.d = y;
-    xu.s.hi &= ~JSDOUBLE_HI32_SIGNBIT;
-    xu.s.hi |= yu.s.hi & JSDOUBLE_HI32_SIGNBIT;
-    return xu.d;
-}
-#endif
-
-
 JSBool /* ES5 15.8.2.15. */
 js_math_round(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -590,17 +577,13 @@ js_math_round(JSContext *cx, unsigned argc, Value *vp)
         return false;
 
     int32_t i;
-    if (JSDOUBLE_IS_INT32(x, &i)) { 
+    if (MOZ_DOUBLE_IS_INT32(x, &i)) {
         args.rval().setInt32(i);
         return true;
     }
 
-    jsdpun u;
-    u.d = x;
-
     /* Some numbers are so big that adding 0.5 would give the wrong number */
-    int exponent = ((u.s.hi & JSDOUBLE_HI32_EXPMASK) >> JSDOUBLE_HI32_EXPSHIFT) - JSDOUBLE_EXPBIAS;
-    if (exponent >= 52) {
+    if (MOZ_DOUBLE_EXPONENT(x) >= 52) {
         args.rval().setNumber(x);
         return true;
     }
@@ -711,9 +694,11 @@ js_IsMathFunction(Native native)
 }
 
 JSObject *
-js_InitMathClass(JSContext *cx, JSObject *obj)
+js_InitMathClass(JSContext *cx, JSObject *obj_)
 {
-    JSObject *Math = NewObjectWithClassProto(cx, &MathClass, NULL, obj);
+    RootedVarObject obj(cx, obj_);
+
+    RootedVarObject Math(cx, NewObjectWithClassProto(cx, &MathClass, NULL, obj));
     if (!Math || !Math->setSingletonType(cx))
         return NULL;
 
