@@ -93,13 +93,13 @@ typedef enum JSOp {
 #define JOF_INT8          18      /* int8_t immediate operand */
 #define JOF_ATOMOBJECT    19      /* uint16_t constant index + object index */
 #define JOF_UINT16PAIR    20      /* pair of uint16_t immediates */
+#define JOF_SCOPECOORD    21      /* pair of uint16_t immediates followed by atom index */
 #define JOF_TYPEMASK      0x001f  /* mask for above immediate types */
 
 #define JOF_NAME          (1U<<5) /* name operation */
 #define JOF_PROP          (2U<<5) /* obj.prop operation */
 #define JOF_ELEM          (3U<<5) /* obj[index] operation */
 #define JOF_XMLNAME       (4U<<5) /* XML name: *, a::b, @a, @a::b, etc. */
-#define JOF_VARPROP       (5U<<5) /* x.prop for this, arg, var, or local x */
 #define JOF_MODEMASK      (7U<<5) /* mask for above addressing modes */
 #define JOF_SET           (1U<<8) /* set (i.e., assignment) operation */
 #define JOF_DEL           (1U<<9) /* delete operation */
@@ -521,10 +521,10 @@ FlowsIntoNext(JSOp op)
 
 /*
  * Counts accumulated for a single opcode in a script. The counts tracked vary
- * between opcodes, and this structure ensures that counts are accessed in
- * a coherent fashion.
+ * between opcodes, and this structure ensures that counts are accessed in a
+ * coherent fashion.
  */
-class OpcodeCounts
+class PCCounts
 {
     friend struct ::JSScript;
     double *counts;
@@ -542,11 +542,11 @@ class OpcodeCounts
         BASE_METHODJIT_CODE,
         BASE_METHODJIT_PICS,
 
-        BASE_COUNT
+        BASE_LIMIT
     };
 
     enum AccessCounts {
-        ACCESS_MONOMORPHIC = BASE_COUNT,
+        ACCESS_MONOMORPHIC = BASE_LIMIT,
         ACCESS_DIMORPHIC,
         ACCESS_POLYMORPHIC,
 
@@ -561,7 +561,7 @@ class OpcodeCounts
         ACCESS_STRING,
         ACCESS_OBJECT,
 
-        ACCESS_COUNT
+        ACCESS_LIMIT
     };
 
     static bool accessOp(JSOp op) {
@@ -569,7 +569,7 @@ class OpcodeCounts
          * Access ops include all name, element and property reads, as well as
          * SETELEM and SETPROP (for ElementCounts/PropertyCounts alignment).
          */
-        if (op == JSOP_SETELEM || op == JSOP_SETPROP || op == JSOP_SETMETHOD)
+        if (op == JSOP_SETELEM || op == JSOP_SETPROP)
             return true;
         int format = js_CodeSpec[op].format;
         return !!(format & (JOF_NAME | JOF_GNAME | JOF_ELEM | JOF_PROP))
@@ -577,7 +577,7 @@ class OpcodeCounts
     }
 
     enum ElementCounts {
-        ELEM_ID_INT = ACCESS_COUNT,
+        ELEM_ID_INT = ACCESS_LIMIT,
         ELEM_ID_DOUBLE,
         ELEM_ID_OTHER,
         ELEM_ID_UNKNOWN,
@@ -587,7 +587,7 @@ class OpcodeCounts
         ELEM_OBJECT_DENSE,
         ELEM_OBJECT_OTHER,
 
-        ELEM_COUNT
+        ELEM_LIMIT
     };
 
     static bool elementOp(JSOp op) {
@@ -595,11 +595,11 @@ class OpcodeCounts
     }
 
     enum PropertyCounts {
-        PROP_STATIC = ACCESS_COUNT,
+        PROP_STATIC = ACCESS_LIMIT,
         PROP_DEFINITE,
         PROP_OTHER,
 
-        PROP_COUNT
+        PROP_LIMIT
     };
 
     static bool propertyOp(JSOp op) {
@@ -607,12 +607,12 @@ class OpcodeCounts
     }
 
     enum ArithCounts {
-        ARITH_INT = BASE_COUNT,
+        ARITH_INT = BASE_LIMIT,
         ARITH_DOUBLE,
         ARITH_OTHER,
         ARITH_UNKNOWN,
 
-        ARITH_COUNT
+        ARITH_LIMIT
     };
 
     static bool arithOp(JSOp op) {
@@ -623,14 +623,14 @@ class OpcodeCounts
     {
         if (accessOp(op)) {
             if (elementOp(op))
-                return ELEM_COUNT;
+                return ELEM_LIMIT;
             if (propertyOp(op))
-                return PROP_COUNT;
-            return ACCESS_COUNT;
+                return PROP_LIMIT;
+            return ACCESS_LIMIT;
         }
         if (arithOp(op))
-            return ARITH_COUNT;
-        return BASE_COUNT;
+            return ARITH_LIMIT;
+        return BASE_LIMIT;
     }
 
     static const char *countName(JSOp op, size_t which);
