@@ -179,7 +179,9 @@ class NodeBuilder
         : cx(c), saveLoc(l), src(s) {
     }
 
-    bool init(JSObject *userobj = NULL) {
+    bool init(JSObject *userobj_ = NULL) {
+        RootedVarObject userobj(cx, userobj_);
+
         if (src) {
             if (!atomValue(src, &srcval))
                 return false;
@@ -202,7 +204,10 @@ class NodeBuilder
 
             const char *name = callbackNames[i];
             JSAtom *atom = js_Atomize(cx, name, strlen(name));
-            if (!atom || !GetPropertyDefault(cx, userobj, ATOM_TO_JSID(atom), NullValue(), &funv))
+            if (!atom)
+                return false;
+            RootedVarId id(cx, ATOM_TO_JSID(atom));
+            if (!GetPropertyDefault(cx, userobj, id, NullValue(), &funv))
                 return false;
 
             if (funv.isNullOrUndefined()) {
@@ -3123,24 +3128,22 @@ reflect_parse(JSContext *cx, uint32_t argc, jsval *vp)
             return JS_FALSE;
         }
 
-        JSObject *config = &arg.toObject();
+        RootedVarObject config(cx, &arg.toObject());
 
         Value prop;
 
         /* config.loc */
-        if (!GetPropertyDefault(cx, config, ATOM_TO_JSID(cx->runtime->atomState.locAtom),
-                                BooleanValue(true), &prop)) {
+        RootedVarId locId(cx, ATOM_TO_JSID(cx->runtime->atomState.locAtom));
+        if (!GetPropertyDefault(cx, config, locId, BooleanValue(true), &prop))
             return JS_FALSE;
-        }
 
         loc = js_ValueToBoolean(prop);
 
         if (loc) {
             /* config.source */
-            if (!GetPropertyDefault(cx, config, ATOM_TO_JSID(cx->runtime->atomState.sourceAtom),
-                                    NullValue(), &prop)) {
+            RootedVarId sourceId(cx, ATOM_TO_JSID(cx->runtime->atomState.sourceAtom));
+            if (!GetPropertyDefault(cx, config, sourceId, NullValue(), &prop))
                 return JS_FALSE;
-            }
 
             if (!prop.isNullOrUndefined()) {
                 JSString *str = ToString(cx, prop);
@@ -3159,18 +3162,17 @@ reflect_parse(JSContext *cx, uint32_t argc, jsval *vp)
             }
 
             /* config.line */
-            if (!GetPropertyDefault(cx, config, ATOM_TO_JSID(cx->runtime->atomState.lineAtom),
-                                    Int32Value(1), &prop) ||
+            RootedVarId lineId(cx, ATOM_TO_JSID(cx->runtime->atomState.lineAtom));
+            if (!GetPropertyDefault(cx, config, lineId, Int32Value(1), &prop) ||
                 !ToUint32(cx, prop, &lineno)) {
                 return JS_FALSE;
             }
         }
 
         /* config.builder */
-        if (!GetPropertyDefault(cx, config, ATOM_TO_JSID(cx->runtime->atomState.builderAtom),
-                                NullValue(), &prop)) {
+        RootedVarId builderId(cx, ATOM_TO_JSID(cx->runtime->atomState.builderAtom));
+        if (!GetPropertyDefault(cx, config, builderId, NullValue(), &prop))
             return JS_FALSE;
-        }
 
         if (!prop.isNullOrUndefined()) {
             if (!prop.isObject()) {
