@@ -46,7 +46,6 @@
 #include <string.h>
 #include "jstypes.h"
 #include "jsclist.h"
-#include "jsdhash.h"
 #include "jsutil.h"
 #include "jsapi.h"
 #include "jsatom.h"
@@ -89,7 +88,7 @@ PropertyTable::init(JSRuntime *rt, Shape *lastProp)
     if (!entries)
         return false;
 
-    hashShift = JS_DHASH_BITS - sizeLog2;
+    hashShift = HASH_BITS - sizeLog2;
     for (Shape::Range r = lastProp->all(); !r.empty(); r.popFront()) {
         const Shape &shape = r.front();
         Shape **spp = search(shape.propid(), true);
@@ -108,6 +107,7 @@ bool
 Shape::makeOwnBaseShape(JSContext *cx)
 {
     JS_ASSERT(!base()->isOwned());
+    assertSameCompartment(cx, compartment());
 
     RootedVarShape self(cx, this);
 
@@ -201,7 +201,7 @@ PropertyTable::search(jsid id, bool adding)
         return spp;
 
     /* Collision: double hash. */
-    sizeLog2 = JS_DHASH_BITS - hashShift;
+    sizeLog2 = HASH_BITS - hashShift;
     hash2 = HASH2(hash0, sizeLog2, hashShift);
     sizeMask = JS_BITMASK(sizeLog2);
 
@@ -260,7 +260,7 @@ PropertyTable::change(int log2Delta, JSContext *cx)
     /*
      * Grow, shrink, or compress by changing this->entries.
      */
-    int oldlog2 = JS_DHASH_BITS - hashShift;
+    int oldlog2 = HASH_BITS - hashShift;
     int newlog2 = oldlog2 + log2Delta;
     uint32_t oldsize = JS_BIT(oldlog2);
     uint32_t newsize = JS_BIT(newlog2);
@@ -269,7 +269,7 @@ PropertyTable::change(int log2Delta, JSContext *cx)
         return false;
 
     /* Now that we have newTable allocated, update members. */
-    hashShift = JS_DHASH_BITS - newlog2;
+    hashShift = HASH_BITS - newlog2;
     removedCount = 0;
     Shape **oldTable = entries;
     entries = newTable;
@@ -312,7 +312,7 @@ Shape::getChildBinding(JSContext *cx, const StackShape &child)
 
     Shape *shape = JS_PROPERTY_TREE(cx).getChild(cx, this, numFixedSlots(), child);
     if (shape) {
-        JS_ASSERT(shape->parent == this);
+        //JS_ASSERT(shape->parent == this); // XXX 'this' is not rooted here
 
         /*
          * Update the number of fixed slots which bindings of this shape will
@@ -497,91 +497,14 @@ NormalizeGetterAndSetter(JSContext *cx, JSObject *obj,
         JS_ASSERT(!(attrs & JSPROP_SETTER));
         setter = NULL;
     }
-    if (flags & Shape::METHOD) {
-        JS_ASSERT_IF(getter, getter == JS_PropertyStub);
-        JS_ASSERT(!setter);
-        JS_ASSERT(!(attrs & (JSPROP_GETTER | JSPROP_SETTER)));
+    if (getter == JS_PropertyStub) {
+        JS_ASSERT(!(attrs & JSPROP_GETTER));
         getter = NULL;
-    } else {
-        if (getter == JS_PropertyStub) {
-            JS_ASSERT(!(attrs & JSPROP_GETTER));
-            getter = NULL;
-        }
     }
 
     return true;
 }
 
-#ifdef DEBUG
-# define CHECK_SHAPE_CONSISTENCY(obj) obj->checkShapeConsistency()
-
-void
-JSObject::checkShapeConsistency()
-{
-    static int throttle = -1;
-    if (throttle < 0) {
-        if (const char *var = getenv("JS_CHECK_SHAPE_THROTTLE"))
-            throttle = atoi(var);
-        if (throttle < 0)
-            throttle = 0;
-    }
-    if (throttle == 0)
-        return;
-
-    JS_ASSERT(isNative());
-
-    Shape *shape = lastProperty();
-    Shape *prev = NULL;
-
-    if (inDictionaryMode()) {
-        JS_ASSERT(shape->hasTable());
-
-        PropertyTable &table = shape->table();
-        for (uint32_t fslot = table.freelist; fslot != SHAPE_INVALID_SLOT;
-             fslot = getSlot(fslot).toPrivateUint32()) {
-            JS_ASSERT(fslot < slotSpan());
-        }
-
-        for (int n = throttle; --n >= 0 && shape->parent; shape = shape->parent) {
-            JS_ASSERT_IF(shape != lastProperty(), !shape->hasTable());
-
-            Shape **spp = table.search(shape->propid(), false);
-            JS_ASSERT(SHAPE_FETCH(spp) == shape);
-        }
-
-        shape = lastProperty();
-        for (int n = throttle; --n >= 0 && shape; shape = shape->parent) {
-            JS_ASSERT_IF(shape->slot() != SHAPE_INVALID_SLOT, shape->slot() < slotSpan());
-            if (!prev) {
-                JS_ASSERT(shape == lastProperty());
-                JS_ASSERT(shape->listp == &shape_);
-            } else {
-                JS_ASSERT(shape->listp == &prev->parent);
-            }
-            prev = shape;
-        }
-    } else {
-        for (int n = throttle; --n >= 0 && shape->parent; shape = shape->parent) {
-            if (shape->hasTable()) {
-                PropertyTable &table = shape->table();
-                JS_ASSERT(shape->parent);
-                for (Shape::Range r(shape); !r.empty(); r.popFront()) {
-                    Shape **spp = table.search(r.front().propid(), false);
-                    JS_ASSERT(SHAPE_FETCH(spp) == &r.front());
-                }
-            }
-            if (prev) {
-                JS_ASSERT(prev->maybeSlot() >= shape->maybeSlot());
-                shape->kids.checkConsistency(prev);
-            }
-            prev = shape;
-        }
-    }
-}
-#else
-# define CHECK_SHAPE_CONSISTENCY(obj) ((void)0)
-#endif
-
 Shape *
 JSObject::addProperty(JSContext *cx, jsid id,
                       PropertyOp getter, StrictPropertyOp setter,
@@ -597,12 +520,14 @@ JSObject::addProperty(JSContext *cx, jsid id,
 
     NormalizeGetterAndSetter(cx, this, id, attrs, flags, getter, setter);
 
+    RootedVarObject self(cx, this);
+
     Shape **spp = NULL;
     if (inDictionaryMode())
         spp = lastProperty()->table().search(id, true);
 
-    return addPropertyInternal(cx, id, getter, setter, slot, attrs, flags, shortid,
-                               spp, allowDictionary);
+    return self->addPropertyInternal(cx, id, getter, setter, slot, attrs, flags, shortid,
+                                     spp, allowDictionary);
 }
 
 Shape *
@@ -628,7 +553,7 @@ JSObject::addPropertyInternal(JSContext *cx, jsid id,
             (!stableSlot || lastProperty()->entryCount() >= PropertyTree::MAX_HEIGHT)) {
             if (!toDictionaryMode(cx))
                 return NULL;
-            table = &lastProperty()->table();
+            table = &self->lastProperty()->table();
             spp = table->search(id, true);
         }
     } else {
@@ -680,11 +605,11 @@ JSObject::addPropertyInternal(JSContext *cx, jsid id,
             shape->parent->handoffTableTo(shape);
         }
 
-        CHECK_SHAPE_CONSISTENCY(self);
+        self->checkShapeConsistency();
         return shape;
     }
 
-    CHECK_SHAPE_CONSISTENCY(self);
+    self->checkShapeConsistency();
     return NULL;
 }
 
@@ -841,7 +766,7 @@ JSObject::putProperty(JSContext *cx, jsid id,
         Shape *newShape = self->getChildProperty(cx, shape->parent, child);
 
         if (!newShape) {
-            CHECK_SHAPE_CONSISTENCY(self);
+            self->checkShapeConsistency();
             return NULL;
         }
 
@@ -860,7 +785,7 @@ JSObject::putProperty(JSContext *cx, jsid id,
         JS_ATOMIC_INCREMENT(&cx->runtime->propertyRemovals);
     }
 
-    CHECK_SHAPE_CONSISTENCY(self);
+    self->checkShapeConsistency();
 
     return shape;
 }
@@ -877,9 +802,6 @@ JSObject::changeProperty(JSContext *cx, Shape *shape, unsigned attrs, unsigned m
     JS_ASSERT(!((attrs ^ shape->attrs) & JSPROP_SHARED) ||
               !(attrs & JSPROP_SHARED));
 
-    /* Don't allow method properties to be changed to have a getter or setter. */
-    JS_ASSERT_IF(shape->isMethod(), !getter && !setter);
-
     types::MarkTypePropertyConfigured(cx, this, shape->propid());
     if (attrs & (JSPROP_GETTER | JSPROP_SETTER))
         types::AddTypePropertyId(cx, this, shape->propid(), types::Type::UnknownType());
@@ -904,7 +826,7 @@ JSObject::changeProperty(JSContext *cx, Shape *shape, unsigned attrs, unsigned m
     Shape *newShape = putProperty(cx, shape->propid(), getter, setter, shape->maybeSlot(),
                                   attrs, shape->flags, shape->maybeShortid());
 
-    CHECK_SHAPE_CONSISTENCY(this);
+    checkShapeConsistency();
     return newShape;
 }
 
@@ -1021,7 +943,7 @@ JSObject::removeProperty(JSContext *cx, jsid id)
         self->removeLastProperty(cx);
     }
 
-    CHECK_SHAPE_CONSISTENCY(self);
+    self->checkShapeConsistency();
     return true;
 }
 
@@ -1043,7 +965,7 @@ JSObject::clear(JSContext *cx)
     JS_ALWAYS_TRUE(setLastProperty(cx, shape));
 
     JS_ATOMIC_INCREMENT(&cx->runtime->propertyRemovals);
-    CHECK_SHAPE_CONSISTENCY(this);
+    checkShapeConsistency();
 }
 
 void
@@ -1075,7 +997,7 @@ JSObject::replaceWithNewEquivalentShape(JSContext *cx, Shape *oldShape, Shape *n
         RootShape newRoot(cx, &newShape);
         if (!toDictionaryMode(cx))
             return NULL;
-        oldShape = lastProperty();
+        oldShape = self->lastProperty();
     }
 
     if (!newShape) {
@@ -1102,7 +1024,7 @@ JSObject::replaceWithNewEquivalentShape(JSContext *cx, Shape *oldShape, Shape *n
     JS_ASSERT(newShape->parent == oldShape);
     oldShape->removeFromDictionary(self);
 
-    if (newShape == lastProperty())
+    if (newShape == self->lastProperty())
         oldShape->handoffTableTo(newShape);
 
     if (spp)
@@ -1110,44 +1032,6 @@ JSObject::replaceWithNewEquivalentShape(JSContext *cx, Shape *oldShape, Shape *n
     return newShape;
 }
 
-Shape *
-JSObject::methodShapeChange(JSContext *cx, const Shape &shape)
-{
-    JS_ASSERT(shape.isMethod());
-
-    if (!inDictionaryMode() && !toDictionaryMode(cx))
-        return NULL;
-
-    Shape *spare = js_NewGCShape(cx);
-    if (!spare)
-        return NULL;
-    new (spare) Shape(shape.base()->unowned(), 0);
-
-#ifdef DEBUG
-    JS_ASSERT(canHaveMethodBarrier());
-    JS_ASSERT(!shape.setter());
-    JS_ASSERT(!shape.hasShortID());
-#endif
-
-    /*
-     * Clear Shape::METHOD from flags as we are despecializing from a
-     * method memoized in the property tree to a plain old function-valued
-     * property.
-     */
-    Shape *result =
-        putProperty(cx, shape.propid(), NULL, NULL, shape.slot(),
-                    shape.attrs,
-                    shape.getFlags() & ~Shape::METHOD,
-                    0);
-    if (!result)
-        return NULL;
-
-    if (result != lastProperty())
-        JS_ALWAYS_TRUE(generateOwnShape(cx, spare));
-
-    return result;
-}
-
 bool
 JSObject::shadowingShapeChange(JSContext *cx, const Shape &shape)
 {
@@ -1268,7 +1152,7 @@ Shape::setObjectFlag(JSContext *cx, BaseShape::Flag flag, JSObject *proto, Shape
 /* static */ inline HashNumber
 StackBaseShape::hash(const StackBaseShape *base)
 {
-    JSDHashNumber hash = base->flags;
+    HashNumber hash = base->flags;
     hash = JS_ROTATE_LEFT32(hash, 4) ^ (uintptr_t(base->clasp) >> 3);
     hash = JS_ROTATE_LEFT32(hash, 4) ^ (uintptr_t(base->parent) >> 3);
     hash = JS_ROTATE_LEFT32(hash, 4) ^ uintptr_t(base->rawGetter);
@@ -1333,7 +1217,7 @@ BaseShape::getUnowned(JSContext *cx, const StackBaseShape &base)
 }
 
 void
-JSCompartment::sweepBaseShapeTable(JSContext *cx)
+JSCompartment::sweepBaseShapeTable()
 {
     if (baseShapes.initialized()) {
         for (BaseShapeSet::Enum e(baseShapes); !e.empty(); e.popFront()) {
@@ -1345,10 +1229,10 @@ JSCompartment::sweepBaseShapeTable(JSContext *cx)
 }
 
 void
-BaseShape::finalize(JSContext *cx, bool background)
+BaseShape::finalize(FreeOp *fop)
 {
     if (table_) {
-        cx->delete_(table_);
+        fop->delete_(table_);
         table_ = NULL;
     }
 }
@@ -1387,7 +1271,7 @@ Bindings::setParent(JSContext *cx, JSObject *obj)
      * mark the stack value as non-relocatable for the stack root analysis.
      */
     Bindings *self = this;
-    CheckRoot root(cx, &self);
+    SkipRoot root(cx, &self);
 
     RootObject rootObj(cx, &obj);
 
@@ -1405,7 +1289,7 @@ Bindings::setParent(JSContext *cx, JSObject *obj)
 /* static */ inline HashNumber
 InitialShapeEntry::hash(const Lookup &lookup)
 {
-    JSDHashNumber hash = uintptr_t(lookup.clasp) >> 3;
+    HashNumber hash = uintptr_t(lookup.clasp) >> 3;
     hash = JS_ROTATE_LEFT32(hash, 4) ^ (uintptr_t(lookup.proto) >> 3);
     hash = JS_ROTATE_LEFT32(hash, 4) ^ (uintptr_t(lookup.parent) >> 3);
     return hash + lookup.nfixed;
@@ -1438,6 +1322,9 @@ EmptyShape::getInitialShape(JSContext *cx, Class *clasp, JSObject *proto, JSObje
     if (p)
         return p->shape;
 
+    RootObject protoRoot(cx, &lookup.proto);
+    RootObject parentRoot(cx, &lookup.parent);
+
     RootedVar<UnownedBaseShape*> nbase(cx);
 
     StackBaseShape base(clasp, parent, objectFlags);
@@ -1452,7 +1339,7 @@ EmptyShape::getInitialShape(JSContext *cx, Class *clasp, JSObject *proto, JSObje
 
     InitialShapeEntry entry;
     entry.shape = shape;
-    entry.proto = proto;
+    entry.proto = lookup.proto;
 
     if (!table.relookupOrAdd(p, lookup, entry))
         return NULL;
@@ -1515,7 +1402,7 @@ EmptyShape::insertInitialShape(JSContext *cx, Shape *shape, JSObject *proto)
 }
 
 void
-JSCompartment::sweepInitialShapeTable(JSContext *cx)
+JSCompartment::sweepInitialShapeTable()
 {
     if (initialShapes.initialized()) {
         for (InitialShapeSet::Enum e(initialShapes); !e.empty(); e.popFront()) {
