@@ -63,61 +63,6 @@ typedef enum JSTryNoteKind {
     JSTRY_ITER
 } JSTryNoteKind;
 
-namespace js {
-
-/*
- * Indicates a location in the stack that an upvar value can be retrieved from
- * as a two tuple of (level, slot).
- *
- * Some existing client code uses the level value as a delta, or level "skip"
- * quantity. We could probably document that through use of more types at some
- * point in the future.
- *
- * Existing XDR code wants this to be backed by a 32b integer for serialization,
- * so we oblige.
- *
- * TODO: consider giving more bits to the slot value and takings ome from the level.
- */
-class UpvarCookie
-{
-    uint32_t value;
-
-    static const uint32_t FREE_VALUE = 0xfffffffful;
-
-    void checkInvariants() {
-        JS_STATIC_ASSERT(sizeof(UpvarCookie) == sizeof(uint32_t));
-        JS_STATIC_ASSERT(UPVAR_LEVEL_LIMIT < FREE_LEVEL);
-    }
-
-  public:
-    /*
-     * All levels above-and-including FREE_LEVEL are reserved so that
-     * FREE_VALUE can be used as a special value.
-     */
-    static const uint16_t FREE_LEVEL = 0x3fff;
-
-    /*
-     * If a function has a higher static level than this limit, we will not
-     * optimize it using UPVAR opcodes.
-     */
-    static const uint16_t UPVAR_LEVEL_LIMIT = 16;
-    static const uint16_t CALLEE_SLOT = 0xffff;
-    static bool isLevelReserved(uint16_t level) { return level >= FREE_LEVEL; }
-
-    bool isFree() const { return value == FREE_VALUE; }
-    uint32_t asInteger() const { return value; }
-    /* isFree check should be performed before using these accessors. */
-    uint16_t level() const { JS_ASSERT(!isFree()); return uint16_t(value >> 16); }
-    uint16_t slot() const { JS_ASSERT(!isFree()); return uint16_t(value); }
-
-    void set(const UpvarCookie &other) { set(other.level(), other.slot()); }
-    void set(uint16_t newLevel, uint16_t newSlot) { value = (uint32_t(newLevel) << 16) | newSlot; }
-    void makeFree() { set(0xffff, 0xffff); JS_ASSERT(isFree()); }
-    void fromInteger(uint32_t u32) { value = u32; }
-};
-
-}
-
 /*
  * Exception handling record.
  */
@@ -140,11 +85,6 @@ typedef struct JSObjectArray {
     uint32_t        length;     /* count of indexed objects */
 } JSObjectArray;
 
-typedef struct JSUpvarArray {
-    js::UpvarCookie *vector;    /* array of indexed upvar cookies */
-    uint32_t        length;     /* count of indexed upvar cookies */
-} JSUpvarArray;
-
 typedef struct JSConstArray {
     js::HeapValue   *vector;    /* array of indexed constant values */
     uint32_t        length;
@@ -161,21 +101,26 @@ struct GlobalSlotArray {
     uint32_t        length;
 };
 
+struct ClosedSlotArray {
+    uint32_t        *vector;    /* array of closed slots */
+    uint32_t        length;     /* count of closed slots */
+};
+
 struct Shape;
 
-enum BindingKind { NONE, ARGUMENT, VARIABLE, CONSTANT, UPVAR };
+enum BindingKind { NONE, ARGUMENT, VARIABLE, CONSTANT };
 
 /*
- * Formal parameters, local variables, and upvars are stored in a shape tree
+ * Formal parameters and local variables are stored in a shape tree
  * path encapsulated within this class.  This class represents bindings for
  * both function and top-level scripts (the latter is needed to track names in
  * strict mode eval code, to give such code its own lexical environment).
  */
-class Bindings {
+class Bindings
+{
     HeapPtr<Shape> lastBinding;
     uint16_t nargs;
     uint16_t nvars;
-    uint16_t nupvars;
     bool     hasDup_:1;     // true if there are duplicate argument names
 
     inline Shape *initialShape(JSContext *cx) const;
@@ -196,16 +141,20 @@ class Bindings {
      */
     inline void clone(JSContext *cx, Bindings *bindings);
 
-    uint16_t countArgs() const { return nargs; }
-    uint16_t countVars() const { return nvars; }
-    uint16_t countUpvars() const { return nupvars; }
+    uint16_t numArgs() const { return nargs; }
+    uint16_t numVars() const { return nvars; }
+    unsigned count() const { return nargs + nvars; }
 
-    unsigned countArgsAndVars() const { return nargs + nvars; }
-
-    unsigned countLocalNames() const { return nargs + nvars + nupvars; }
-
-    bool hasUpvars() const { return nupvars > 0; }
-    bool hasLocalNames() const { return countLocalNames() > 0; }
+    /*
+     * These functions map between argument/var indices [0, nargs/nvars) and
+     * and Bindings indices [0, nargs + nvars).
+     */
+    bool bindingIsArg(uint16_t i) const { return i < nargs; }
+    bool bindingIsLocal(uint16_t i) const { return i >= nargs; }
+    uint16_t argToBinding(uint16_t i) { JS_ASSERT(i < nargs); return i; }
+    uint16_t localToBinding(uint16_t i) { return i + nargs; }
+    uint16_t bindingToArg(uint16_t i) { JS_ASSERT(bindingIsArg(i)); return i; }
+    uint16_t bindingToLocal(uint16_t i) { JS_ASSERT(bindingIsLocal(i)); return i - nargs; }
 
     /* Ensure these bindings have a shape lineage. */
     inline bool ensureShape(JSContext *cx);
@@ -226,10 +175,7 @@ class Bindings {
     bool setParent(JSContext *cx, JSObject *obj);
 
     enum {
-        /*
-         * A script may have no more than this many arguments, variables, or
-         * upvars.
-         */
+        /* A script may have no more than this many arguments or variables. */
         BINDING_COUNT_LIMIT = 0xFFFF
     };
 
@@ -246,29 +192,25 @@ class Bindings {
      *
      * The parser builds shape paths for functions, usable by Call objects at
      * runtime, by calling an "add" method. All ARGUMENT bindings must be added
-     * before before any VARIABLE or CONSTANT bindings, which themselves must
-     * be added before all UPVAR bindings.
+     * before before any VARIABLE or CONSTANT bindings.
      */
-    bool add(JSContext *cx, JSAtom *name, BindingKind kind);
+    bool add(JSContext *cx, HandleAtom name, BindingKind kind);
 
     /* Convenience specializations. */
-    bool addVariable(JSContext *cx, JSAtom *name) {
+    bool addVariable(JSContext *cx, HandleAtom name) {
         return add(cx, name, VARIABLE);
     }
-    bool addConstant(JSContext *cx, JSAtom *name) {
+    bool addConstant(JSContext *cx, HandleAtom name) {
         return add(cx, name, CONSTANT);
     }
-    bool addUpvar(JSContext *cx, JSAtom *name) {
-        return add(cx, name, UPVAR);
-    }
-    bool addArgument(JSContext *cx, JSAtom *name, uint16_t *slotp) {
+    bool addArgument(JSContext *cx, HandleAtom name, uint16_t *slotp) {
         JS_ASSERT(name != NULL); /* not destructuring */
         *slotp = nargs;
         return add(cx, name, ARGUMENT);
     }
     bool addDestructuring(JSContext *cx, uint16_t *slotp) {
         *slotp = nargs;
-        return add(cx, NULL, ARGUMENT);
+        return add(cx, RootedVarAtom(cx), ARGUMENT);
     }
 
     void noteDup() { hasDup_ = true; }
@@ -289,7 +231,7 @@ class Bindings {
 
     /*
      * This method returns the local variable, argument, etc. names used by a
-     * script.  This function must be called only when hasLocalNames().
+     * script.  This function must be called only when count() > 0.
      *
      * The elements of the vector with index less than nargs correspond to the
      * the names of arguments. An index >= nargs addresses a var binding.
@@ -316,7 +258,6 @@ class Bindings {
      */
     const js::Shape *lastArgument() const;
     const js::Shape *lastVariable() const;
-    const js::Shape *lastUpvar() const;
 
     void trace(JSTracer *trc);
 
@@ -339,39 +280,50 @@ namespace JSC {
     class ExecutablePool;
 }
 
-#define JS_UNJITTABLE_SCRIPT (reinterpret_cast<void*>(1))
+namespace js {
+namespace mjit {
+    struct JITScript;
+    class CallCompiler;
+}
+}
 
-namespace js { namespace mjit { struct JITScript; } }
 #endif
 
 namespace js {
 
 namespace analyze { class ScriptAnalysis; }
 
-class ScriptOpcodeCounts
+class ScriptCounts
 {
     friend struct ::JSScript;
-    friend struct ScriptOpcodeCountsPair;
-    OpcodeCounts *counts;
+    friend struct ScriptAndCounts;
+    /*
+     * This points to a single block that holds an array of PCCounts followed
+     * by an array of doubles.  Each element in the PCCounts array has a
+     * pointer into the array of doubles.
+     */
+    PCCounts *pcCountsVector;
 
  public:
+    ScriptCounts() : pcCountsVector(NULL) { }
 
-    ScriptOpcodeCounts() : counts(NULL) {
-    }
+    inline void destroy(FreeOp *fop);
 
-    inline void destroy(JSContext *cx);
-
-    void steal(ScriptOpcodeCounts &other) {
-        *this = other;
-        js::PodZero(&other);
-    }
-
-    // Boolean conversion, for 'if (counters) ...'
-    operator void*() const {
-        return counts;
+    void set(js::ScriptCounts counts) {
+        pcCountsVector = counts.pcCountsVector;
     }
 };
 
+typedef HashMap<JSScript *,
+                ScriptCounts,
+                DefaultHasher<JSScript *>,
+                SystemAllocPolicy> ScriptCountsMap;
+
+typedef HashMap<JSScript *,
+                jschar *,
+                DefaultHasher<JSScript *>,
+                SystemAllocPolicy> SourceMapMap;
+
 class DebugScript
 {
     friend struct ::JSScript;
@@ -395,128 +347,89 @@ class DebugScript
     BreakpointSite  *breakpoints[1];
 };
 
+typedef HashMap<JSScript *,
+                DebugScript *,
+                DefaultHasher<JSScript *>,
+                SystemAllocPolicy> DebugScriptMap;
+
 } /* namespace js */
 
 static const uint32_t JS_SCRIPT_COOKIE = 0xc00cee;
 
-struct JSScript : public js::gc::Cell {
-    /*
-     * Two successively less primitive ways to make a new JSScript.  The first
-     * does *not* call a non-null cx->runtime->newScriptHook -- only the second,
-     * NewScriptFromEmitter, calls this optional debugger hook.
-     *
-     * The NewScript function can't know whether the script it creates belongs
-     * to a function, or is top-level or eval code, but the debugger wants access
-     * to the newly made script's function, if any -- so callers of NewScript
-     * are responsible for notifying the debugger after successfully creating any
-     * kind (function or other) of new JSScript.
-     */
-    static JSScript *NewScript(JSContext *cx, uint32_t length, uint32_t nsrcnotes, uint32_t natoms,
-                               uint32_t nobjects, uint32_t nupvars, uint32_t nregexps,
-                               uint32_t ntrynotes, uint32_t nconsts, uint32_t nglobals,
-                               uint16_t nClosedArgs, uint16_t nClosedVars, uint32_t nTypeSets,
-                               JSVersion version);
-
-    static JSScript *NewScriptFromEmitter(JSContext *cx, js::BytecodeEmitter *bce);
-
-#ifdef JS_CRASH_DIAGNOSTICS
-    /*
-     * Make sure that the cookie size does not affect the GC alignment
-     * requirements.
-     */
-    uint32_t        cookie1[Cell::CellSize / sizeof(uint32_t)];
-#endif
-    jsbytecode      *code;      /* bytecodes and their immediate operands */
-    uint8_t         *data;      /* pointer to variable-length data array */
-
-    uint32_t        length;     /* length of code vector */
+struct JSScript : public js::gc::Cell
+{
   private:
-    uint16_t        version;    /* JS version under which script was compiled */
+    static const uint32_t stepFlagMask = 0x80000000U;
+    static const uint32_t stepCountMask = 0x7fffffffU;
 
   public:
-    uint16_t        nfixed;     /* number of slots besides stack operands in
-                                   slot array */
-    /*
-     * Offsets to various array structures from the end of this script, or
-     * JSScript::INVALID_OFFSET if the array has length 0.
-     */
-    uint8_t         objectsOffset;  /* offset to the array of nested function,
-                                       block, scope, xml and one-time regexps
-                                       objects */
-    uint8_t         upvarsOffset;   /* offset of the array of display ("up")
-                                       closure vars */
-    uint8_t         regexpsOffset;  /* offset to the array of to-be-cloned
-                                       regexps  */
-    uint8_t         trynotesOffset; /* offset to the array of try notes */
-    uint8_t         globalsOffset;  /* offset to the array of global slots */
-    uint8_t         constOffset;    /* offset to the array of constants */
-
-    uint16_t        nTypeSets;      /* number of type sets used in this script for
-                                       dynamic type monitoring */
-
-    uint32_t        lineno;     /* base line number of script */
-
-    uint32_t        mainOffset; /* offset of main entry point from code, after
-                                   predef'ing prolog */
-    bool            noScriptRval:1; /* no need for result value of last
-                                       expression statement */
-    bool            savedCallerFun:1; /* can call getCallerFunction() */
-    bool            strictModeCode:1; /* code is in strict mode */
-    bool            compileAndGo:1;   /* script was compiled with TCF_COMPILE_N_GO */
-    bool            usesEval:1;       /* script uses eval() */
-    bool            usesArguments:1;  /* script uses arguments */
-    bool            warnedAboutTwoArgumentEval:1; /* have warned about use of
-                                                     obsolete eval(s, o) in
-                                                     this script */
-    bool            warnedAboutUndefinedProp:1; /* have warned about uses of
-                                                   undefined properties in this
-                                                   script */
-    bool            hasSingletons:1;  /* script has singleton objects */
-    bool            isOuterFunction:1; /* function is heavyweight, with inner functions */
-    bool            isInnerFunction:1; /* function is directly nested in a heavyweight
-                                        * outer function */
-    bool            isActiveEval:1;   /* script came from eval(), and is still active */
-    bool            isCachedEval:1;   /* script came from eval(), and is in eval cache */
-    bool            usedLazyArgs:1;   /* script has used lazy arguments at some point */
-    bool            createdArgs:1;    /* script has had arguments objects created */
-    bool            uninlineable:1;   /* script is considered uninlineable by analysis */
-    bool            reentrantOuterFunction:1; /* outer function marked reentrant */
-    bool            typesPurged:1;    /* TypeScript has been purged at some point */
 #ifdef JS_METHODJIT
-    bool            debugMode:1;      /* script was compiled in debug mode */
-    bool            failedBoundsCheck:1; /* script has had hoisted bounds checks fail */
-#endif
-    bool            callDestroyHook:1;/* need to call destroy hook */
+    // This type wraps JITScript.  It has three possible states.
+    // - "Empty": no compilation has been attempted and there is no JITScript.
+    // - "Unjittable": compilation failed and there is no JITScript.
+    // - "Valid": compilation succeeded and there is a JITScript.
+    class JITScriptHandle
+    {
+        // CallCompiler must be a friend because it generates code that uses
+        // UNJITTABLE.
+        friend class js::mjit::CallCompiler;
 
-    uint32_t        natoms;     /* length of atoms array */
-    uint16_t        nslots;     /* vars plus maximum stack depth */
-    uint16_t        staticLevel;/* static level for display maintenance */
+        // The exact representation:
+        // - NULL means "empty".
+        // - UNJITTABLE means "unjittable".
+        // - Any other value means "valid".
+        // UNJITTABLE = 1 so that we can check that a JITScript is valid
+        // with a single |> 1| test.  It's defined outside the class because
+        // non-integral static const fields can't be defined in the class.
+        static const js::mjit::JITScript *UNJITTABLE;   // = (JITScript *)1;
+        js::mjit::JITScript *value;
 
-    uint16_t        nClosedArgs; /* number of args which are closed over. */
-    uint16_t        nClosedVars; /* number of vars which are closed over. */
+      public:
+        JITScriptHandle()       { value = NULL; }
 
-    /*
-     * To ensure sizeof(JSScript) % gc::Cell::CellSize  == 0 on we must pad
-     * the script with 4 bytes. We use them to store tiny scripts like empty
-     * scripts.
-     */
-#if JS_BITS_PER_WORD == 64
-#define JS_SCRIPT_INLINE_DATA_LIMIT 4
-    uint8_t         inlineData[JS_SCRIPT_INLINE_DATA_LIMIT];
-#endif
+        bool isEmpty()          { return value == NULL; }
+        bool isUnjittable()     { return value == UNJITTABLE; }
+        bool isValid()          { return value  > UNJITTABLE; }
+
+        js::mjit::JITScript *getValid() {
+            JS_ASSERT(isValid());
+            return value;
+        }
+
+        void setEmpty()         { value = NULL; }
+        void setUnjittable()    { value = const_cast<js::mjit::JITScript *>(UNJITTABLE); }
+        void setValid(js::mjit::JITScript *jit) {
+            value = jit;
+            JS_ASSERT(isValid());
+        }
+
+        static void staticAsserts();
+    };
+#endif  // JS_METHODJIT
+
+    //
+    // We order fields according to their size in order to avoid wasting space
+    // for alignment.
+    //
+
+    // Larger-than-word-sized fields.
 
-    const char      *filename;  /* source filename or null */
-    JSAtom          **atoms;    /* maps immediate index to literal struct */
-  private:
-    size_t          useCount;  /* Number of times the script has been called
-                                 * or has had backedges taken. Reset if the
-                                 * script's JIT code is forcibly discarded. */
   public:
     js::Bindings    bindings;   /* names of top-level variables in this script
                                    (and arguments if this is a function script) */
+
+    // Word-sized fields.
+
+  public:
+    jsbytecode      *code;      /* bytecodes and their immediate operands */
+    uint8_t         *data;      /* pointer to variable-length data array (see 
+                                   comment above NewScript() for details) */
+
+    const char      *filename;  /* source filename or null */
+    js::HeapPtrAtom *atoms;     /* maps immediate index to literal struct */
+
     JSPrincipals    *principals;/* principals for this script */
     JSPrincipals    *originPrincipals; /* see jsapi.h 'originPrincipals' comment */
-    jschar          *sourceMap; /* source map file or null */
 
     /*
      * A global object for the script.
@@ -531,18 +444,170 @@ struct JSScript : public js::gc::Cell {
      */
     js::HeapPtr<js::GlobalObject, JSScript*> globalObject;
 
-    /* Hash table chaining for JSCompartment::evalCache. */
-    JSScript        *&evalHashLink() { return *globalObject.unsafeGetUnioned(); }
+    /* Persistent type information retained across GCs. */
+    js::types::TypeScript *types;
 
-    uint32_t        *closedSlots; /* vector of closed slots; args first, then vars. */
-
-    /* Execution and profiling information for JIT code in the script. */
-    js::ScriptOpcodeCounts pcCounters;
+  public:
+#ifdef JS_METHODJIT
+    JITScriptHandle jitHandleNormal; // extra JIT info for normal scripts
+    JITScriptHandle jitHandleCtor;   // extra JIT info for constructors
+#endif
 
   private:
-    js::DebugScript     *debug;
     js::HeapPtrFunction function_;
+
+    size_t          useCount;   /* Number of times the script has been called
+                                 * or has had backedges taken. Reset if the
+                                 * script's JIT code is forcibly discarded. */
+
+    // 32-bit fields.
+
   public:
+    uint32_t        length;     /* length of code vector */
+
+    uint32_t        lineno;     /* base line number of script */
+
+    uint32_t        mainOffset; /* offset of main entry point from code, after
+                                   predef'ing prolog */
+
+    uint32_t        natoms;     /* length of atoms array */
+
+#ifdef DEBUG
+    // Unique identifier within the compartment for this script, used for
+    // printing analysis information.
+    uint32_t        id_;
+  private:
+    uint32_t        idpad;
+#endif
+
+#if JS_BITS_PER_WORD == 32
+  private:
+    uint32_t        pad32;
+#endif
+
+    // 16-bit fields.
+
+  private:
+    uint16_t        version;    /* JS version under which script was compiled */
+
+  public:
+    uint16_t        nfixed;     /* number of slots besides stack operands in
+                                   slot array */
+
+    uint16_t        nTypeSets;  /* number of type sets used in this script for
+                                   dynamic type monitoring */
+
+    uint16_t        nslots;     /* vars plus maximum stack depth */
+    uint16_t        staticLevel;/* static level for display maintenance */
+
+  private:
+    uint16_t        argsSlot_;  /* slot holding 'arguments' (if argumentsHasLocalBindings) */
+
+    // 8-bit fields.
+
+  public:
+    // Offsets to various array structures from the end of this script, or
+    // JSScript::INVALID_OFFSET if the array has length 0.
+    uint8_t         constsOffset;   /* offset to the array of constants */
+    uint8_t         objectsOffset;  /* offset to the array of nested function,
+                                       block, scope, xml and one-time regexps
+                                       objects */
+    uint8_t         regexpsOffset;  /* offset to the array of to-be-cloned
+                                       regexps  */
+    uint8_t         trynotesOffset; /* offset to the array of try notes */
+    uint8_t         globalsOffset;  /* offset to the array of global slots */
+    uint8_t         closedArgsOffset; /* offset to the array of closed args */
+    uint8_t         closedVarsOffset; /* offset to the array of closed vars */
+
+    // 1-bit fields.
+
+  public:
+    bool            noScriptRval:1; /* no need for result value of last
+                                       expression statement */
+    bool            savedCallerFun:1; /* can call getCallerFunction() */
+    bool            strictModeCode:1; /* code is in strict mode */
+    bool            compileAndGo:1;   /* script was compiled with TCF_COMPILE_N_GO */
+    bool            bindingsAccessedDynamically:1; /* see TCF_BINDINGS_ACCESSED_DYNAMICALLY */
+    bool            warnedAboutTwoArgumentEval:1; /* have warned about use of
+                                                     obsolete eval(s, o) in
+                                                     this script */
+    bool            warnedAboutUndefinedProp:1; /* have warned about uses of
+                                                   undefined properties in this
+                                                   script */
+    bool            hasSingletons:1;  /* script has singleton objects */
+    bool            isOuterFunction:1; /* function is heavyweight, with inner functions */
+    bool            isInnerFunction:1; /* function is directly nested in a heavyweight
+                                        * outer function */
+    bool            isActiveEval:1;   /* script came from eval(), and is still active */
+    bool            isCachedEval:1;   /* script came from eval(), and is in eval cache */
+    bool            uninlineable:1;   /* script is considered uninlineable by analysis */
+    bool            reentrantOuterFunction:1; /* outer function marked reentrant */
+    bool            typesPurged:1;    /* TypeScript has been purged at some point */
+#ifdef JS_METHODJIT
+    bool            debugMode:1;      /* script was compiled in debug mode */
+    bool            failedBoundsCheck:1; /* script has had hoisted bounds checks fail */
+#endif
+    bool            callDestroyHook:1;/* need to call destroy hook */
+    bool            isGenerator:1;    /* is a generator */
+    bool            hasScriptCounts:1;/* script has an entry in
+                                         JSCompartment::scriptCountsMap */
+    bool            hasSourceMap:1;   /* script has an entry in
+                                         JSCompartment::sourceMapMap */
+    bool            hasDebugScript:1; /* script has an entry in
+                                         JSCompartment::debugScriptMap */
+
+  private:
+    /* See comments below. */
+    bool            argsHasLocalBinding_:1;
+    bool            needsArgsAnalysis_:1;
+    bool            needsArgsObj_:1;
+
+    //
+    // End of fields.  Start methods.
+    //
+
+    /*
+     * Two successively less primitive ways to make a new JSScript.  The first
+     * does *not* call a non-null cx->runtime->newScriptHook -- only the second,
+     * NewScriptFromEmitter, calls this optional debugger hook.
+     *
+     * The NewScript function can't know whether the script it creates belongs
+     * to a function, or is top-level or eval code, but the debugger wants access
+     * to the newly made script's function, if any -- so callers of NewScript
+     * are responsible for notifying the debugger after successfully creating any
+     * kind (function or other) of new JSScript.
+     */
+  public:
+    static JSScript *NewScript(JSContext *cx, uint32_t length, uint32_t nsrcnotes, uint32_t natoms,
+                               uint32_t nobjects, uint32_t nregexps,
+                               uint32_t ntrynotes, uint32_t nconsts, uint32_t nglobals,
+                               uint16_t nClosedArgs, uint16_t nClosedVars, uint32_t nTypeSets,
+                               JSVersion version);
+    static JSScript *NewScriptFromEmitter(JSContext *cx, js::BytecodeEmitter *bce);
+
+    /* See TCF_ARGUMENTS_HAS_LOCAL_BINDING comment. */
+    bool argumentsHasLocalBinding() const { return argsHasLocalBinding_; }
+    jsbytecode *argumentsBytecode() const { JS_ASSERT(code[0] == JSOP_ARGUMENTS); return code; }
+    unsigned argumentsLocalSlot() const { JS_ASSERT(argsHasLocalBinding_); return argsSlot_; }
+    void setArgumentsHasLocalBinding(uint16_t slot);
+
+    /*
+     * As an optimization, even when argsHasLocalBinding, the function prologue
+     * may not need to create an arguments object. This is determined by
+     * needsArgsObj which is set by ScriptAnalysis::analyzeSSA before running
+     * the script the first time. When !needsArgsObj, the prologue may simply
+     * write MagicValue(JS_OPTIMIZED_ARGUMENTS) to 'arguments's slot and any
+     * uses of 'arguments' will be guaranteed to handle this magic value.
+     * So avoid spurious arguments object creation, we maintain the invariant
+     * that needsArgsObj is only called after the script has been analyzed.
+     */
+    bool analyzedArgsUsage() const { return !needsArgsAnalysis_; }
+    bool needsArgsObj() const { JS_ASSERT(analyzedArgsUsage()); return needsArgsObj_; }
+    void setNeedsArgsObj(bool needsArgsObj);
+    bool applySpeculationFailed(JSContext *cx);
+
+    /* Hash table chaining for JSCompartment::evalCache. */
+    JSScript *&evalHashLink() { return *globalObject.unsafeGetUnioned(); }
 
     /*
      * Original compiled function for the script, if it has a function.
@@ -551,36 +616,12 @@ struct JSScript : public js::gc::Cell {
     JSFunction *function() const { return function_; }
     void setFunction(JSFunction *fun);
 
-#ifdef JS_CRASH_DIAGNOSTICS
-    /* All diagnostic fields must be multiples of Cell::CellSize. */
-    uint32_t        cookie2[Cell::CellSize / sizeof(uint32_t)];
-
-    void CheckScript(JSScript *prev);
-#else
-    void CheckScript(JSScript *prev) {}
-#endif /* !JS_CRASH_DIAGNOSTICS */
-
 #ifdef DEBUG
-    /*
-     * Unique identifier within the compartment for this script, used for
-     * printing analysis information.
-     */
-    uint32_t id_;
-    uint32_t idpad;
     unsigned id();
 #else
     unsigned id() { return 0; }
 #endif
 
-    /* Persistent type information retained across GCs. */
-    js::types::TypeScript *types;
-
-#if JS_BITS_PER_WORD == 32
-  private:
-    void *padding_;
-  public:
-#endif
-
     /* Ensure the script has a TypeScript. */
     inline bool ensureHasTypes(JSContext *cx);
 
@@ -621,33 +662,36 @@ struct JSScript : public js::gc::Cell {
   private:
     bool makeTypes(JSContext *cx);
     bool makeAnalysis(JSContext *cx);
-  public:
 
 #ifdef JS_METHODJIT
-    // Fast-cached pointers to make calls faster. These are also used to
-    // quickly test whether there is JIT code; a NULL value means no
-    // compilation has been attempted. A JS_UNJITTABLE_SCRIPT value means
-    // compilation failed. Any value is the arity-check entry point.
-    void *jitArityCheckNormal;
-    void *jitArityCheckCtor;
+  private:
+    // CallCompiler must be a friend because it generates code that directly
+    // accesses jitHandleNormal/jitHandleCtor, via jitHandleOffset().
+    friend class js::mjit::CallCompiler;
 
-    js::mjit::JITScript *jitNormal;   /* Extra JIT info for normal scripts */
-    js::mjit::JITScript *jitCtor;     /* Extra JIT info for constructors */
-#endif
-
-#ifdef JS_METHODJIT
-    bool hasJITCode() {
-        return jitNormal || jitCtor;
+    static size_t jitHandleOffset(bool constructing) {
+        return constructing ? offsetof(JSScript, jitHandleCtor)
+                            : offsetof(JSScript, jitHandleNormal);
     }
 
+  public:
+    bool hasJITCode()   { return jitHandleNormal.isValid() || jitHandleCtor.isValid(); }
+
+    JITScriptHandle *jitHandle(bool constructing) {
+        return constructing ? &jitHandleCtor : &jitHandleNormal;
+    }
+
+    js::mjit::JITScript *getJIT(bool constructing) {
+        JITScriptHandle *jith = jitHandle(constructing);
+        return jith->isValid() ? jith->getValid() : NULL;
+    }
+
+    static void ReleaseCode(js::FreeOp *fop, JITScriptHandle *jith);
+
     // These methods are implemented in MethodJIT.h.
     inline void **nativeMap(bool constructing);
     inline void *nativeCodeForPC(bool constructing, jsbytecode *pc);
 
-    js::mjit::JITScript *getJIT(bool constructing) {
-        return constructing ? jitCtor : jitNormal;
-    }
-
     size_t getUseCount() const  { return useCount; }
     size_t incUseCount() { return ++useCount; }
     size_t *addressOfUseCount() { return &useCount; }
@@ -659,17 +703,18 @@ struct JSScript : public js::gc::Cell {
      * MethodJIT.cpp.)
      */
     size_t sizeOfJitScripts(JSMallocSizeOfFun mallocSizeOf);
-
 #endif
 
-    /* Counter accessors. */
-    js::OpcodeCounts getCounts(jsbytecode *pc) {
-        JS_ASSERT(size_t(pc - code) < length);
-        return pcCounters.counts[pc - code];
-    }
+  public:
+    bool initScriptCounts(JSContext *cx);
+    js::PCCounts getPCCounts(jsbytecode *pc);
+    js::ScriptCounts releaseScriptCounts();
+    void destroyScriptCounts(js::FreeOp *fop);
 
-    bool initCounts(JSContext *cx);
-    void destroyCounts(JSContext *cx);
+    bool setSourceMap(JSContext *cx, jschar *sourceMap);
+    jschar *getSourceMap();
+    jschar *releaseSourceMap();
+    void destroySourceMap(js::FreeOp *fop);
 
     jsbytecode *main() {
         return code + mainOffset;
@@ -691,16 +736,16 @@ struct JSScript : public js::gc::Cell {
     static const uint8_t INVALID_OFFSET = 0xFF;
     static bool isValidOffset(uint8_t offset) { return offset != INVALID_OFFSET; }
 
+    JSConstArray *consts() {
+        JS_ASSERT(isValidOffset(constsOffset));
+        return reinterpret_cast<JSConstArray *>(data + constsOffset);
+    }
+
     JSObjectArray *objects() {
         JS_ASSERT(isValidOffset(objectsOffset));
         return reinterpret_cast<JSObjectArray *>(data + objectsOffset);
     }
 
-    JSUpvarArray *upvars() {
-        JS_ASSERT(isValidOffset(upvarsOffset));
-        return reinterpret_cast<JSUpvarArray *>(data + upvarsOffset);
-    }
-
     JSObjectArray *regexps() {
         JS_ASSERT(isValidOffset(regexpsOffset));
         return reinterpret_cast<JSObjectArray *>(data + regexpsOffset);
@@ -716,12 +761,25 @@ struct JSScript : public js::gc::Cell {
         return reinterpret_cast<js::GlobalSlotArray *>(data + globalsOffset);
     }
 
-    JSConstArray *consts() {
-        JS_ASSERT(isValidOffset(constOffset));
-        return reinterpret_cast<JSConstArray *>(data + constOffset);
+    js::ClosedSlotArray *closedArgs() {
+        JS_ASSERT(isValidOffset(closedArgsOffset));
+        return reinterpret_cast<js::ClosedSlotArray *>(data + closedArgsOffset);
     }
 
-    JSAtom *getAtom(size_t index) {
+    js::ClosedSlotArray *closedVars() {
+        JS_ASSERT(isValidOffset(closedVarsOffset));
+        return reinterpret_cast<js::ClosedSlotArray *>(data + closedVarsOffset);
+    }
+
+    uint32_t numClosedArgs() {
+        return isValidOffset(closedArgsOffset) ? closedArgs()->length : 0;
+    }
+
+    uint32_t numClosedVars() {
+        return isValidOffset(closedVarsOffset) ? closedVars()->length : 0;
+    }
+
+    js::HeapPtrAtom &getAtom(size_t index) const {
         JS_ASSERT(index < natoms);
         return atoms[index];
     }
@@ -759,49 +817,56 @@ struct JSScript : public js::gc::Cell {
     inline bool isEmpty() const;
 
     uint32_t getClosedArg(uint32_t index) {
-        JS_ASSERT(index < nClosedArgs);
-        return closedSlots[index];
+        js::ClosedSlotArray *arr = closedArgs();
+        JS_ASSERT(index < arr->length);
+        return arr->vector[index];
     }
 
     uint32_t getClosedVar(uint32_t index) {
-        JS_ASSERT(index < nClosedVars);
-        return closedSlots[nClosedArgs + index];
+        js::ClosedSlotArray *arr = closedVars();
+        JS_ASSERT(index < arr->length);
+        return arr->vector[index];
     }
 
-    void copyClosedSlotsTo(JSScript *other);
 
+#ifdef DEBUG
+    bool varIsAliased(unsigned varSlot);
+    bool argIsAliased(unsigned argSlot);
+    bool argLivesInArgumentsObject(unsigned argSlot);
+    bool argLivesInCallObject(unsigned argSlot);
+#endif
   private:
-    static const uint32_t stepFlagMask = 0x80000000U;
-    static const uint32_t stepCountMask = 0x7fffffffU;
-
     /*
-     * Attempt to recompile with or without single-stepping support, as directed
+     * Recompile with or without single-stepping support, as directed
      * by stepModeEnabled().
      */
-    bool recompileForStepMode(JSContext *cx);
+    void recompileForStepMode(js::FreeOp *fop);
 
     /* Attempt to change this->stepMode to |newValue|. */
     bool tryNewStepMode(JSContext *cx, uint32_t newValue);
 
-    bool ensureHasDebug(JSContext *cx);
+    bool ensureHasDebugScript(JSContext *cx);
+    js::DebugScript *debugScript();
+    js::DebugScript *releaseDebugScript();
+    void destroyDebugScript(js::FreeOp *fop);
 
   public:
     bool hasBreakpointsAt(jsbytecode *pc) { return !!getBreakpointSite(pc); }
-    bool hasAnyBreakpointsOrStepMode() { return !!debug; }
+    bool hasAnyBreakpointsOrStepMode() { return hasDebugScript; }
 
     js::BreakpointSite *getBreakpointSite(jsbytecode *pc)
     {
         JS_ASSERT(size_t(pc - code) < length);
-        return debug ? debug->breakpoints[pc - code] : NULL;
+        return hasDebugScript ? debugScript()->breakpoints[pc - code] : NULL;
     }
 
     js::BreakpointSite *getOrCreateBreakpointSite(JSContext *cx, jsbytecode *pc,
                                                   js::GlobalObject *scriptGlobal);
 
-    void destroyBreakpointSite(JSRuntime *rt, jsbytecode *pc);
+    void destroyBreakpointSite(js::FreeOp *fop, jsbytecode *pc);
 
-    void clearBreakpointsIn(JSContext *cx, js::Debugger *dbg, JSObject *handler);
-    void clearTraps(JSContext *cx);
+    void clearBreakpointsIn(js::FreeOp *fop, js::Debugger *dbg, JSObject *handler);
+    void clearTraps(js::FreeOp *fop);
 
     void markTrapClosures(JSTracer *trc);
 
@@ -821,13 +886,13 @@ struct JSScript : public js::gc::Cell {
      */
     bool changeStepModeCount(JSContext *cx, int delta);
 
-    bool stepModeEnabled() { return debug && !!debug->stepMode; }
+    bool stepModeEnabled() { return hasDebugScript && !!debugScript()->stepMode; }
 
 #ifdef DEBUG
-    uint32_t stepModeCount() { return debug ? (debug->stepMode & stepCountMask) : 0; }
+    uint32_t stepModeCount() { return hasDebugScript ? (debugScript()->stepMode & stepCountMask) : 0; }
 #endif
 
-    void finalize(JSContext *cx, bool background);
+    void finalize(js::FreeOp *fop);
 
     static inline void writeBarrierPre(JSScript *script);
     static inline void writeBarrierPost(JSScript *script, void *addr);
@@ -842,7 +907,7 @@ struct JSScript : public js::gc::Cell {
     void markChildren(JSTracer *trc);
 };
 
-/* If this fails, padding_ can be removed. */
+/* If this fails, add/remove padding within JSScript. */
 JS_STATIC_ASSERT(sizeof(JSScript) % js::gc::Cell::CellSize == 0);
 
 static JS_INLINE unsigned
@@ -851,12 +916,6 @@ StackDepth(JSScript *script)
     return script->nslots - script->nfixed;
 }
 
-extern void
-js_MarkScriptFilename(const char *filename);
-
-extern void
-js_SweepScriptFilenames(JSCompartment *comp);
-
 /*
  * New-script-hook calling is factored from NewScriptFromEmitter so that it
  * and callers of XDRScript can share this code.  In the case of callers
@@ -866,36 +925,34 @@ js_SweepScriptFilenames(JSCompartment *comp);
 extern JS_FRIEND_API(void)
 js_CallNewScriptHook(JSContext *cx, JSScript *script, JSFunction *fun);
 
-extern void
-js_CallDestroyScriptHook(JSContext *cx, JSScript *script);
-
 namespace js {
 
-struct ScriptOpcodeCountsPair
+extern void
+CallDestroyScriptHook(FreeOp *fop, JSScript *script);
+
+extern const char *
+SaveScriptFilename(JSContext *cx, const char *filename);
+
+extern void
+MarkScriptFilename(const char *filename);
+
+extern void
+SweepScriptFilenames(JSCompartment *comp);
+
+extern void
+FreeScriptFilenames(JSCompartment *comp);
+
+struct ScriptAndCounts
 {
     JSScript *script;
-    ScriptOpcodeCounts counters;
+    ScriptCounts scriptCounts;
 
-    OpcodeCounts &getCounts(jsbytecode *pc) const {
+    PCCounts &getPCCounts(jsbytecode *pc) const {
         JS_ASSERT(unsigned(pc - script->code) < script->length);
-        return counters.counts[pc - script->code];
+        return scriptCounts.pcCountsVector[pc - script->code];
     }
 };
 
-#ifdef JS_CRASH_DIAGNOSTICS
-
-void
-CheckScript(JSScript *script, JSScript *prev);
-
-#else
-
-inline void
-CheckScript(JSScript *script, JSScript *prev)
-{
-}
-
-#endif /* !JS_CRASH_DIAGNOSTICS */
-
 } /* namespace js */
 
 /*
@@ -946,13 +1003,14 @@ extern JSScript *
 CloneScript(JSContext *cx, JSScript *script);
 
 /*
- * NB: after a successful JSXDR_DECODE, XDRScript callers must do any
- * required subsequent set-up of owning function or script object and then call
+ * NB: after a successful XDR_DECODE, XDRScript callers must do any required
+ * subsequent set-up of owning function or script object and then call
  * js_CallNewScriptHook.
  */
-extern JSBool
-XDRScript(JSXDRState *xdr, JSScript **scriptp);
+template<XDRMode mode>
+bool
+XDRScript(XDRState<mode> *xdr, JSScript **scriptp, JSScript *parentScript);
 
-}
+} /* namespace js */
 
 #endif /* jsscript_h___ */
