@@ -48,16 +48,22 @@
 #include "nsDOMClassInfo.h"
 #include "nsGlobalWindow.h"
 #include "nsWrapperCacheInlines.h"
+#include "mozilla/dom/bindings/Utils.h"
 
 #include "jsapi.h"
 #include "jsatom.h"
 
 using namespace JS;
+using namespace mozilla::dom::bindings;
 
 namespace mozilla {
 namespace dom {
 namespace binding {
 
+enum {
+    JSPROXYSLOT_PROTOSHAPE = 0,
+    JSPROXYSLOT_EXPANDO = 1
+};
 
 static jsid s_prototype_id = JSID_VOID;
 
@@ -99,62 +105,6 @@ Throw(JSContext *cx, nsresult rv)
     return false;
 }
 
-
-// Only set allowNativeWrapper to false if you really know you need it, if in
-// doubt use true. Setting it to false disables security wrappers.
-static bool
-XPCOMObjectToJsval(JSContext *cx, JSObject *scope, xpcObjectHelper &helper,
-                   bool allowNativeWrapper, jsval *rval)
-{
-    XPCLazyCallContext lccx(JS_CALLER, cx, scope);
-
-    nsresult rv;
-    if (!XPCConvert::NativeInterface2JSObject(lccx, rval, NULL, helper, NULL, NULL,
-                                              allowNativeWrapper, &rv)) {
-        // I can't tell if NativeInterface2JSObject throws JS exceptions
-        // or not.  This is a sloppy stab at the right semantics; the
-        // method really ought to be fixed to behave consistently.
-        if (!JS_IsExceptionPending(cx))
-            Throw(cx, NS_FAILED(rv) ? rv : NS_ERROR_UNEXPECTED);
-        return false;
-    }
-
-#ifdef DEBUG
-    JSObject* jsobj = JSVAL_TO_OBJECT(*rval);
-    if (jsobj && !js::GetObjectParent(jsobj))
-        NS_ASSERTION(js::GetObjectClass(jsobj)->flags & JSCLASS_IS_GLOBAL,
-                     "Why did we recreate this wrapper?");
-#endif
-
-    return true;
-}
-
-template<class T>
-static inline JSObject*
-WrapNativeParent(JSContext *cx, JSObject *scope, T *p)
-{
-    if (!p)
-        return NULL;
-
-    nsWrapperCache *cache = GetWrapperCache(p);
-    JSObject* obj;
-    if (cache && (obj = cache->GetWrapper())) {
-#ifdef DEBUG
-        qsObjectHelper helper(p, cache);
-        jsval debugVal;
-
-        bool ok = XPCOMObjectToJsval(cx, scope, helper, false, &debugVal);
-        NS_ASSERTION(ok && JSVAL_TO_OBJECT(debugVal) == obj,
-                     "Unexpected object in nsWrapperCache");
-#endif
-        return obj;
-    }
-
-    qsObjectHelper helper(p, cache);
-    jsval v;
-    return XPCOMObjectToJsval(cx, scope, helper, false, &v) ? JSVAL_TO_OBJECT(v) : NULL;
-}
-
 template<class T>
 static bool
 Wrap(JSContext *cx, JSObject *scope, T *p, nsWrapperCache *cache, jsval *vp)
@@ -162,7 +112,7 @@ Wrap(JSContext *cx, JSObject *scope, T *p, nsWrapperCache *cache, jsval *vp)
     if (xpc_FastGetCachedWrapper(cache, scope, vp))
         return true;
     qsObjectHelper helper(p, cache);
-    return XPCOMObjectToJsval(cx, scope, helper, true, vp);
+    return XPCOMObjectToJsval(cx, scope, helper, NULL, true, vp);
 }
 
 template<class T>
@@ -258,7 +208,7 @@ bool
 DefineConstructor(JSContext *cx, JSObject *obj, DefineInterface aDefine, nsresult *aResult)
 {
     bool enabled;
-    bool defined = !!aDefine(cx, XPCWrappedNativeScope::FindInJSObjectScope(cx, obj), &enabled);
+    bool defined = aDefine(cx, obj, &enabled);
     NS_ASSERTION(!defined || enabled,
                  "We defined a constructor but the new bindings are disabled?");
     *aResult = defined ? NS_OK : NS_ERROR_FAILURE;
@@ -464,7 +414,22 @@ InvalidateProtoShape_set(JSContext *cx, JSObject *obj, jsid id, JSBool strict, j
 
 template<class LC>
 JSObject *
-ListBase<LC>::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope)
+ListBase<LC>::getPrototype(JSContext *cx, JSObject *receiver, bool *enabled)
+{
+    *enabled = true;
+
+    XPCWrappedNativeScope *scope =
+        XPCWrappedNativeScope::FindInJSObjectScope(cx, receiver);
+    if (!scope)
+        return false;
+
+    return getPrototype(cx, scope, receiver);
+}
+
+template<class LC>
+JSObject *
+ListBase<LC>::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope,
+                           JSObject *receiver)
 {
     nsDataHashtable<nsDepCharHashKey, JSObject*> &cache =
         scope->GetCachedDOMPrototypes();
@@ -479,7 +444,7 @@ ListBase<LC>::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope)
         return NULL;
     }
 
-    JSObject* proto = Base::getPrototype(cx, scope);
+    JSObject* proto = Base::getPrototype(cx, scope, receiver);
     if (!proto)
         return NULL;
 
@@ -518,7 +483,7 @@ ListBase<LC>::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope)
     if (!JS_LinkConstructorAndPrototype(cx, interface, interfacePrototype))
         return NULL;
 
-    if (!JS_DefineProperty(cx, global, sInterfaceClass.name, OBJECT_TO_JSVAL(interface), NULL,
+    if (!JS_DefineProperty(cx, receiver, sInterfaceClass.name, OBJECT_TO_JSVAL(interface), NULL,
                            NULL, 0))
         return NULL;
 
@@ -534,26 +499,26 @@ ListBase<LC>::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope)
 
 template<class LC>
 JSObject *
-ListBase<LC>::create(JSContext *cx, XPCWrappedNativeScope *scope, ListType *aList,
+ListBase<LC>::create(JSContext *cx, JSObject *scope, ListType *aList,
                      nsWrapperCache* aWrapperCache, bool *triedToWrap)
 {
     *triedToWrap = true;
 
-    JSObject *parent = WrapNativeParent(cx, scope->GetGlobalJSObject(), aList->GetParentObject());
+    JSObject *parent = WrapNativeParent(cx, scope, aList->GetParentObject());
     if (!parent)
         return NULL;
 
-    JSAutoEnterCompartment ac;
-    if (js::GetGlobalForObjectCrossCompartment(parent) != scope->GetGlobalJSObject()) {
-        if (!ac.enter(cx, parent))
-            return NULL;
+    JSObject *global = js::GetGlobalForObjectCrossCompartment(parent);
 
-        scope = XPCWrappedNativeScope::FindInJSObjectScope(cx, parent);
+    JSAutoEnterCompartment ac;
+    if (global != scope) {
+        if (!ac.enter(cx, global))
+            return NULL;
     }
 
-    JSObject *proto = getPrototype(cx, scope, triedToWrap);
+    JSObject *proto = getPrototype(cx, global, triedToWrap);
     if (!proto && !*triedToWrap)
-        aWrapperCache->ClearIsProxy();
+        aWrapperCache->ClearIsDOMBinding();
     if (!proto)
         return NULL;
     JSObject *obj = NewProxyObject(cx, &ListBase<LC>::instance,
@@ -737,9 +702,13 @@ ListBase<LC>::ensureExpandoObject(JSContext *cx, JSObject *obj)
         JSCompartment *compartment = js::GetObjectCompartment(obj);
         xpc::CompartmentPrivate *priv =
             static_cast<xpc::CompartmentPrivate *>(JS_GetCompartmentPrivate(compartment));
-        if (!priv->RegisterDOMExpandoObject(expando))
+        if (!priv->RegisterDOMExpandoObject(obj))
             return NULL;
 
+        nsWrapperCache* cache;
+        CallQueryInterface(getListObject(obj), &cache);
+        cache->SetPreservingWrapper(true);
+
         js::SetProxyExtra(obj, JSPROXYSLOT_EXPANDO, ObjectValue(*expando));
         JS_SetPrivate(expando, js::GetProxyPrivate(obj).toPrivate());
     }
@@ -1277,7 +1246,7 @@ ListBase<LC>::obj_toString(JSContext *cx, JSObject *proxy)
 
 template<class LC>
 void
-ListBase<LC>::finalize(JSContext *cx, JSObject *proxy)
+ListBase<LC>::finalize(JSFreeOp *fop, JSObject *proxy)
 {
     ListType *list = getListObject(proxy);
     nsWrapperCache *cache;
@@ -1296,12 +1265,12 @@ ListBase<LC>::finalize(JSContext *cx, JSObject *proxy)
 
 
 JSObject*
-NoBase::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope)
+NoBase::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope, JSObject *receiver)
 {
     // We need to pass the object prototype to JS_NewObject. If we pass NULL then the JS engine
     // will look up a prototype on the global by using the class' name and we'll recurse into
     // getPrototype.
-    return JS_GetObjectPrototype(cx, scope->GetGlobalJSObject());
+    return JS_GetObjectPrototype(cx, receiver);
 }
 
 
