@@ -233,15 +233,22 @@ class Configuration:
         execfile(filename, config)
 
         # required settings
-        if 'classes' not in config:
+        if 'list_classes' not in config:
             raise UserError(filename + ": `%s` was not defined." % name)
-        self.classes = {}
-        for clazz in config['classes']:
-            self.classes[clazz] = DOMClass(name=clazz, nativeClass=config['classes'][clazz], prefable=False)
+        if 'list_classes' not in config:
+            raise UserError(filename + ": `%s` was not defined." % name)
+        self.list_classes = {}
+        for clazz in config['list_classes']:
+            self.list_classes[clazz['name']] = \
+                DOMClass(name = clazz['name'],
+                         nativeClass = clazz['nativeClass'],
+                         prefable = False)
+
+        # optional settings
         if 'prefableClasses' in config:
             for clazz in config['prefableClasses']:
-                self.classes[clazz] = DOMClass(name=clazz, nativeClass=config['prefableClasses'][clazz], prefable=True)
-        # optional settings
+                self.list_classes[clazz] = DOMClass(name=clazz, nativeClass=config['prefableClasses'][clazz], prefable=True)
+
         self.customInheritance = config.get('customInheritance', {})
         self.derivedClasses = {}
         self.irregularFilenames = config.get('irregularFilenames', {})
@@ -279,7 +286,7 @@ def completeConfiguration(conf, includePath, cachedir):
 
     stubbedInterfaces = []
 
-    for clazz in conf.classes.itervalues():
+    for clazz in conf.list_classes.itervalues():
         interfaceName = 'nsIDOM' + clazz.name
 
         iface = getInterface(interfaceName, errorLoc='looking for %r' % clazz.name)
@@ -310,10 +317,11 @@ def completeConfiguration(conf, includePath, cachedir):
 
             assert iface.name.startswith('nsIDOM') and not iface.name.startswith('nsIDOMNS')
             clazz.base = iface.name[6:]
-            # For now we only support base classes that are real DOM classes
-            assert clazz.base in conf.classes
-            if not conf.classes[clazz.base].isBase:
-                conf.classes[clazz.base].isBase = True
+            # For now we only support base classes that are real DOM
+            # list classes
+            assert clazz.base in conf.list_classes
+            if not conf.list_classes[clazz.base].isBase:
+                conf.list_classes[clazz.base].isBase = True
                 conf.derivedClasses[clazz.base] = []
             conf.derivedClasses[clazz.base].append(clazz.name)
 
@@ -353,16 +361,16 @@ listDefinitionTemplate = (
 "class ${name} {\n"
 "public:\n"
 "    template<typename I>\n"
-"    static JSObject *create(JSContext *cx, XPCWrappedNativeScope *scope, I *list, bool *triedToWrap)\n"
+"    static JSObject *create(JSContext *cx, JSObject *scope, I *list, bool *triedToWrap)\n"
 "    {\n"
-"        return create(cx, scope, list, GetWrapperCache(list), triedToWrap);\n"
+"        return create(cx, scope, list, list, triedToWrap);\n"
 "    }\n"
 "\n"
 "    static bool objIsWrapper(JSObject *obj);\n"
 "    static ${nativeClass} *getNative(JSObject *obj);\n"
 "\n"
 "private:\n"
-"    static JSObject *create(JSContext *cx, XPCWrappedNativeScope *scope, ${nativeClass} *list, nsWrapperCache *cache, bool *triedToWrap);\n"
+"    static JSObject *create(JSContext *cx, JSObject *scope, ${nativeClass} *list, nsWrapperCache *cache, bool *triedToWrap);\n"
 "};"
 "\n"
 "\n")
@@ -378,7 +386,7 @@ def writeHeaderFile(filename, config):
                 "#define " + headerMacro + "\n\n")
 
         namespaces = []
-        for type in sorted(getTypes(config.classes, {})):
+        for type in sorted(getTypes(config.list_classes, {})):
             newNamespaces = type.split('::')
             type = newNamespaces.pop()
             j = 0
@@ -403,7 +411,7 @@ def writeHeaderFile(filename, config):
         f.write("bool\n"
                 "DefinePropertyStaticJSVals(JSContext *cx);\n\n")
 
-        for clazz in config.classes.itervalues():
+        for clazz in config.list_classes.itervalues():
             f.write(string.Template(listDefinitionTemplate).substitute(clazz))
 
         f.write("\n"
@@ -452,10 +460,10 @@ listTemplate = (
 "    JS_EnumerateStub,\n"
 "    JS_ResolveStub,\n"
 "    JS_ConvertStub,\n"
-"    JS_FinalizeStub,\n"
+"    NULL,                   /* finalize    */\n"
 "    NULL,                   /* checkAccess */\n"
-"    NULL,                   /* call */\n"
-"    NULL,                   /* construct */\n"
+"    NULL,                   /* call        */\n"
+"    NULL,                   /* construct   */\n"
 "    interface_hasInstance\n"
 "};\n"
 "\n")
@@ -489,15 +497,20 @@ derivedClassTemplate = (
 prefableClassTemplate = (
 "template<>\n"
 "JSObject *\n"
-"${name}Wrapper::getPrototype(JSContext *cx, XPCWrappedNativeScope *scope, bool *enabled)\n"
+"${name}Wrapper::getPrototype(JSContext *cx, JSObject *receiver, bool *enabled)\n"
 "{\n"
+"    XPCWrappedNativeScope *scope =\n"
+"        XPCWrappedNativeScope::FindInJSObjectScope(cx, receiver);\n"
+"    if (!scope)\n"
+"        return false;\n"
+"\n"
 "    if (!scope->NewDOMBindingsEnabled()) {\n"
 "        *enabled = false;\n"
 "        return NULL;\n"
 "    }\n"
 "\n"
 "    *enabled = true;\n"
-"    return getPrototype(cx, scope);\n"
+"    return getPrototype(cx, scope, receiver);\n"
 "}\n"
 "\n")
 
@@ -573,7 +586,7 @@ listTemplateFooter = (
 "template class ListBase<${name}Class>;\n"
 "\n"
 "JSObject*\n"
-"${name}::create(JSContext *cx, XPCWrappedNativeScope *scope, ${nativeClass} *list, nsWrapperCache *cache, bool *triedToWrap)\n"
+"${name}::create(JSContext *cx, JSObject *scope, ${nativeClass} *list, nsWrapperCache *cache, bool *triedToWrap)\n"
 "{\n"
 "    return ${name}Wrapper::create(cx, scope, list, cache, triedToWrap);\n"
 "}\n"
@@ -643,8 +656,9 @@ def writeStubFile(filename, config, interfaces):
     try:
         f.write("/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */\n\n")
 
-        types = getTypes(config.classes, config.irregularFilenames)
-        for clazz in config.classes.itervalues():
+        f.write("#include \"nsScriptNameSpaceManager.h\"\n")
+        types = getTypes(config.list_classes, config.irregularFilenames)
+        for clazz in config.list_classes.itervalues():
             for member in clazz.members:
                 addType(types, member.realtype, config.irregularFilenames)
                 if member.kind == 'method':
@@ -661,7 +675,7 @@ def writeStubFile(filename, config, interfaces):
         f.write("// Property name ids\n\n")
 
         ids = set()
-        for clazz in config.classes.itervalues():
+        for clazz in config.list_classes.itervalues():
             assert clazz.indexGetter
             ids.add(clazz.indexGetter.name)
             if clazz.indexSetter:
@@ -685,7 +699,7 @@ def writeStubFile(filename, config, interfaces):
         f.write("\n"
                 "}\n\n")
 
-        classes = sorted(config.classes.values())
+        classes = sorted(config.list_classes.values())
 
         f.write("// Typedefs\n\n")
 
@@ -750,12 +764,12 @@ def writeStubFile(filename, config, interfaces):
         f.write("// Register prototypes\n\n")
 
         f.write("void\n"
-                "Register(nsDOMClassInfoData *aData)\n"
+                "Register(nsScriptNameSpaceManager* aNameSpaceManager)\n"
                 "{\n"
                 "#define REGISTER_PROTO(_dom_class) \\\n"
-                "    aData[eDOMClassInfo_##_dom_class##_id].mDefineDOMInterface = _dom_class##Wrapper::getPrototype\n"
+                "    aNameSpaceManager->RegisterDefineDOMInterface(NS_LITERAL_STRING(#_dom_class), _dom_class##Wrapper::DefineDOMInterface);\n\n"""
                 "\n")
-        for clazz in config.classes.itervalues():
+        for clazz in config.list_classes.itervalues():
             f.write("    REGISTER_PROTO(%s);\n" % clazz.name)
         f.write("\n"
                 "#undef REGISTER_PROTO\n"
