@@ -37,7 +37,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "WrapperFactory.h"
 #include "CrossOriginWrapper.h"
 #include "FilteringWrapper.h"
 #include "XrayWrapper.h"
@@ -47,7 +46,7 @@
 #include "xpcprivate.h"
 #include "dombindings.h"
 #include "XPCMaps.h"
-
+#include "mozilla/dom/bindings/Utils.h"
 #include "jsfriendapi.h"
 
 using namespace js;
@@ -81,7 +80,7 @@ GetCurrentOuter(JSContext *cx, JSObject *obj)
 {
     obj = JS_ObjectToOuterObject(cx, obj);
     if (!obj)
-        return NULL;
+        return nsnull;
 
     if (IsWrapper(obj) && !js::GetObjectClass(obj)->ext.innerObject) {
         obj = UnwrapObject(obj);
@@ -194,11 +193,6 @@ WrapperFactory::PrepareForWrapping(JSContext *cx, JSObject *scope, JSObject *obj
 
     XPCWrappedNative *wn = static_cast<XPCWrappedNative *>(xpc_GetJSPrivate(obj));
 
-    // If the object doesn't have classinfo we want to return the same
-    // XPCWrappedNative so that we keep the same set of interfaces.
-    if (!wn->GetClassInfo())
-        return DoubleWrap(cx, obj, flags);
-
     JSAutoEnterCompartment ac;
     if (!ac.enter(cx, obj))
         return nsnull;
@@ -242,9 +236,21 @@ WrapperFactory::PrepareForWrapping(JSContext *cx, JSObject *scope, JSObject *obj
         obj = JSVAL_TO_OBJECT(v);
         NS_ASSERTION(IS_WN_WRAPPER(obj), "bad object");
 
+        // Because the underlying native didn't have a PreCreate hook, we had
+        // to a new (or possibly pre-existing) XPCWN in our compartment.
+        // This could be a problem for chrome code that passes XPCOM objects
+        // across compartments, because the effects of QI would disappear across
+        // compartments.
+        //
+        // So whenever we pull an XPCWN across compartments in this manner, we
+        // give the destination object the union of the two native sets. We try
+        // to do this cleverly in the common case to avoid too much overhead.
         XPCWrappedNative *newwn = static_cast<XPCWrappedNative *>(xpc_GetJSPrivate(obj));
-        if (newwn->GetSet()->GetInterfaceCount() < wn->GetSet()->GetInterfaceCount())
-            newwn->SetSet(wn->GetSet());
+        XPCNativeSet *unionSet = XPCNativeSet::GetNewOrUsed(ccx, newwn->GetSet(),
+                                                            wn->GetSet(), false);
+        if (!unionSet)
+            return nsnull;
+        newwn->SetSet(unionSet);
     }
 
     return DoubleWrap(cx, obj, flags);
@@ -259,14 +265,29 @@ GetWrappedNative(JSContext *cx, JSObject *obj)
            : nsnull;
 }
 
-static bool
-CanXray(JSObject *obj, bool *proxy)
+enum XrayType {
+    XrayForDOMObject,
+    XrayForDOMProxyObject,
+    XrayForWrappedNative,
+    NotXray
+};
+
+static XrayType
+GetXrayType(JSObject *obj)
 {
-    if (IS_WN_WRAPPER(obj) || js::GetObjectClass(obj)->ext.innerObject) {
-        *proxy = false;
-        return true;
+    js::Class* clasp = js::GetObjectClass(obj);
+    if (mozilla::dom::bindings::IsDOMClass(Jsvalify(clasp))) {
+        return XrayForDOMObject;
     }
-    return (*proxy = mozilla::dom::binding::instanceIsProxy(obj));
+    if (mozilla::dom::binding::instanceIsProxy(obj)) {
+        return XrayForDOMProxyObject;
+    }
+    if (IS_WRAPPER_CLASS(clasp) || clasp->ext.innerObject) {
+        NS_ASSERTION(clasp->ext.innerObject || IS_WN_WRAPPER_OBJECT(obj),
+                     "We forgot to Morph a slim wrapper!");
+        return XrayForWrappedNative;
+    }
+    return NotXray;
 }
 
 JSObject *
@@ -281,7 +302,7 @@ WrapperFactory::Rewrap(JSContext *cx, JSObject *obj, JSObject *wrappedProto, JSO
 
     JSCompartment *origin = js::GetObjectCompartment(obj);
     JSCompartment *target = js::GetContextCompartment(cx);
-    JSObject *xrayHolder = nsnull;
+    bool usingXray = false;
 
     Wrapper *wrapper;
     CompartmentPrivate *targetdata =
@@ -308,17 +329,15 @@ WrapperFactory::Rewrap(JSContext *cx, JSObject *obj, JSObject *wrappedProto, JSO
                 wrapper = &CrossOriginWrapper::singleton;
             } else {
                 // Native objects must be wrapped into an X-ray wrapper.
-                bool proxy;
-                if (CanXray(obj, &proxy)) {
-                    if (proxy) {
-                        wrapper = &XrayProxy::singleton;
-                    } else {
-                        typedef XrayWrapper<CrossCompartmentSecurityWrapper> Xray;
-                        wrapper = &Xray::singleton;
-                        xrayHolder = Xray::createHolder(cx, obj, parent);
-                        if (!xrayHolder)
-                            return nsnull;
-                    }
+                XrayType type = GetXrayType(obj);
+                if (type == XrayForDOMObject) {
+                    wrapper = &XrayDOM::singleton;
+                } else if (type == XrayForDOMProxyObject) {
+                    wrapper = &XrayProxy::singleton;
+                } else if (type == XrayForWrappedNative) {
+                    typedef XrayWrapper<CrossCompartmentWrapper> Xray;
+                    usingXray = true;
+                    wrapper = &Xray::singleton;
                 } else {
                     wrapper = &NoWaiverWrapper::singleton;
                 }
@@ -338,34 +357,70 @@ WrapperFactory::Rewrap(JSContext *cx, JSObject *obj, JSObject *wrappedProto, JSO
             (wn = GetWrappedNative(cx, obj)) &&
             wn->HasProto() && wn->GetProto()->ClassIsDOMObject()) {
             typedef XrayWrapper<CrossCompartmentSecurityWrapper> Xray;
-            wrapper = &FilteringWrapper<Xray,
-                                        CrossOriginAccessiblePropertiesOnly>::singleton;
-            xrayHolder = Xray::createHolder(cx, obj, parent);
-            if (!xrayHolder)
-                return nsnull;
+            usingXray = true;
+            if (IsLocationObject(obj))
+                wrapper = &FilteringWrapper<Xray, LocationPolicy>::singleton;
+            else
+                wrapper = &FilteringWrapper<Xray, CrossOriginAccessiblePropertiesOnly>::singleton;
+        } else if (mozilla::dom::binding::instanceIsProxy(obj)) {
+            wrapper = &FilteringWrapper<XrayProxy, CrossOriginAccessiblePropertiesOnly>::singleton;
+        } else if (mozilla::dom::bindings::IsDOMClass(JS_GetClass(obj))) {
+            wrapper = &FilteringWrapper<XrayDOM, CrossOriginAccessiblePropertiesOnly>::singleton;
         } else {
             wrapper = &FilteringWrapper<CrossCompartmentSecurityWrapper,
                                         ExposedPropertiesOnly>::singleton;
         }
     } else if (AccessCheck::isSameOrigin(origin, target)) {
-        // Same origin we use a transparent wrapper, unless the compartment asks
-        // for an Xray or the wrapper needs a SOW.
-        bool proxy;
+        // For the same-origin case we use a transparent wrapper, unless one
+        // of the following is true:
+        // * The object is flagged as needing a SOW.
+        // * The object is a location object.
+        // * The context compartment specifically requested Xray vision into
+        //   same-origin compartments.
+        //
+        // The first two cases always require a security wrapper for non-chrome
+        // access, regardless of the origin of the object.
+        //
+        // The Location case is a bit tricky. Because the security characteristics
+        // depend on the current outer window, we always have a security wrapper
+        // around locations, same-compartment or cross-compartment. We would
+        // normally just use an identical security policy and just switch between
+        // Wrapper and CrossCompartmentWrapper to differentiate the cases (LW/XLW).
+        // However, there's an added wrinkle that same-origin-but-cross-compartment
+        // scripts expect to be able to see expandos on each others' location
+        // objects. So if all cross-compartment access used XLWs, then the expandos
+        // would live on the per-compartment XrayWrapper expando object, and would
+        // not be shared. So to make sure that expandos work correctly in the
+        // same-origin case, we need to use a transparent CrossCompartmentWrapper
+        // to the LW in the host compartment, rather than an XLW directly to the
+        // Location object. This still doesn't share expandos in the
+        // document.domain case, but that's probably fine. Double-wrapping sucks,
+        // but it's kind of unavoidable here.
+        XrayType type;
         if (AccessCheck::needsSystemOnlyWrapper(obj)) {
             wrapper = &FilteringWrapper<CrossCompartmentSecurityWrapper,
                                         OnlyIfSubjectIsSystem>::singleton;
-        } else if (targetdata && targetdata->wantXrays && CanXray(obj, &proxy)) {
-            if (proxy) {
-                wrapper = &XrayProxy::singleton;
-            } else {
-                typedef XrayWrapper<CrossCompartmentWrapper> Xray;
-                wrapper = &Xray::singleton;
-                xrayHolder = Xray::createHolder(cx, obj, parent);
-                if (!xrayHolder)
+        } else if (!targetdata || !targetdata->wantXrays ||
+                   (type = GetXrayType(obj)) == NotXray) {
+            // Do the double-wrapping if need be.
+            if (IsLocationObject(obj)) {
+                JSAutoEnterCompartment ac;
+                if (!ac.enter(cx, obj))
                     return nsnull;
+                XPCWrappedNative *wn = GetWrappedNative(cx, obj);
+                if (!wn)
+                    return nsnull;
+                obj = wn->GetSameCompartmentSecurityWrapper(cx);
             }
-        } else {
             wrapper = &CrossCompartmentWrapper::singleton;
+        } else if (type == XrayForDOMObject) {
+            wrapper = &XrayDOM::singleton;
+        } else if (type == XrayForDOMProxyObject) {
+            wrapper = &XrayProxy::singleton;
+        } else {
+            typedef XrayWrapper<CrossCompartmentWrapper> Xray;
+            usingXray = true;
+            wrapper = &Xray::singleton;
         }
     } else {
         NS_ASSERTION(!AccessCheck::needsSystemOnlyWrapper(obj),
@@ -375,44 +430,62 @@ WrapperFactory::Rewrap(JSContext *cx, JSObject *obj, JSObject *wrappedProto, JSO
         // a predefined set of properties. XrayWrapper adds a property
         // (.wrappedJSObject) which allows bypassing the XrayWrapper, but
         // we filter out access to that property.
-        bool proxy;
-        if (!CanXray(obj, &proxy)) {
+        XrayType type = GetXrayType(obj);
+        if (type == NotXray) {
             wrapper = &FilteringWrapper<CrossCompartmentSecurityWrapper,
                                         CrossOriginAccessiblePropertiesOnly>::singleton;
+        } else if (type == XrayForDOMObject) {
+            wrapper = &FilteringWrapper<XrayDOM,
+                                        CrossOriginAccessiblePropertiesOnly>::singleton;
+        } else if (type == XrayForDOMProxyObject) {
+            wrapper = &FilteringWrapper<XrayProxy,
+                                        CrossOriginAccessiblePropertiesOnly>::singleton;
         } else {
-            if (proxy) {
-                wrapper = &FilteringWrapper<XrayProxy,
-                                            CrossOriginAccessiblePropertiesOnly>::singleton;
+            typedef XrayWrapper<CrossCompartmentSecurityWrapper> Xray;
+            usingXray = true;
+
+            // Location objects can become same origin after navigation, so we might
+            // have to grant transparent access later on.
+            if (IsLocationObject(obj)) {
+                wrapper = &FilteringWrapper<Xray, LocationPolicy>::singleton;
             } else {
-                typedef XrayWrapper<CrossCompartmentSecurityWrapper> Xray;
-
-                // Location objects can become same origin after navigation, so we might
-                // have to grant transparent access later on.
-                if (IsLocationObject(obj)) {
-                    wrapper = &FilteringWrapper<Xray,
-                        SameOriginOrCrossOriginAccessiblePropertiesOnly>::singleton;
-                } else {
-                    wrapper = &FilteringWrapper<Xray,
-                        CrossOriginAccessiblePropertiesOnly>::singleton;
-                }
-
-                xrayHolder = Xray::createHolder(cx, obj, parent);
-                if (!xrayHolder)
-                    return nsnull;
+                wrapper = &FilteringWrapper<Xray,
+                    CrossOriginAccessiblePropertiesOnly>::singleton;
             }
         }
     }
 
     JSObject *wrapperObj = Wrapper::New(cx, obj, wrappedProto, parent, wrapper);
-    if (!wrapperObj || !xrayHolder)
+    if (!wrapperObj || !usingXray)
         return wrapperObj;
 
+    JSObject *xrayHolder = XrayUtils::createHolder(cx, obj, parent);
+    if (!xrayHolder)
+        return nsnull;
     js::SetProxyExtra(wrapperObj, 0, js::ObjectValue(*xrayHolder));
     return wrapperObj;
 }
 
-typedef FilteringWrapper<XrayWrapper<SameCompartmentSecurityWrapper>,
-                         SameOriginOrCrossOriginAccessiblePropertiesOnly> LW;
+JSObject *
+WrapperFactory::WrapForSameCompartment(JSContext *cx, JSObject *obj)
+{
+    // Only WNs have same-compartment wrappers.
+    //
+    // NB: The contract of WrapForSameCompartment says that |obj| may or may not
+    // be a security wrapper. This check implicitly handles the security wrapper
+    // case.
+    if (!IS_WN_WRAPPER(obj))
+        return obj;
+
+    // Extract the WN. It should exist.
+    XPCWrappedNative *wn = static_cast<XPCWrappedNative *>(xpc_GetJSPrivate(obj));
+    MOZ_ASSERT(wn, "Trying to wrap a dead WN!");
+
+    // The WN knows what to do.
+    return wn->GetSameCompartmentSecurityWrapper(cx);
+}
+
+typedef FilteringWrapper<XrayWrapper<SameCompartmentSecurityWrapper>, LocationPolicy> LW;
 
 bool
 WrapperFactory::IsLocationObject(JSObject *obj)
@@ -424,7 +497,7 @@ WrapperFactory::IsLocationObject(JSObject *obj)
 JSObject *
 WrapperFactory::WrapLocationObject(JSContext *cx, JSObject *obj)
 {
-    JSObject *xrayHolder = LW::createHolder(cx, obj, js::GetObjectParent(obj));
+    JSObject *xrayHolder = XrayUtils::createHolder(cx, obj, js::GetObjectParent(obj));
     if (!xrayHolder)
         return nsnull;
     JSObject *wrapperObj = Wrapper::New(cx, obj, js::GetObjectProto(obj), js::GetObjectParent(obj),
