@@ -61,6 +61,10 @@
 #include "nsIStringBundle.h"
 #include "nsContentUtils.h"
 
+#ifdef ACCESSIBILITY
+#include "nsAccessibilityService.h"
+#endif
+
 namespace dom = mozilla::dom;
 
 static NS_DEFINE_CID(kCStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
@@ -811,7 +815,16 @@ nsImageMap::UpdateAreas()
   bool foundAnchor = false;
   mContainsBlockContents = false;
 
-  return SearchForAreas(mMap, foundArea, foundAnchor);
+  nsresult rv = SearchForAreas(mMap, foundArea, foundAnchor);
+#ifdef ACCESSIBILITY
+  if (NS_SUCCEEDED(rv)) {
+    nsAccessibilityService* accService = GetAccService();
+    if (accService) {
+      accService->UpdateImageMap(mImageFrame);
+    }
+  }
+#endif
+  return rv;
 }
 
 nsresult
@@ -886,6 +899,12 @@ nsImageMap::GetArea(nscoord aX, nscoord aY) const
   return nsnull;
 }
 
+nsIContent*
+nsImageMap::GetAreaAt(PRUint32 aIndex) const
+{
+  return mAreas.ElementAt(aIndex)->mArea;
+}
+
 void
 nsImageMap::Draw(nsIFrame* aFrame, nsRenderingContext& aRC)
 {
