@@ -36,22 +36,26 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+// Main header first:
 #include "nsSVGForeignObjectFrame.h"
 
-#include "nsIDOMSVGForeignObjectElem.h"
-#include "nsIDOMSVGSVGElement.h"
-#include "nsSVGOuterSVGFrame.h"
-#include "nsRegion.h"
-#include "nsGkAtoms.h"
-#include "nsLayoutUtils.h"
-#include "nsSVGUtils.h"
-#include "nsIURI.h"
-#include "nsSVGRect.h"
-#include "nsINameSpaceManager.h"
-#include "nsSVGForeignObjectElement.h"
-#include "nsSVGContainerFrame.h"
+// Keep others in (case-insensitive) order:
 #include "gfxContext.h"
 #include "gfxMatrix.h"
+#include "nsGkAtoms.h"
+#include "nsIDOMSVGForeignObjectElem.h"
+#include "nsINameSpaceManager.h"
+#include "nsLayoutUtils.h"
+#include "nsRegion.h"
+#include "nsRenderingContext.h"
+#include "nsSVGContainerFrame.h"
+#include "nsSVGEffects.h"
+#include "nsSVGForeignObjectElement.h"
+#include "nsSVGOuterSVGFrame.h"
+#include "nsSVGUtils.h"
+#include "mozilla/AutoRestore.h"
+
+using namespace mozilla;
 
 //----------------------------------------------------------------------
 // Implementation
@@ -91,20 +95,12 @@ nsSVGForeignObjectFrame::Init(nsIContent* aContent,
 
   nsresult rv = nsSVGForeignObjectFrameBase::Init(aContent, aParent, aPrevInFlow);
   AddStateBits(aParent->GetStateBits() &
-               (NS_STATE_SVG_NONDISPLAY_CHILD | NS_STATE_SVG_CLIPPATH_CHILD |
-                NS_STATE_SVG_REDRAW_SUSPENDED));
-  if (NS_SUCCEEDED(rv)) {
-    nsSVGUtils::GetOuterSVGFrame(this)->RegisterForeignObject(this);
-  }
+               (NS_STATE_SVG_NONDISPLAY_CHILD | NS_STATE_SVG_CLIPPATH_CHILD));
+  AddStateBits(NS_FRAME_FONT_INFLATION_CONTAINER |
+               NS_FRAME_FONT_INFLATION_FLOW_ROOT);
   return rv;
 }
 
-void nsSVGForeignObjectFrame::DestroyFrom(nsIFrame* aDestructRoot)
-{
-  nsSVGUtils::GetOuterSVGFrame(this)->UnregisterForeignObject(this);
-  nsSVGForeignObjectFrameBase::DestroyFrom(aDestructRoot);
-}
-
 nsIAtom *
 nsSVGForeignObjectFrame::GetType() const
 {
@@ -119,17 +115,18 @@ nsSVGForeignObjectFrame::AttributeChanged(PRInt32  aNameSpaceID,
   if (aNameSpaceID == kNameSpaceID_None) {
     if (aAttribute == nsGkAtoms::width ||
         aAttribute == nsGkAtoms::height) {
-      UpdateGraphic(); // update mRect before requesting reflow
+      nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
       // XXXjwatt: why mark intrinsic widths dirty? can't we just use eResize?
       RequestReflow(nsIPresShell::eStyleChange);
     } else if (aAttribute == nsGkAtoms::x ||
                aAttribute == nsGkAtoms::y ||
-               aAttribute == nsGkAtoms::viewBox ||
-               aAttribute == nsGkAtoms::preserveAspectRatio ||
                aAttribute == nsGkAtoms::transform) {
       // make sure our cached transform matrix gets (lazily) updated
       mCanvasTM = nsnull;
-      UpdateGraphic();
+      nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
+    } else if (aAttribute == nsGkAtoms::viewBox ||
+               aAttribute == nsGkAtoms::preserveAspectRatio) {
+      nsSVGUtils::InvalidateBounds(this);
     }
   }
 
@@ -145,7 +142,10 @@ nsSVGForeignObjectFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
   // Moreover we haven't been initialised properly yet so we may not have the
   // right state bits.
   if (!(GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
-    UpdateGraphic();
+    // XXXperf: probably only need a bounds update if 'font-size' changed and
+    // we have em unit width/height. Or, once we map 'transform' into style,
+    // if some transform property changed.
+    nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
   }
 }
 
@@ -155,8 +155,18 @@ nsSVGForeignObjectFrame::Reflow(nsPresContext*           aPresContext,
                                 const nsHTMLReflowState& aReflowState,
                                 nsReflowStatus&          aStatus)
 {
-  // InitialUpdate and AttributeChanged make sure mRect is up to date before
-  // we're called (UpdateCoveredRegion sets mRect).
+  NS_ABORT_IF_FALSE(!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+                    "Should not have been called");
+
+  // Only InvalidateAndScheduleBoundsUpdate marks us with NS_FRAME_IS_DIRTY,
+  // so if that bit is still set we still have a resize pending. If we hit
+  // this assertion, then we should get the presShell to skip reflow roots
+  // that have a dirty parent since a reflow is going to come via the
+  // reflow root's parent anyway.
+  NS_ASSERTION(!(GetStateBits() & NS_FRAME_IS_DIRTY),
+               "Reflowing while a resize is pending is wasteful");
+
+  // UpdateBounds makes sure mRect is up to date before we're called.
 
   NS_ASSERTION(!aReflowState.parentReflowState,
                "should only get reflow from being reflow root");
@@ -183,13 +193,27 @@ nsSVGForeignObjectFrame::InvalidateInternal(const nsRect& aDamageRect,
 {
   // This is called by our descendants when they change.
 
-  if (GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD)
+  if (GetStateBits() & NS_FRAME_IS_DIRTY) {
+    // Our entire area has been (or will be) invalidated, so no point
+    // keeping track of sub-areas that our descendants dirty.
     return;
+  }
+
+  if (GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD) {
+    nsSVGEffects::InvalidateRenderingObservers(this);
+    return;
+  }
+
+  if (!mInReflow) {
+    // We can't collect dirty areas, since we don't have a place to reliably
+    // call FlushDirtyRegion before we paint, so we have to invalidate now.
+    InvalidateDirtyRect(nsSVGUtils::GetOuterSVGFrame(this), aDamageRect + nsPoint(aX, aY), aFlags);
+    return;
+  }
 
   nsRegion* region = (aFlags & INVALIDATE_CROSS_DOC)
     ? &mSubDocDirtyRegion : &mSameDocDirtyRegion;
   region->Or(*region, aDamageRect + nsPoint(aX, aY));
-  FlushDirtyRegion(aFlags);
 }
 
 
@@ -345,11 +369,21 @@ nsSVGForeignObjectFrame::GetCoveredRegion()
   return mCoveredRegion;
 }
 
-NS_IMETHODIMP
-nsSVGForeignObjectFrame::UpdateCoveredRegion()
+void
+nsSVGForeignObjectFrame::UpdateBounds()
 {
-  if (GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD)
-    return NS_ERROR_FAILURE;
+  NS_ASSERTION(nsSVGUtils::OuterSVGIsCallingUpdateBounds(this),
+               "This call is probaby a wasteful mistake");
+
+  NS_ABORT_IF_FALSE(!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+                    "UpdateBounds mechanism not designed for this");
+
+  if (!nsSVGUtils::NeedsUpdatedBounds(this)) {
+    return;
+  }
+
+  // We update mRect before the DoReflow call so that DoReflow uses the
+  // correct dimensions:
 
   float x, y, w, h;
   static_cast<nsSVGForeignObjectElement*>(mContent)->
@@ -365,30 +399,32 @@ nsSVGForeignObjectFrame::UpdateCoveredRegion()
                            PresContext()->AppUnitsPerCSSPixel());
   mCoveredRegion = ToCanvasBounds(gfxRect(0.0, 0.0, w, h), GetCanvasTM(), PresContext());
 
-  return NS_OK;
-}
+  // Since we'll invalidate our entire area at the end of this method, we
+  // empty our cached dirty regions to prevent FlushDirtyRegion under DoReflow
+  // from wasting time invalidating:
+  mSameDocDirtyRegion.SetEmpty();
+  mSubDocDirtyRegion.SetEmpty();
 
-NS_IMETHODIMP
-nsSVGForeignObjectFrame::InitialUpdate()
-{
-  NS_ASSERTION(GetStateBits() & NS_FRAME_FIRST_REFLOW,
-               "Yikes! We've been called already! Hopefully we weren't called "
-               "before our nsSVGOuterSVGFrame's initial Reflow()!!!");
+  // Fully mark our kid dirty so that it gets resized if necessary
+  // (NS_FRAME_HAS_DIRTY_CHILDREN isn't enough in that case):
+  nsIFrame* kid = GetFirstPrincipalChild();
+  kid->AddStateBits(NS_FRAME_IS_DIRTY);
 
-  UpdateCoveredRegion();
-
-  // Make sure to not allow interrupts if we're not being reflown as a root
+  // Make sure to not allow interrupts if we're not being reflown as a root:
   nsPresContext::InterruptPreventer noInterrupts(PresContext());
+
   DoReflow();
 
-  NS_ASSERTION(!(mState & NS_FRAME_IN_REFLOW),
-               "We don't actually participate in reflow");
-  
-  // Do unset the various reflow bits, though.
+  // Now unset the various reflow bits:
   mState &= ~(NS_FRAME_FIRST_REFLOW | NS_FRAME_IS_DIRTY |
               NS_FRAME_HAS_DIRTY_CHILDREN);
 
-  return NS_OK;
+  if (!(GetParent()->GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
+    // We only invalidate if our outer-<svg> has already had its
+    // initial reflow (since if it hasn't, its entire area will be
+    // invalidated when it gets that initial reflow):
+    nsSVGUtils::InvalidateBounds(this, true);
+  }
 }
 
 void
@@ -401,9 +437,32 @@ nsSVGForeignObjectFrame::NotifySVGChanged(PRUint32 aFlags)
   NS_ABORT_IF_FALSE(aFlags & (TRANSFORM_CHANGED | COORD_CONTEXT_CHANGED),
                     "Invalidation logic may need adjusting");
 
-  bool reflow = false;
+  bool needNewBounds = false; // i.e. mRect or visual overflow rect
+  bool needReflow = false;
+  bool needNewCanvasTM = false;
+
+  if (aFlags & COORD_CONTEXT_CHANGED) {
+    nsSVGForeignObjectElement *fO =
+      static_cast<nsSVGForeignObjectElement*>(mContent);
+    // Coordinate context changes affect mCanvasTM if we have a
+    // percentage 'x' or 'y'
+    if (fO->mLengthAttributes[nsSVGForeignObjectElement::X].IsPercentage() ||
+        fO->mLengthAttributes[nsSVGForeignObjectElement::Y].IsPercentage()) {
+      needNewBounds = true;
+      needNewCanvasTM = true;
+    }
+    // Our coordinate context's width/height has changed. If we have a
+    // percentage width/height our dimensions will change so we must reflow.
+    if (fO->mLengthAttributes[nsSVGForeignObjectElement::WIDTH].IsPercentage() ||
+        fO->mLengthAttributes[nsSVGForeignObjectElement::HEIGHT].IsPercentage()) {
+      needNewBounds = true;
+      needReflow = true;
+    }
+  }
 
   if (aFlags & TRANSFORM_CHANGED) {
+    needNewBounds = true; // needed if it was _our_ transform that changed
+    needNewCanvasTM = true;
     // In an ideal world we would reflow when our CTM changes. This is because
     // glyph metrics do not necessarily scale uniformly with change in scale
     // and, as a result, CTM changes may require text to break at different
@@ -411,64 +470,32 @@ nsSVGForeignObjectFrame::NotifySVGChanged(PRUint32 aFlags)
     // e.g. the transform of an ancestor is animated.
     // We also seem to get some sort of infinite loop post bug 421584 if we
     // reflow.
+  }
+
+  if (needNewBounds &&
+      !(aFlags & DO_NOT_NOTIFY_RENDERING_OBSERVERS)) {
+    nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
+  }
+
+  // If we're called while the PresShell is handling reflow events then we
+  // must have been called as a result of the NotifyViewportChange() call in
+  // our nsSVGOuterSVGFrame's Reflow() method. We must not call RequestReflow
+  // at this point (i.e. during reflow) because it could confuse the
+  // PresShell and prevent it from reflowing us properly in future. Besides
+  // that, nsSVGOuterSVGFrame::DidReflow will take care of reflowing us
+  // synchronously, so there's no need.
+  if (needReflow && !PresContext()->PresShell()->IsReflowLocked()) {
+    RequestReflow(nsIPresShell::eResize);
+  }
+
+  if (needNewCanvasTM) {
+    // Do this after calling InvalidateAndScheduleBoundsUpdate in case we
+    // change the code and it needs to use it.
     mCanvasTM = nsnull;
-    if (!(aFlags & DO_NOT_NOTIFY_RENDERING_OBSERVERS)) {
-      UpdateGraphic();
-    }
-
-  } else if (aFlags & COORD_CONTEXT_CHANGED) {
-    nsSVGForeignObjectElement *fO =
-      static_cast<nsSVGForeignObjectElement*>(mContent);
-    // Coordinate context changes affect mCanvasTM if we have a
-    // percentage 'x' or 'y'
-    if (fO->mLengthAttributes[nsSVGForeignObjectElement::X].IsPercentage() ||
-        fO->mLengthAttributes[nsSVGForeignObjectElement::Y].IsPercentage()) {
-      mCanvasTM = nsnull;
-    }
-    // Our coordinate context's width/height has changed. If we have a
-    // percentage width/height our dimensions will change so we must reflow.
-    if (fO->mLengthAttributes[nsSVGForeignObjectElement::WIDTH].IsPercentage() ||
-        fO->mLengthAttributes[nsSVGForeignObjectElement::HEIGHT].IsPercentage()) {
-      reflow = true;
-    }
-  }
-
-  if (reflow) {
-    // If we're called while the PresShell is handling reflow events then we
-    // must have been called as a result of the NotifyViewportChange() call in
-    // our nsSVGOuterSVGFrame's Reflow() method. We must not call RequestReflow
-    // at this point (i.e. during reflow) because it could confuse the
-    // PresShell and prevent it from reflowing us properly in future. Besides
-    // that, nsSVGOuterSVGFrame::DidReflow will take care of reflowing us
-    // synchronously, so there's no need.
-    if (!PresContext()->PresShell()->IsReflowLocked()) {
-      UpdateGraphic(); // update mRect before requesting reflow
-      RequestReflow(nsIPresShell::eResize);
-    }
   }
 }
 
-void
-nsSVGForeignObjectFrame::NotifyRedrawSuspended()
-{
-  AddStateBits(NS_STATE_SVG_REDRAW_SUSPENDED);
-}
-
-void
-nsSVGForeignObjectFrame::NotifyRedrawUnsuspended()
-{
-  RemoveStateBits(NS_STATE_SVG_REDRAW_SUSPENDED);
-
-  if (!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD)) {
-    if (GetStateBits() & NS_STATE_SVG_DIRTY) {
-      UpdateGraphic(); // invalidate our entire area
-    } else {
-      FlushDirtyRegion(0); // only invalidate areas dirtied by our descendants
-    }
-  }
-}
-
-gfxRect
+SVGBBox
 nsSVGForeignObjectFrame::GetBBoxContribution(const gfxMatrix &aToBBoxUserspace,
                                              PRUint32 aFlags)
 {
@@ -483,7 +510,7 @@ nsSVGForeignObjectFrame::GetBBoxContribution(const gfxMatrix &aToBBoxUserspace,
 
   if (aToBBoxUserspace.IsSingular()) {
     // XXX ReportToConsole
-    return gfxRect(0.0, 0.0, 0.0, 0.0);
+    return SVGBBox();
   }
   return aToBBoxUserspace.TransformBounds(gfxRect(0.0, 0.0, w, h));
 }
@@ -522,7 +549,7 @@ nsSVGForeignObjectFrame::GetCanvasTMForChildren()
 void nsSVGForeignObjectFrame::RequestReflow(nsIPresShell::IntrinsicDirty aType)
 {
   if (GetStateBits() & NS_FRAME_FIRST_REFLOW)
-    // If we haven't had an InitialUpdate yet, nothing to do.
+    // If we haven't had a UpdateBounds yet, nothing to do.
     return;
 
   nsIFrame* kid = GetFirstPrincipalChild();
@@ -532,63 +559,14 @@ void nsSVGForeignObjectFrame::RequestReflow(nsIPresShell::IntrinsicDirty aType)
   PresContext()->PresShell()->FrameNeedsReflow(kid, aType, NS_FRAME_IS_DIRTY);
 }
 
-void nsSVGForeignObjectFrame::UpdateGraphic()
-{
-  nsSVGUtils::UpdateGraphic(this);
-
-  // We just invalidated our entire area, so clear the caches of areas dirtied
-  // by our descendants:
-  mSameDocDirtyRegion.SetEmpty();
-  mSubDocDirtyRegion.SetEmpty();
-}
-
-void
-nsSVGForeignObjectFrame::MaybeReflowFromOuterSVGFrame()
-{
-  // If IsDisabled() is true, then we know that our DoReflow() call will return
-  // early, leaving us with a marked-dirty but not-reflowed kid. That'd be bad;
-  // it'd mean that all future calls to this method would be doomed to take the
-  // NS_FRAME_IS_DIRTY early-return below. To avoid that problem, we need to
-  // bail out *before* we mark our kid as dirty.
-  if (IsDisabled()) {
-    return;
-  }
-
-  nsIFrame* kid = GetFirstPrincipalChild();
-
-  // If we're already scheduled to reflow (if we or our kid is dirty) we don't
-  // want to reflow now or else our presShell will do extra work trying to
-  // reflow us a second time. (It will also complain if it finds that a reflow
-  // root scheduled for reflow isn't dirty).
-
-  if (kid->GetStateBits() & NS_FRAME_IS_DIRTY) {
-    return;
-  }
-  kid->AddStateBits(NS_FRAME_IS_DIRTY); // we must be fully marked dirty
-  if (kid->GetStateBits() & NS_FRAME_HAS_DIRTY_CHILDREN) {
-    return;
-  }
-
-  // Make sure to not allow interrupts if we're not being reflown as a root
-  nsPresContext::InterruptPreventer noInterrupts(PresContext());
-  DoReflow();
-}
-
 void
 nsSVGForeignObjectFrame::DoReflow()
 {
-  NS_ASSERTION(!(nsSVGUtils::GetOuterSVGFrame(this)->
-                             GetStateBits() & NS_FRAME_FIRST_REFLOW),
-               "Calling InitialUpdate too early - must not call DoReflow!!!");
-
   // Skip reflow if we're zero-sized, unless this is our first reflow.
   if (IsDisabled() &&
       !(GetStateBits() & NS_FRAME_FIRST_REFLOW))
     return;
 
-  if (GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD)
-    return;
-
   nsPresContext *presContext = PresContext();
   nsIFrame* kid = GetFirstPrincipalChild();
   if (!kid)
@@ -602,48 +580,36 @@ nsSVGForeignObjectFrame::DoReflow()
   if (!renderingContext)
     return;
 
-  nsSVGForeignObjectElement *fO = static_cast<nsSVGForeignObjectElement*>
-                                             (mContent);
-
-  float width =
-    fO->mLengthAttributes[nsSVGForeignObjectElement::WIDTH].GetAnimValue(fO);
-  float height =
-    fO->mLengthAttributes[nsSVGForeignObjectElement::HEIGHT].GetAnimValue(fO);
-
-  // Clamp height & width to be non-negative (to match UpdateCoveredRegion).
-  width = NS_MAX(width, 0.0f);
-  height = NS_MAX(height, 0.0f);
-
-  nsSize size(nsPresContext::CSSPixelsToAppUnits(width),
-              nsPresContext::CSSPixelsToAppUnits(height));
-
   mInReflow = true;
 
   nsHTMLReflowState reflowState(presContext, kid,
                                 renderingContext,
-                                nsSize(size.width, NS_UNCONSTRAINEDSIZE));
+                                nsSize(mRect.width, NS_UNCONSTRAINEDSIZE));
   nsHTMLReflowMetrics desiredSize;
   nsReflowStatus status;
 
-  // We don't use size.height above because that tells the child to do
+  // We don't use mRect.height above because that tells the child to do
   // page/column breaking at that height.
   NS_ASSERTION(reflowState.mComputedBorderPadding == nsMargin(0, 0, 0, 0) &&
                reflowState.mComputedMargin == nsMargin(0, 0, 0, 0),
                "style system should ensure that :-moz-svg-foreign-content "
                "does not get styled");
-  NS_ASSERTION(reflowState.ComputedWidth() == size.width,
+  NS_ASSERTION(reflowState.ComputedWidth() == mRect.width,
                "reflow state made child wrong size");
-  reflowState.SetComputedHeight(size.height);
-  
+  reflowState.SetComputedHeight(mRect.height);
+
   ReflowChild(kid, presContext, desiredSize, reflowState, 0, 0,
               NS_FRAME_NO_MOVE_FRAME, status);
-  NS_ASSERTION(size.width == desiredSize.width &&
-               size.height == desiredSize.height, "unexpected size");
+  NS_ASSERTION(mRect.width == desiredSize.width &&
+               mRect.height == desiredSize.height, "unexpected size");
   FinishReflowChild(kid, presContext, &reflowState, desiredSize, 0, 0,
                     NS_FRAME_NO_MOVE_FRAME);
   
   mInReflow = false;
-  FlushDirtyRegion(0);
+
+  if (!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD)) {
+    FlushDirtyRegion(0);
+  }
 }
 
 void
@@ -672,10 +638,15 @@ nsSVGForeignObjectFrame::InvalidateDirtyRect(nsSVGOuterSVGFrame* aOuter,
 void
 nsSVGForeignObjectFrame::FlushDirtyRegion(PRUint32 aFlags)
 {
-  if ((mSameDocDirtyRegion.IsEmpty() && mSubDocDirtyRegion.IsEmpty()) ||
-      mInReflow ||
-      (GetStateBits() & NS_STATE_SVG_REDRAW_SUSPENDED))
+  NS_ABORT_IF_FALSE(!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+                    "Should not have been called");
+
+  NS_ASSERTION(!mInReflow,
+               "We shouldn't be flushing while we have a pending flush");
+
+  if (mSameDocDirtyRegion.IsEmpty() && mSubDocDirtyRegion.IsEmpty()) {
     return;
+  }
 
   nsSVGOuterSVGFrame *outerSVGFrame = nsSVGUtils::GetOuterSVGFrame(this);
   if (!outerSVGFrame) {
