@@ -36,25 +36,26 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsSVGTextFrame.h"
-#include "mozilla/LookAndFeel.h"
-#include "nsTextFragment.h"
-#include "nsBidiPresUtils.h"
-#include "nsSVGUtils.h"
-#include "SVGLengthList.h"
-#include "nsIDOMSVGLength.h"
-#include "nsIDOMSVGRect.h"
-#include "DOMSVGPoint.h"
+// Main header first:
 #include "nsSVGGlyphFrame.h"
-#include "nsSVGTextPathFrame.h"
-#include "nsSVGPathElement.h"
-#include "nsSVGRect.h"
-#include "nsDOMError.h"
+
+// Keep others in (case-insensitive) order:
+#include "DOMSVGPoint.h"
 #include "gfxContext.h"
 #include "gfxMatrix.h"
 #include "gfxPlatform.h"
+#include "mozilla/LookAndFeel.h"
+#include "nsBidiPresUtils.h"
+#include "nsDOMError.h"
+#include "nsIDOMSVGRect.h"
+#include "nsRenderingContext.h"
 #include "nsSVGEffects.h"
 #include "nsSVGPaintServerFrame.h"
+#include "nsSVGRect.h"
+#include "nsSVGTextPathFrame.h"
+#include "nsSVGUtils.h"
+#include "nsTextFragment.h"
+#include "SVGLengthList.h"
 
 using namespace mozilla;
 
@@ -254,11 +255,18 @@ NS_QUERYFRAME_TAIL_INHERITING(nsSVGGlyphFrameBase)
 NS_IMETHODIMP
 nsSVGGlyphFrame::CharacterDataChanged(CharacterDataChangeInfo* aInfo)
 {
-  ClearTextRun();
+  // NotifyGlyphMetricsChange takes care of calling
+  // nsSVGUtils::InvalidateAndScheduleBoundsUpdate on the appropriate frames.
+
   NotifyGlyphMetricsChange();
+
+  ClearTextRun();
   if (IsTextEmpty()) {
-    // That's it for this frame. Leave no trace we were here
-    nsSVGUtils::UpdateGraphic(this);
+    // The one time that NotifyGlyphMetricsChange fails to call
+    // nsSVGUtils::InvalidateAndScheduleBoundsUpdate properly is when all our
+    // text is gone, since it skips empty frames. So we have to invalidate
+    // ourself.
+    nsSVGUtils::InvalidateBounds(this);
   }
 
   return NS_OK;
@@ -445,9 +453,15 @@ nsSVGGlyphFrame::GetCoveredRegion()
   return mCoveredRegion;
 }
 
-NS_IMETHODIMP
-nsSVGGlyphFrame::UpdateCoveredRegion()
+void
+nsSVGGlyphFrame::UpdateBounds()
 {
+  NS_ASSERTION(nsSVGUtils::OuterSVGIsCallingUpdateBounds(this),
+               "This call is probaby a wasteful mistake");
+
+  NS_ABORT_IF_FALSE(!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+                    "UpdateBounds mechanism not designed for this");
+
   mRect.SetEmpty();
 
   // XXX here we have tmpCtx use its default identity matrix, but does this
@@ -458,7 +472,7 @@ nsSVGGlyphFrame::UpdateCoveredRegion()
   if (hasStroke) {
     SetupCairoStrokeGeometry(tmpCtx);
   } else if (GetStyleSVG()->mFill.mType == eStyleSVGPaintType_None) {
-    return NS_OK;
+    return;
   }
 
   CharacterIterator iter(this, true);
@@ -498,24 +512,15 @@ nsSVGGlyphFrame::UpdateCoveredRegion()
   mCoveredRegion = nsSVGUtils::TransformFrameRectToOuterSVG(
     mRect, GetCanvasTM(), PresContext());
 
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsSVGGlyphFrame::InitialUpdate()
-{
-  NS_ASSERTION(GetStateBits() & NS_FRAME_FIRST_REFLOW,
-               "Yikes! We've been called already! Hopefully we weren't called "
-               "before our nsSVGOuterSVGFrame's initial Reflow()!!!");
-
-  NS_ASSERTION(!(mState & NS_FRAME_IN_REFLOW),
-               "We don't actually participate in reflow");
-
-  // Do unset the various reflow bits, though.
   mState &= ~(NS_FRAME_FIRST_REFLOW | NS_FRAME_IS_DIRTY |
               NS_FRAME_HAS_DIRTY_CHILDREN);
-  
-  return NS_OK;
+
+  if (!(GetParent()->GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
+    // We only invalidate if our outer-<svg> has already had its
+    // initial reflow (since if it hasn't, its entire area will be
+    // invalidated when it gets that initial reflow):
+    nsSVGUtils::InvalidateBounds(this, true);
+  }
 }  
 
 void
@@ -528,27 +533,15 @@ nsSVGGlyphFrame::NotifySVGChanged(PRUint32 aFlags)
   NS_ABORT_IF_FALSE(aFlags & (TRANSFORM_CHANGED | COORD_CONTEXT_CHANGED),
                     "Invalidation logic may need adjusting");
 
+  // XXXjwatt: seems to me that this could change the glyph metrics,
+  // in which case we should call NotifyGlyphMetricsChange instead.
+  if (!(aFlags & DO_NOT_NOTIFY_RENDERING_OBSERVERS)) {
+    nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
+  }
+
   if (aFlags & TRANSFORM_CHANGED) {
     ClearTextRun();
   }
-  if (!(aFlags & DO_NOT_NOTIFY_RENDERING_OBSERVERS)) {
-    nsSVGUtils::UpdateGraphic(this);
-  }
-}
-
-void
-nsSVGGlyphFrame::NotifyRedrawSuspended()
-{
-  AddStateBits(NS_STATE_SVG_REDRAW_SUSPENDED);
-}
-
-void
-nsSVGGlyphFrame::NotifyRedrawUnsuspended()
-{
-  RemoveStateBits(NS_STATE_SVG_REDRAW_SUSPENDED);
-
-  if (GetStateBits() & NS_STATE_SVG_DIRTY)
-    nsSVGUtils::UpdateGraphic(this);
 }
 
 void
@@ -597,7 +590,7 @@ nsSVGGlyphFrame::DrawCharacters(CharacterIterator *aIter,
   }
 }
 
-gfxRect
+SVGBBox
 nsSVGGlyphFrame::GetBBoxContribution(const gfxMatrix &aToBBoxUserspace,
                                      PRUint32 aFlags)
 {
