@@ -34,23 +34,23 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsSVGPathGeometryFrame.h"
-#include "imgIContainer.h"
-#include "nsStubImageDecoderObserver.h"
-#include "nsImageLoadingContent.h"
-#include "nsIDOMSVGImageElement.h"
-#include "nsLayoutUtils.h"
-#include "nsSVGImageElement.h"
-#include "nsSVGUtils.h"
+// Keep in (case-insensitive) order:
 #include "gfxContext.h"
 #include "gfxMatrix.h"
-#include "nsIInterfaceRequestorUtils.h"
 #include "gfxPlatform.h"
+#include "imgIContainer.h"
+#include "nsIDOMSVGImageElement.h"
+#include "nsLayoutUtils.h"
+#include "nsRenderingContext.h"
+#include "nsStubImageDecoderObserver.h"
+#include "nsSVGEffects.h"
+#include "nsSVGImageElement.h"
+#include "nsSVGPathGeometryFrame.h"
 #include "nsSVGSVGElement.h"
+#include "nsSVGUtils.h"
 
 using namespace mozilla;
 
-class nsRenderingContext;
 class nsSVGImageFrame;
 
 class nsSVGImageListener MOZ_FINAL : public nsStubImageDecoderObserver
@@ -93,9 +93,9 @@ public:
   // nsISVGChildFrame interface:
   NS_IMETHOD PaintSVG(nsRenderingContext *aContext, const nsIntRect *aDirtyRect);
   NS_IMETHOD_(nsIFrame*) GetFrameForPoint(const nsPoint &aPoint);
+  virtual void UpdateBounds();
 
   // nsSVGPathGeometryFrame methods:
-  NS_IMETHOD UpdateCoveredRegion();
   virtual PRUint16 GetHitTestFlags();
 
   // nsIFrame interface:
@@ -218,14 +218,18 @@ nsSVGImageFrame::AttributeChanged(PRInt32         aNameSpaceID,
                                   nsIAtom*        aAttribute,
                                   PRInt32         aModType)
 {
-  if (aNameSpaceID == kNameSpaceID_None &&
-      (aAttribute == nsGkAtoms::x ||
-       aAttribute == nsGkAtoms::y ||
-       aAttribute == nsGkAtoms::width ||
-       aAttribute == nsGkAtoms::height ||
-       aAttribute == nsGkAtoms::preserveAspectRatio)) {
-    nsSVGUtils::UpdateGraphic(this);
-    return NS_OK;
+  if (aNameSpaceID == kNameSpaceID_None) {
+    if (aAttribute == nsGkAtoms::x ||
+        aAttribute == nsGkAtoms::y ||
+        aAttribute == nsGkAtoms::width ||
+        aAttribute == nsGkAtoms::height) {
+      nsSVGUtils::InvalidateAndScheduleBoundsUpdate(this);
+      return NS_OK;
+    }
+    else if (aAttribute == nsGkAtoms::preserveAspectRatio) {
+      nsSVGUtils::InvalidateBounds(this);
+      return NS_OK;
+    }
   }
   if (aNameSpaceID == kNameSpaceID_XLink &&
       aAttribute == nsGkAtoms::href) {
@@ -479,10 +483,18 @@ nsSVGImageFrame::GetType() const
 
 // Lie about our fill/stroke so that covered region and hit detection work properly
 
-NS_IMETHODIMP
-nsSVGImageFrame::UpdateCoveredRegion()
+void
+nsSVGImageFrame::UpdateBounds()
 {
-  mRect.SetEmpty();
+  NS_ASSERTION(nsSVGUtils::OuterSVGIsCallingUpdateBounds(this),
+               "This call is probaby a wasteful mistake");
+
+  NS_ABORT_IF_FALSE(!(GetStateBits() & NS_STATE_SVG_NONDISPLAY_CHILD),
+                    "UpdateBounds mechanism not designed for this");
+
+  if (!nsSVGUtils::NeedsUpdatedBounds(this)) {
+    return;
+  }
 
   gfxContext context(gfxPlatform::GetPlatform()->ScreenReferenceSurface());
 
@@ -494,13 +506,23 @@ nsSVGImageFrame::UpdateCoveredRegion()
   if (!extent.IsEmpty()) {
     mRect = nsLayoutUtils::RoundGfxRectToAppRect(extent, 
               PresContext()->AppUnitsPerCSSPixel());
+  } else {
+    mRect.SetEmpty();
   }
 
   // See bug 614732 comment 32.
   mCoveredRegion = nsSVGUtils::TransformFrameRectToOuterSVG(
     mRect, GetCanvasTM(), PresContext());
 
-  return NS_OK;
+  mState &= ~(NS_FRAME_FIRST_REFLOW | NS_FRAME_IS_DIRTY |
+              NS_FRAME_HAS_DIRTY_CHILDREN);
+
+  if (!(GetParent()->GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
+    // We only invalidate if our outer-<svg> has already had its
+    // initial reflow (since if it hasn't, its entire area will be
+    // invalidated when it gets that initial reflow):
+    nsSVGUtils::InvalidateBounds(this, true);
+  }
 }
 
 PRUint16
@@ -560,7 +582,7 @@ NS_IMETHODIMP nsSVGImageListener::OnStopDecode(imgIRequest *aRequest,
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsSVGUtils::UpdateGraphic(mFrame);
+  nsSVGUtils::InvalidateAndScheduleBoundsUpdate(mFrame);
   return NS_OK;
 }
 
@@ -571,18 +593,23 @@ NS_IMETHODIMP nsSVGImageListener::FrameChanged(imgIRequest *aRequest,
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsSVGUtils::UpdateGraphic(mFrame);
+  // No new dimensions, so we don't need to call
+  // nsSVGUtils::InvalidateAndScheduleBoundsUpdate.
+  nsSVGEffects::InvalidateRenderingObservers(mFrame);
+  nsSVGUtils::InvalidateBounds(mFrame);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsSVGImageListener::OnStartContainer(imgIRequest *aRequest,
                                                    imgIContainer *aContainer)
 {
+  // Called once the resource's dimensions have been obtained.
+
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
   mFrame->mImageContainer = aContainer;
-  nsSVGUtils::UpdateGraphic(mFrame);
+  nsSVGUtils::InvalidateAndScheduleBoundsUpdate(mFrame);
 
   return NS_OK;
 }
