@@ -35,20 +35,21 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+// Main header first:
 #include "nsSVGIntegrationUtils.h"
 
-#include "nsSVGUtils.h"
-#include "nsSVGEffects.h"
-#include "nsRegion.h"
-#include "nsLayoutUtils.h"
-#include "nsDisplayList.h"
-#include "nsSVGFilterPaintCallback.h"
-#include "nsSVGFilterFrame.h"
-#include "nsSVGClipPathFrame.h"
-#include "nsSVGMaskFrame.h"
-#include "gfxPlatform.h"
+// Keep others in (case-insensitive) order:
 #include "gfxDrawable.h"
+#include "nsDisplayList.h"
+#include "nsLayoutUtils.h"
+#include "nsRenderingContext.h"
+#include "nsSVGClipPathFrame.h"
+#include "nsSVGEffects.h"
+#include "nsSVGFilterFrame.h"
+#include "nsSVGFilterPaintCallback.h"
+#include "nsSVGMaskFrame.h"
 #include "nsSVGPaintServerFrame.h"
+#include "nsSVGUtils.h"
 
 // ----------------------------------------------------------------------
 
