@@ -36,25 +36,24 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+// Main header first:
 #include "nsSVGPatternFrame.h"
 
-#include "nsGkAtoms.h"
-#include "nsIDOMSVGAnimatedRect.h"
-#include "SVGAnimatedTransformList.h"
-#include "nsStyleContext.h"
-#include "nsINameSpaceManager.h"
-#include "nsISVGChildFrame.h"
-#include "nsSVGRect.h"
-#include "nsSVGUtils.h"
-#include "nsSVGEffects.h"
-#include "nsSVGOuterSVGFrame.h"
-#include "nsSVGPatternElement.h"
-#include "nsSVGGeometryFrame.h"
+// Keep others in (case-insensitive) order:
 #include "gfxContext.h"
-#include "gfxPlatform.h"
-#include "gfxPattern.h"
 #include "gfxMatrix.h"
+#include "gfxPattern.h"
+#include "gfxPlatform.h"
 #include "nsContentUtils.h"
+#include "nsGkAtoms.h"
+#include "nsISVGChildFrame.h"
+#include "nsRenderingContext.h"
+#include "nsStyleContext.h"
+#include "nsSVGEffects.h"
+#include "nsSVGGeometryFrame.h"
+#include "nsSVGPatternElement.h"
+#include "nsSVGUtils.h"
+#include "SVGAnimatedTransformList.h"
 
 using namespace mozilla;
 
@@ -581,11 +580,15 @@ nsSVGPatternFrame::ConstructCTM(const gfxRect &callerBBox,
     tCTM.Scale(scale, scale);
   }
 
-  const nsSVGViewBoxRect viewBox = GetViewBox().GetAnimValue();
-
-  if (viewBox.height <= 0.0f || viewBox.width <= 0.0f) {
+  const nsSVGViewBox& viewBox = GetViewBox();
+  if (!viewBox.IsValid()) {
     return tCTM;
   }
+  const nsSVGViewBoxRect viewBoxRect = GetViewBox().GetAnimValue();
+
+  if (viewBoxRect.height <= 0.0f || viewBoxRect.width <= 0.0f) {
+    return gfxMatrix(0.0, 0.0, 0.0, 0.0, 0.0, 0.0); // singular
+  }
 
   float viewportWidth, viewportHeight;
   if (targetContent->IsSVG()) {
@@ -603,11 +606,16 @@ nsSVGPatternFrame::ConstructCTM(const gfxRect &callerBBox,
     viewportHeight =
       GetLengthValue(nsSVGPatternElement::HEIGHT)->GetAnimValue(aTarget);
   }
+
+  if (viewportWidth <= 0.0f || viewportHeight <= 0.0f) {
+    return gfxMatrix(0.0, 0.0, 0.0, 0.0, 0.0, 0.0); // singular
+  }
+
   gfxMatrix tm = nsSVGUtils::GetViewBoxTransform(
     static_cast<nsSVGPatternElement*>(mContent),
     viewportWidth, viewportHeight,
-    viewBox.x, viewBox.y,
-    viewBox.width, viewBox.height,
+    viewBoxRect.x, viewBoxRect.y,
+    viewBoxRect.width, viewBoxRect.height,
     GetPreserveAspectRatio());
 
   return tm * tCTM;
