@@ -255,8 +255,8 @@ public class AwesomeBarTabs extends TabHost {
                 return c.getString(c.getColumnIndexOrThrow(Bookmarks.TITLE));
 
             // Use localized strings for special folder names.
-            if (guid.equals(Bookmarks.MOBILE_FOLDER_GUID))
-                return mResources.getString(R.string.bookmarks_folder_mobile);
+            if (guid.equals(Bookmarks.FAKE_DESKTOP_FOLDER_GUID))
+                return mResources.getString(R.string.bookmarks_folder_desktop);
             else if (guid.equals(Bookmarks.MENU_FOLDER_GUID))
                 return mResources.getString(R.string.bookmarks_folder_menu);
             else if (guid.equals(Bookmarks.TOOLBAR_FOLDER_GUID))
@@ -356,41 +356,48 @@ public class AwesomeBarTabs extends TabHost {
         }
 
         @Override
-        protected void onPostExecute(Cursor cursor) {
-            ListView list = (ListView) findViewById(R.id.bookmarks_list);
-            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
-                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
-                    handleBookmarkItemClick(parent, view, position, id);
+        protected void onPostExecute(final Cursor cursor) {
+            final ListView list = (ListView) findViewById(R.id.bookmarks_list);
+
+            // Hack: force this to the main thread, even though it should already be on it
+            GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
+                public void run() {
+                    list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
+                        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
+                            handleBookmarkItemClick(parent, view, position, id);
+                        }
+                    });
+                    
+                    // We need to add the header before we set the adapter, hence make it null
+                    list.setAdapter(null);
+
+                    if (mBookmarksAdapter == null) {
+                        mBookmarksAdapter = new BookmarksListAdapter(mContext, cursor);
+                    } else {
+                        mBookmarksAdapter.changeCursor(cursor);
+                    }
+
+                    LinearLayout headerView = mBookmarksAdapter.getHeaderView();
+                    if (headerView == null) {
+                        headerView = (LinearLayout) mInflater.inflate(R.layout.awesomebar_header_row, null);
+                        mBookmarksAdapter.setHeaderView(headerView);
+                    }
+
+                    // Add/Remove header based on the root folder
+                    if (mFolderId == Bookmarks.FIXED_ROOT_ID) {
+                        if (list.getHeaderViewsCount() == 1)
+                            list.removeHeaderView(headerView);
+                    } else {
+                        if (list.getHeaderViewsCount() == 0)
+                            list.addHeaderView(headerView, null, true);
+
+                        ((TextView) headerView.findViewById(R.id.title)).setText(mFolderTitle);
+                    }
+
+                    list.setAdapter(mBookmarksAdapter);
                 }
             });
-            
-            // We need to add the header before we set the adapter, hence make it null
-            list.setAdapter(null);
 
-            if (mBookmarksAdapter == null) {
-                mBookmarksAdapter = new BookmarksListAdapter(mContext, cursor);
-            } else {
-                mBookmarksAdapter.changeCursor(cursor);
-            }
-
-            LinearLayout headerView = mBookmarksAdapter.getHeaderView();
-            if (headerView == null) {
-                headerView = (LinearLayout) mInflater.inflate(R.layout.awesomebar_header_row, null);
-                mBookmarksAdapter.setHeaderView(headerView);
-            }
-
-            // Add/Remove header based on the root folder
-            if (mFolderId == Bookmarks.FIXED_ROOT_ID) {
-                if (list.getHeaderViewsCount() == 1)
-                    list.removeHeaderView(headerView);
-            } else {
-                if (list.getHeaderViewsCount() == 0)
-                    list.addHeaderView(headerView, null, true);
-
-                ((TextView) headerView.findViewById(R.id.title)).setText(mFolderTitle);
-            }
-
-            list.setAdapter(mBookmarksAdapter);
             mBookmarksQueryTask = null;
         }
     }
@@ -562,28 +569,32 @@ public class AwesomeBarTabs extends TabHost {
             final ExpandableListView historyList =
                     (ExpandableListView) findViewById(R.id.history_list);
 
-            historyList.setOnChildClickListener(new ExpandableListView.OnChildClickListener() {
-                public boolean onChildClick(ExpandableListView parent, View view,
-                        int groupPosition, int childPosition, long id) {
-                    handleHistoryItemClick(groupPosition, childPosition);
-                    return true;
+            // Hack: force this to the main thread, even though it should already be on it
+            GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
+                public void run() {
+                    historyList.setOnChildClickListener(new ExpandableListView.OnChildClickListener() {
+                        public boolean onChildClick(ExpandableListView parent, View view,
+                                int groupPosition, int childPosition, long id) {
+                            handleHistoryItemClick(groupPosition, childPosition);
+                            return true;
+                        }
+                    });
+
+                    // This is to disallow collapsing the expandable groups in the
+                    // history expandable list view to mimic simpler sections. We should
+                    // Remove this if we decide to allow expanding/collapsing groups.
+                    historyList.setOnGroupClickListener(new ExpandableListView.OnGroupClickListener() {
+                        public boolean onGroupClick(ExpandableListView parent, View v,
+                                int groupPosition, long id) {
+                            return true;
+                        }
+                    });
+
+                    historyList.setAdapter(mHistoryAdapter);
+                    expandAllGroups(historyList);
                 }
             });
 
-            // This is to disallow collapsing the expandable groups in the
-            // history expandable list view to mimic simpler sections. We should
-            // Remove this if we decide to allow expanding/collapsing groups.
-            historyList.setOnGroupClickListener(new ExpandableListView.OnGroupClickListener() {
-                public boolean onGroupClick(ExpandableListView parent, View v,
-                        int groupPosition, long id) {
-                    return true;
-                }
-            });
-
-            historyList.setAdapter(mHistoryAdapter);
-
-            expandAllGroups(historyList);
-
             mHistoryQueryTask = null;
         }
     }
@@ -728,13 +739,6 @@ public class AwesomeBarTabs extends TabHost {
         // to the TabHost.
         setup();
 
-        mListTouchListener = new View.OnTouchListener() {
-            public boolean onTouch(View view, MotionEvent event) {
-                hideSoftInput(view);
-                return false;
-            }
-        };
-
         addAllPagesTab();
         addBookmarksTab();
         addHistoryTab();
@@ -823,7 +827,6 @@ public class AwesomeBarTabs extends TabHost {
         });
 
         allPagesList.setAdapter(mAllPagesCursorAdapter);
-        allPagesList.setOnTouchListener(mListTouchListener);
     }
 
     private void addBookmarksTab() {
@@ -834,7 +837,6 @@ public class AwesomeBarTabs extends TabHost {
                       R.id.bookmarks_list);
 
         ListView bookmarksList = (ListView) findViewById(R.id.bookmarks_list);
-        bookmarksList.setOnTouchListener(mListTouchListener);
 
         // Only load bookmark list when tab is actually used.
         // See OnTabChangeListener above.
@@ -848,7 +850,6 @@ public class AwesomeBarTabs extends TabHost {
                       R.id.history_list);
 
         ListView historyList = (ListView) findViewById(R.id.history_list);
-        historyList.setOnTouchListener(mListTouchListener);
 
         // Only load history list when tab is actually used.
         // See OnTabChangeListener above.
@@ -990,4 +991,13 @@ public class AwesomeBarTabs extends TabHost {
             }
         });
     }
+
+    @Override
+    public boolean onInterceptTouchEvent(MotionEvent ev) {
+        hideSoftInput(this);
+
+        // the android docs make no sense, but returning false will cause this and other
+        // motion events to be sent to the view the user tapped on
+        return false;
+    }
 }
