@@ -5,12 +5,17 @@
 package org.mozilla.gecko.sync;
 
 import java.io.IOException;
-import java.io.UnsupportedEncodingException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicLong;
 
 import org.json.simple.parser.ParseException;
@@ -35,19 +40,21 @@ import org.mozilla.gecko.sync.stage.AndroidBrowserHistoryServerSyncStage;
 import org.mozilla.gecko.sync.stage.CheckPreconditionsStage;
 import org.mozilla.gecko.sync.stage.CompletedStage;
 import org.mozilla.gecko.sync.stage.EnsureClusterURLStage;
-import org.mozilla.gecko.sync.stage.EnsureKeysStage;
+import org.mozilla.gecko.sync.stage.EnsureCrypto5KeysStage;
 import org.mozilla.gecko.sync.stage.FennecTabsServerSyncStage;
 import org.mozilla.gecko.sync.stage.FetchInfoCollectionsStage;
 import org.mozilla.gecko.sync.stage.FetchMetaGlobalStage;
+import org.mozilla.gecko.sync.stage.FormHistoryServerSyncStage;
 import org.mozilla.gecko.sync.stage.GlobalSyncStage;
 import org.mozilla.gecko.sync.stage.GlobalSyncStage.Stage;
 import org.mozilla.gecko.sync.stage.NoSuchStageException;
+import org.mozilla.gecko.sync.stage.PasswordsServerSyncStage;
 import org.mozilla.gecko.sync.stage.SyncClientsEngineStage;
+import org.mozilla.gecko.sync.stage.UploadMetaGlobalStage;
 
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.os.Bundle;
-import android.util.Log;
 import ch.boye.httpclientandroidlib.HttpResponse;
 
 public class GlobalSession implements CredentialsSource, PrefsSource, HttpResponseObserver {
@@ -65,19 +72,16 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   private Context context;
   private ClientsDataDelegate clientsDelegate;
 
+  /**
+   * Map from engine name to new settings for an updated meta/global record.
+   */
+  public final Map<String, EngineSettings> enginesToUpdate = new HashMap<String, EngineSettings>();
+
   /*
    * Key accessors.
    */
-  public void setCollectionKeys(CollectionKeys k) {
-    config.setCollectionKeys(k);
-  }
-  @Override
-  public CollectionKeys getCollectionKeys() {
-    return config.collectionKeys;
-  }
-  @Override
-  public KeyBundle keyForCollection(String collection) throws NoCollectionKeysSetException {
-    return config.keyForCollection(collection);
+  public KeyBundle keyBundleForCollection(String collection) throws NoCollectionKeysSetException {
+    return config.getCollectionKeys().keyBundleForCollection(collection);
   }
 
   /*
@@ -158,6 +162,8 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
     registerCommands();
     prepareStages();
+
+    // TODO: data-driven plan for the sync, referring to prepareStages.
   }
 
   protected void registerCommands() {
@@ -166,14 +172,32 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     processor.registerCommand("resetEngine", new CommandRunner() {
       @Override
       public void executeCommand(List<String> args) {
-        resetClient(new String[] { args.get(0) });
+        HashSet<String> names = new HashSet<String>();
+        names.add(args.get(0));
+        resetStagesByName(names);
       }
     });
 
     processor.registerCommand("resetAll", new CommandRunner() {
       @Override
       public void executeCommand(List<String> args) {
-        resetClient(null);
+        resetAllStages();
+      }
+    });
+
+    processor.registerCommand("wipeEngine", new CommandRunner() {
+      @Override
+      public void executeCommand(List<String> args) {
+        HashSet<String> names = new HashSet<String>();
+        names.add(args.get(0));
+        wipeStagesByName(names);
+      }
+    });
+
+    processor.registerCommand("wipeAll", new CommandRunner() {
+      @Override
+      public void executeCommand(List<String> args) {
+        wipeAllStages();
       }
     });
 
@@ -186,22 +210,32 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   }
 
   protected void prepareStages() {
-    stages = new HashMap<Stage, GlobalSyncStage>();
-    stages.put(Stage.checkPreconditions,      new CheckPreconditionsStage());
-    stages.put(Stage.ensureClusterURL,        new EnsureClusterURLStage());
-    stages.put(Stage.fetchInfoCollections,    new FetchInfoCollectionsStage());
-    stages.put(Stage.fetchMetaGlobal,         new FetchMetaGlobalStage());
-    stages.put(Stage.ensureKeysStage,         new EnsureKeysStage());
-    stages.put(Stage.syncClientsEngine,       new SyncClientsEngineStage());
+    HashMap<Stage, GlobalSyncStage> stages = new HashMap<Stage, GlobalSyncStage>();
 
-    // TODO: more stages.
-    stages.put(Stage.syncTabs,                new FennecTabsServerSyncStage());
-    stages.put(Stage.syncBookmarks,           new AndroidBrowserBookmarksServerSyncStage());
-    stages.put(Stage.syncHistory,             new AndroidBrowserHistoryServerSyncStage());
-    stages.put(Stage.completed,               new CompletedStage());
+    stages.put(Stage.checkPreconditions,      new CheckPreconditionsStage(this));
+    stages.put(Stage.ensureClusterURL,        new EnsureClusterURLStage(this));
+    stages.put(Stage.fetchInfoCollections,    new FetchInfoCollectionsStage(this));
+    stages.put(Stage.fetchMetaGlobal,         new FetchMetaGlobalStage(this));
+    stages.put(Stage.ensureKeysStage,         new EnsureCrypto5KeysStage(this));
+    stages.put(Stage.syncClientsEngine,       new SyncClientsEngineStage(this));
+
+    stages.put(Stage.syncTabs,                new FennecTabsServerSyncStage(this));
+    stages.put(Stage.syncPasswords,           new PasswordsServerSyncStage(this));
+    stages.put(Stage.syncBookmarks,           new AndroidBrowserBookmarksServerSyncStage(this));
+    stages.put(Stage.syncHistory,             new AndroidBrowserHistoryServerSyncStage(this));
+    stages.put(Stage.syncFormHistory,         new FormHistoryServerSyncStage(this));
+
+    stages.put(Stage.uploadMetaGlobal,        new UploadMetaGlobalStage(this));
+    stages.put(Stage.completed,               new CompletedStage(this));
+
+    this.stages = Collections.unmodifiableMap(stages);
   }
 
-  protected GlobalSyncStage getStageByName(Stage next) throws NoSuchStageException {
+  public GlobalSyncStage getSyncStageByName(String name) throws NoSuchStageException {
+    return getSyncStageByName(Stage.byName(name));
+  }
+
+  public GlobalSyncStage getSyncStageByName(Stage next) throws NoSuchStageException {
     GlobalSyncStage stage = stages.get(next);
     if (stage == null) {
       throw new NoSuchStageException(next);
@@ -209,6 +243,32 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     return stage;
   }
 
+  public Collection<GlobalSyncStage> getSyncStagesByEnum(Collection<Stage> enums) {
+    ArrayList<GlobalSyncStage> out = new ArrayList<GlobalSyncStage>();
+    for (Stage name : enums) {
+      try {
+        GlobalSyncStage stage = this.getSyncStageByName(name);
+        out.add(stage);
+      } catch (NoSuchStageException e) {
+        Logger.warn(LOG_TAG, "Unable to find stage with name " + name);
+      }
+    }
+    return out;
+  }
+
+  public Collection<GlobalSyncStage> getSyncStagesByName(Collection<String> names) {
+    ArrayList<GlobalSyncStage> out = new ArrayList<GlobalSyncStage>();
+    for (String name : names) {
+      try {
+        GlobalSyncStage stage = this.getSyncStageByName(name);
+        out.add(stage);
+      } catch (NoSuchStageException e) {
+        Logger.warn(LOG_TAG, "Unable to find stage with name " + name);
+      }
+    }
+    return out;
+  }
+
   /**
    * Advance and loop around the stages of a sync.
    * @param current
@@ -236,7 +296,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     Stage next = nextStage(this.currentState);
     GlobalSyncStage nextStage;
     try {
-      nextStage = this.getStageByName(next);
+      nextStage = this.getSyncStageByName(next);
     } catch (NoSuchStageException e) {
       this.abort(e, "No such stage " + next);
       return;
@@ -244,7 +304,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.currentState = next;
     Logger.info(LOG_TAG, "Running next stage " + next + " (" + nextStage + ")...");
     try {
-      nextStage.execute(this);
+      nextStage.execute();
     } catch (Exception ex) {
       Logger.warn(LOG_TAG, "Caught exception " + ex + " running stage " + next);
       this.abort(ex, "Uncaught exception in stage.");
@@ -252,15 +312,6 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     }
   }
 
-  private String getSyncID() {
-    return config.syncID;
-  }
-
-  private String generateSyncID() {
-    config.syncID = Utils.generateGuid();
-    return config.syncID;
-  }
-
   /*
    * PrefsSource methods.
    */
@@ -269,7 +320,6 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     return this.getContext().getSharedPreferences(name, mode);
   }
 
-  @Override
   public Context getContext() {
     return this.context;
   }
@@ -313,6 +363,102 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.callback.handleSuccess(this);
   }
 
+  /**
+   * Record that an updated meta/global record should be uploaded with the given
+   * settings for the given engine.
+   *
+   * @param engineName engine to update.
+   * @param engineSettings new syncID and version.
+   */
+  public void updateMetaGlobalWith(String engineName, EngineSettings engineSettings) {
+    enginesToUpdate.put(engineName, engineSettings);
+  }
+
+  public boolean hasUpdatedMetaGlobal() {
+    if (enginesToUpdate.isEmpty()) {
+      Logger.info(LOG_TAG, "Not uploading updated meta/global record since there are no engines requesting upload.");
+      return false;
+    }
+
+    if (Logger.logVerbose(LOG_TAG)) {
+      Logger.trace(LOG_TAG, "Uploading updated meta/global record since there are engines requesting upload: " +
+          Utils.toCommaSeparatedString(enginesToUpdate.keySet()));
+    }
+
+    return true;
+  }
+
+  public void updateMetaGlobalInPlace() {
+    ExtendedJSONObject engines = config.metaGlobal.getEngines();
+    for (Entry<String, EngineSettings> pair : enginesToUpdate.entrySet()) {
+      engines.put(pair.getKey(), pair.getValue().toJSONObject());
+    }
+    enginesToUpdate.clear();
+  }
+
+  /**
+   * Synchronously upload an updated meta/global.
+   * <p>
+   * All problems are logged and ignored.
+   */
+  public void uploadUpdatedMetaGlobal() {
+    updateMetaGlobalInPlace();
+
+    Logger.debug(LOG_TAG, "Uploading updated meta/global record.");
+    final Object monitor = new Object();
+
+    Runnable doUpload = new Runnable() {
+      @Override
+      public void run() {
+        config.metaGlobal.upload(new MetaGlobalDelegate() {
+          @Override
+          public void handleSuccess(MetaGlobal global, SyncStorageResponse response) {
+            Logger.info(LOG_TAG, "Successfully uploaded updated meta/global record.");
+            synchronized (monitor) {
+              monitor.notify();
+            }
+          }
+
+          @Override
+          public void handleMissing(MetaGlobal global, SyncStorageResponse response) {
+            Logger.warn(LOG_TAG, "Got 404 missing uploading updated meta/global record; shouldn't happen.  Ignoring.");
+            synchronized (monitor) {
+              monitor.notify();
+            }
+          }
+
+          @Override
+          public void handleFailure(SyncStorageResponse response) {
+            Logger.warn(LOG_TAG, "Failed to upload updated meta/global record; ignoring.");
+            synchronized (monitor) {
+              monitor.notify();
+            }
+          }
+
+          @Override
+          public void handleError(Exception e) {
+            Logger.warn(LOG_TAG, "Got exception trying to upload updated meta/global record; ignoring.", e);
+            synchronized (monitor) {
+              monitor.notify();
+            }
+          }
+        });
+      }
+    };
+
+    final Thread upload = new Thread(doUpload);
+    synchronized (monitor) {
+      try {
+        upload.start();
+        monitor.wait();
+        Logger.debug(LOG_TAG, "Uploaded updated meta/global record.");
+      } catch (InterruptedException e) {
+        Logger.error(LOG_TAG, "Uploading updated meta/global interrupted; continuing.");
+      }
+    }
+  }
+
+
   public void abort(Exception e, String reason) {
     Logger.warn(LOG_TAG, "Aborting sync: " + reason, e);
     uninstallAsHttpResponseObserver();
@@ -320,6 +466,12 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     if (existingBackoff > 0) {
       callback.requestBackoff(existingBackoff);
     }
+    if (!(e instanceof HTTPFailureException)) {
+      //  e is null, or we aborted for a non-HTTP reason; okay to upload new meta/global record.
+      if (this.hasUpdatedMetaGlobal()) {
+        this.uploadUpdatedMetaGlobal(); // Only logs errors; does not call abort.
+      }
+    }
     this.callback.handleError(this, e);
   }
 
@@ -341,21 +493,40 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       callback.requestBackoff(responseBackoff);
     }
 
-    if (response.getStatusLine() != null && response.getStatusLine().getStatusCode() == 401) {
-      /*
-       * Alert our callback we have a 401 on a cluster URL. This GlobalSession
-       * will fail, but the next one will fetch a new cluster URL and will
-       * distinguish between "node reassignment" and "user password changed".
-       */
-      callback.informUnauthorizedResponse(this, config.getClusterURL());
+    if (response.getStatusLine() != null) {
+      final int statusCode = response.getStatusLine().getStatusCode();
+      switch(statusCode) {
+
+      case 400:
+        SyncStorageResponse storageResponse = new SyncStorageResponse(response);
+        this.interpretHTTPBadRequestBody(storageResponse);
+        break;
+
+      case 401:
+        /*
+         * Alert our callback we have a 401 on a cluster URL. This GlobalSession
+         * will fail, but the next one will fetch a new cluster URL and will
+         * distinguish between "node reassignment" and "user password changed".
+         */
+        callback.informUnauthorizedResponse(this, config.getClusterURL());
+        break;
+      }
     }
   }
 
-  public void fetchMetaGlobal(MetaGlobalDelegate callback) throws URISyntaxException {
-    if (this.config.metaGlobal == null) {
-      this.config.metaGlobal = new MetaGlobal(config.metaURL(), credentials());
+  protected void interpretHTTPBadRequestBody(final SyncStorageResponse storageResponse) {
+    try {
+      final String body = storageResponse.body();
+      if (body == null) {
+        return;
+      }
+      if (SyncStorageResponse.RESPONSE_CLIENT_UPGRADE_REQUIRED.equals(body)) {
+        callback.informUpgradeRequiredResponse(this);
+        return;
+      }
+    } catch (Exception e) {
+      Logger.warn(LOG_TAG, "Exception parsing HTTP 400 body.", e);
     }
-    this.config.metaGlobal.fetch(callback);
   }
 
   public void fetchInfoCollections(InfoCollectionsDelegate callback) throws URISyntaxException {
@@ -365,7 +536,15 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.config.infoCollections.fetch(callback);
   }
 
-  public void uploadKeys(CryptoRecord keysRecord,
+  /**
+   * Upload new crypto/keys.
+   *
+   * @param keys
+   *          new keys.
+   * @param keyUploadDelegate
+   *          a delegate.
+   */
+  public void uploadKeys(final CollectionKeys keys,
                          final KeyUploadDelegate keyUploadDelegate) {
     SyncStorageRecordRequest request;
     final GlobalSession self = this;
@@ -385,12 +564,14 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
       @Override
       public void handleRequestSuccess(SyncStorageResponse response) {
+        Logger.debug(LOG_TAG, "Keys uploaded.");
         BaseResource.consumeEntity(response); // We don't need the response at all.
         keyUploadDelegate.onKeysUploaded();
       }
 
       @Override
       public void handleRequestFailure(SyncStorageResponse response) {
+        Logger.debug(LOG_TAG, "Failed to upload keys.");
         self.interpretHTTPFailure(response.httpResponse());
         BaseResource.consumeEntity(response); // The exception thrown should not need the body of the response.
         keyUploadDelegate.onKeyUploadFailed(new HTTPFailureException(response));
@@ -398,6 +579,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
       @Override
       public void handleRequestError(Exception ex) {
+        Logger.warn(LOG_TAG, "Got exception trying to upload keys", ex);
         keyUploadDelegate.onKeyUploadFailed(ex);
       }
 
@@ -407,50 +589,69 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       }
     };
 
-    keysRecord.setKeyBundle(config.syncKeyBundle);
+    // Convert keys to an encrypted crypto record.
+    CryptoRecord keysRecord;
     try {
+      keysRecord = keys.asCryptoRecord();
+      keysRecord.setKeyBundle(config.syncKeyBundle);
       keysRecord.encrypt();
-    } catch (UnsupportedEncodingException e) {
-      keyUploadDelegate.onKeyUploadFailed(e);
-      return;
-    } catch (CryptoException e) {
+    } catch (Exception e) {
+      Logger.warn(LOG_TAG, "Got exception trying creating crypto record from keys", e);
       keyUploadDelegate.onKeyUploadFailed(e);
       return;
     }
+
     request.put(keysRecord);
   }
 
-
   /*
    * meta/global callbacks.
    */
   public void processMetaGlobal(MetaGlobal global) {
+    config.metaGlobal = global;
+
     Long storageVersion = global.getStorageVersion();
+    if (storageVersion == null) {
+      Logger.warn(LOG_TAG, "Malformed remote meta/global: could not retrieve remote storage version.");
+      freshStart();
+      return;
+    }
     if (storageVersion < STORAGE_VERSION) {
-      // Outdated server.
+      Logger.warn(LOG_TAG, "Outdated server: reported " +
+          "remote storage version " + storageVersion + " < " +
+          "local storage version " + STORAGE_VERSION);
       freshStart();
       return;
     }
     if (storageVersion > STORAGE_VERSION) {
-      // Outdated client!
+      Logger.warn(LOG_TAG, "Outdated client: reported " +
+          "remote storage version " + storageVersion + " > " +
+          "local storage version " + STORAGE_VERSION);
       requiresUpgrade();
       return;
     }
     String remoteSyncID = global.getSyncID();
     if (remoteSyncID == null) {
-      // Corrupt meta/global.
+      Logger.warn(LOG_TAG, "Malformed remote meta/global: could not retrieve remote syncID.");
       freshStart();
       return;
     }
-    String localSyncID = this.getSyncID();
+    String localSyncID = config.syncID;
     if (!remoteSyncID.equals(localSyncID)) {
-      // Sync ID has changed. Reset timestamps and fetch new keys.
-      resetClient(null);
-      if (config.collectionKeys != null) {
-        config.collectionKeys.clear();
-      }
+      Logger.warn(LOG_TAG, "Remote syncID different from local syncID: resetting client and assuming remote syncID.");
+      resetAllStages();
+      config.purgeCryptoKeys();
       config.syncID = remoteSyncID;
-      // TODO TODO TODO
+    }
+    // Persist enabled engine names.
+    config.enabledEngineNames = global.getEnabledEngineNames();
+    if (config.enabledEngineNames == null) {
+      Logger.warn(LOG_TAG, "meta/global reported no enabled engine names!");
+    } else {
+      if (Logger.logVerbose(LOG_TAG)) {
+        Logger.trace(LOG_TAG, "Persisting enabled engine names '" +
+            Utils.toCommaSeparatedString(config.enabledEngineNames) + "' from meta/global.");
+      }
     }
     config.persistToPrefs();
     advance();
@@ -463,7 +664,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   /**
    * Do a fresh start then quietly finish the sync, starting another.
    */
-  protected void freshStart() {
+  public void freshStart() {
     final GlobalSession globalSession = this;
     freshStart(this, new FreshStartDelegate() {
 
@@ -475,6 +676,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       @Override
       public void onFreshStart() {
         try {
+          Logger.warn(LOG_TAG, "Fresh start succeeded; restarting global session.");
           globalSession.config.persistToPrefs();
           globalSession.restart();
         } catch (Exception e) {
@@ -487,70 +689,83 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
   /**
    * Clean the server, aborting the current sync.
+   * <p>
+   * <ol>
+   * <li>Wipe the server storage.</li>
+   * <li>Reset all stages and purge cached state: (meta/global and crypto/keys records).</li>
+   * <li>Upload fresh meta/global record.</li>
+   * <li>Upload fresh crypto/keys record.</li>
+   * <li>Restart the sync entirely in order to re-download meta/global and crypto/keys record.</li>
+   * </ol>
+   * @param session the current session.
+   * @param freshStartDelegate delegate to notify on fresh start or failure.
    */
-  protected void freshStart(final GlobalSession session, final FreshStartDelegate freshStartDelegate) {
+  protected static void freshStart(final GlobalSession session, final FreshStartDelegate freshStartDelegate) {
+    Logger.debug(LOG_TAG, "Fresh starting.");
 
-    final String newSyncID   = session.generateSyncID();
-    final String metaURL     = session.config.metaURL();
-    final String credentials = session.credentials();
+    final MetaGlobal mg = session.generateNewMetaGlobal();
 
-    wipeServer(session, new WipeServerDelegate() {
+    session.wipeServer(session, new WipeServerDelegate() {
 
       @Override
       public void onWiped(long timestamp) {
-        session.resetClient(null);
-        session.config.collectionKeys.clear();      // TODO: make sure we clear our keys timestamp.
+        Logger.debug(LOG_TAG, "Successfully wiped server.  Resetting all stages and purging cached meta/global and crypto/keys records.");
+
+        session.resetAllStages();
+        session.config.purgeMetaGlobal();
+        session.config.purgeCryptoKeys();
         session.config.persistToPrefs();
 
-        MetaGlobal mg = new MetaGlobal(metaURL, credentials);
-        mg.setSyncID(newSyncID);
-        mg.setStorageVersion(STORAGE_VERSION);
+        Logger.info(LOG_TAG, "Uploading new meta/global with sync ID " + mg.syncID + ".");
 
         // It would be good to set the X-If-Unmodified-Since header to `timestamp`
         // for this PUT to ensure at least some level of transactionality.
         // Unfortunately, the servers don't support it after a wipe right now
         // (bug 693893), so we're going to defer this until bug 692700.
         mg.upload(new MetaGlobalDelegate() {
-
           @Override
-          public void handleSuccess(MetaGlobal global, SyncStorageResponse response) {
-            session.config.metaGlobal = global;
-            Logger.info(LOG_TAG, "New meta/global uploaded with sync ID " + newSyncID);
+          public void handleSuccess(MetaGlobal uploadedGlobal, SyncStorageResponse uploadResponse) {
+            Logger.info(LOG_TAG, "Uploaded new meta/global with sync ID " + uploadedGlobal.syncID + ".");
 
-            // Generate and upload new keys.
+            // Generate new keys.
+            CollectionKeys keys = null;
             try {
-              session.uploadKeys(CollectionKeys.generateCollectionKeys().asCryptoRecord(), new KeyUploadDelegate() {
-                @Override
-                public void onKeysUploaded() {
-                  // Now we can download them.
-                  freshStartDelegate.onFreshStart();
-                }
-
-                @Override
-                public void onKeyUploadFailed(Exception e) {
-                  Log.e(LOG_TAG, "Got exception uploading new keys.", e);
-                  freshStartDelegate.onFreshStartFailed(e);
-                }
-              });
-            } catch (NoCollectionKeysSetException e) {
-              Log.e(LOG_TAG, "Got exception generating new keys.", e);
-              freshStartDelegate.onFreshStartFailed(e);
+              keys = session.generateNewCryptoKeys();
             } catch (CryptoException e) {
-              Log.e(LOG_TAG, "Got exception generating new keys.", e);
+              Logger.warn(LOG_TAG, "Got exception generating new keys; failing fresh start.", e);
               freshStartDelegate.onFreshStartFailed(e);
             }
+            if (keys == null) {
+              Logger.warn(LOG_TAG, "Got null keys from generateNewKeys; failing fresh start.");
+              freshStartDelegate.onFreshStartFailed(null);
+            }
+
+            // Upload new keys.
+            Logger.info(LOG_TAG, "Uploading new crypto/keys.");
+            session.uploadKeys(keys, new KeyUploadDelegate() {
+              @Override
+              public void onKeysUploaded() {
+                Logger.info(LOG_TAG, "Uploaded new crypto/keys.");
+                freshStartDelegate.onFreshStart();
+              }
+
+              @Override
+              public void onKeyUploadFailed(Exception e) {
+                Logger.warn(LOG_TAG, "Got exception uploading new keys.", e);
+                freshStartDelegate.onFreshStartFailed(e);
+              }
+            });
           }
 
           @Override
           public void handleMissing(MetaGlobal global, SyncStorageResponse response) {
-            // Shouldn't happen.
+            // Shouldn't happen on upload.
             Logger.warn(LOG_TAG, "Got 'missing' response uploading new meta/global.");
-            freshStartDelegate.onFreshStartFailed(new Exception("meta/global missing"));
+            freshStartDelegate.onFreshStartFailed(new Exception("meta/global missing while uploading."));
           }
 
           @Override
           public void handleFailure(SyncStorageResponse response) {
-            // TODO: respect backoffs etc.
             Logger.warn(LOG_TAG, "Got failure " + response.getStatusCode() + " uploading new meta/global.");
             session.interpretHTTPFailure(response.httpResponse());
             freshStartDelegate.onFreshStartFailed(new HTTPFailureException(response));
@@ -561,54 +776,6 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
             Logger.warn(LOG_TAG, "Got error uploading new meta/global.", e);
             freshStartDelegate.onFreshStartFailed(e);
           }
-
-          @Override
-          public MetaGlobalDelegate deferred() {
-            final MetaGlobalDelegate self = this;
-            return new MetaGlobalDelegate() {
-
-              @Override
-              public void handleSuccess(final MetaGlobal global, final SyncStorageResponse response) {
-                ThreadPool.run(new Runnable() {
-                  @Override
-                  public void run() {
-                    self.handleSuccess(global, response);
-                  }});
-              }
-
-              @Override
-              public void handleMissing(final MetaGlobal global, final SyncStorageResponse response) {
-                ThreadPool.run(new Runnable() {
-                  @Override
-                  public void run() {
-                    self.handleMissing(global, response);
-                  }});
-              }
-
-              @Override
-              public void handleFailure(final SyncStorageResponse response) {
-                ThreadPool.run(new Runnable() {
-                  @Override
-                  public void run() {
-                    self.handleFailure(response);
-                  }});
-              }
-
-              @Override
-              public void handleError(final Exception e) {
-                ThreadPool.run(new Runnable() {
-                  @Override
-                  public void run() {
-                    self.handleError(e);
-                  }});
-              }
-
-              @Override
-              public MetaGlobalDelegate deferred() {
-                return this;
-              }
-            };
-          }
         });
       }
 
@@ -618,10 +785,27 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
         freshStartDelegate.onFreshStartFailed(e);
       }
     });
-
   }
 
-  private void wipeServer(final CredentialsSource credentials, final WipeServerDelegate wipeDelegate) {
+  // Note that we do not yet implement wipeRemote: it's only necessary for
+  // first sync options.
+  // -- reset local stages, wipe server for each stage *except* clients
+  //    (stages only, not whole server!), send wipeEngine commands to each client.
+  //
+  // Similarly for startOver (because we don't receive that notification).
+  // -- remove client data from server, reset local stages, clear keys, reset
+  //    backoff, clear all prefs, discard credentials.
+  //
+  // Change passphrase: wipe entire server, reset client to force upload, sync.
+  //
+  // When an engine is disabled: wipe its collections on the server, reupload
+  // meta/global.
+  //
+  // On syncing each stage: if server has engine version 0 or old, wipe server,
+  // reset client to prompt reupload.
+  // If sync ID mismatch: take that syncID and reset client.
+
+  protected void wipeServer(final CredentialsSource credentials, final WipeServerDelegate wipeDelegate) {
     SyncStorageRequest request;
     final GlobalSession self = this;
 
@@ -669,16 +853,129 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     request.delete();
   }
 
-  /**
-   * Reset our state. Clear our sync ID, reset each engine, drop any
-   * cached records.
-   */
-  private void resetClient(String[] engines) {
-    if (engines == null) {
-      // Set `engines` to be *all* the engines.
+  public void wipeAllStages() {
+    Logger.info(LOG_TAG, "Wiping all stages.");
+    // Includes "clients".
+    this.wipeStagesByEnum(Stage.getNamedStages());
+  }
+
+  public static void wipeStages(Collection<GlobalSyncStage> stages) {
+    for (GlobalSyncStage stage : stages) {
+      try {
+        Logger.info(LOG_TAG, "Wiping " + stage);
+        stage.wipeLocal();
+      } catch (Exception e) {
+        Logger.error(LOG_TAG, "Ignoring wipe failure for stage " + stage, e);
+      }
     }
-    // TODO: futz with config?!
-    // TODO: engines?!
+  }
+
+  public void wipeStagesByEnum(Collection<Stage> stages) {
+    GlobalSession.wipeStages(this.getSyncStagesByEnum(stages));
+  }
+
+  public void wipeStagesByName(Collection<String> names) {
+    GlobalSession.wipeStages(this.getSyncStagesByName(names));
+  }
+
+  public void resetAllStages() {
+    Logger.info(LOG_TAG, "Resetting all stages.");
+    // Includes "clients".
+    this.resetStagesByEnum(Stage.getNamedStages());
+  }
+
+  public static void resetStages(Collection<GlobalSyncStage> stages) {
+    for (GlobalSyncStage stage : stages) {
+      try {
+        Logger.info(LOG_TAG, "Resetting " + stage);
+        stage.resetLocal();
+      } catch (Exception e) {
+        Logger.error(LOG_TAG, "Ignoring reset failure for stage " + stage, e);
+      }
+    }
+  }
+
+  public void resetStagesByEnum(Collection<Stage> stages) {
+    GlobalSession.resetStages(this.getSyncStagesByEnum(stages));
+  }
+
+  public void resetStagesByName(Collection<String> names) {
+    Collection<GlobalSyncStage> stages = new ArrayList<GlobalSyncStage>();
+    for (String name : names) {
+      try {
+        GlobalSyncStage stage = this.getSyncStageByName(name);
+        stages.add(stage);
+      } catch (NoSuchStageException e) {
+        Logger.warn(LOG_TAG, "Cannot reset stage " + name + ": no such stage.");
+      }
+    }
+    GlobalSession.resetStages(stages);
+  }
+
+  /**
+   * Engines to include in a fresh meta/global record.
+   * <p>
+   * Returns either the persisted engine names (perhaps we have been node
+   * re-assigned and are initializing a clean server: we want to upload the
+   * persisted engine names so that we don't accidentally disable engines that
+   * Android Sync doesn't recognize), or the set of engines names that Android
+   * Sync implements.
+   *
+   * @return set of engine names.
+   */
+  protected Set<String> enabledEngineNames() {
+    if (config.enabledEngineNames != null) {
+      return config.enabledEngineNames;
+    }
+    Set<String> engineNames = new HashSet<String>();
+    for (Stage stage : Stage.getNamedStages()) {
+      engineNames.add(stage.getRepositoryName());
+    }
+    return engineNames;
+  }
+
+  /**
+   * Generate fresh crypto/keys collection.
+   * @return crypto/keys collection.
+   * @throws CryptoException
+   */
+  public CollectionKeys generateNewCryptoKeys() throws CryptoException {
+    return CollectionKeys.generateCollectionKeys();
+  }
+
+  /**
+   * Generate a fresh meta/global record.
+   * @return meta/global record.
+   */
+  public MetaGlobal generateNewMetaGlobal() {
+    final String newSyncID   = Utils.generateGuid();
+    final String metaURL     = this.config.metaURL();
+    final String credentials = this.credentials();
+
+    ExtendedJSONObject engines = new ExtendedJSONObject();
+    for (String engineName : enabledEngineNames()) {
+      EngineSettings engineSettings = null;
+      try {
+        GlobalSyncStage globalStage = this.getSyncStageByName(engineName);
+        Integer version = globalStage.getStorageVersion();
+        if (version == null) {
+          continue; // Don't want this stage to be included in meta/global.
+        }
+        engineSettings = new EngineSettings(Utils.generateGuid(), version.intValue());
+      } catch (NoSuchStageException e) {
+        // No trouble; Android Sync might not recognize this engine yet.
+        // By default, version 0.  Other clients will see the 0 version and reset/wipe accordingly.
+        engineSettings = new EngineSettings(Utils.generateGuid(), 0);
+      }
+      engines.put(engineName, engineSettings.toJSONObject());
+    }
+
+    MetaGlobal metaGlobal = new MetaGlobal(metaURL, credentials);
+    metaGlobal.setSyncID(newSyncID);
+    metaGlobal.setStorageVersion(STORAGE_VERSION);
+    metaGlobal.setEngines(engines);
+
+    return metaGlobal;
   }
 
   /**
@@ -688,6 +985,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   public void requiresUpgrade() {
     Logger.info(LOG_TAG, "Client outdated storage version; requires update.");
     // TODO: notify UI.
+    this.abort(null, "Requires upgrade");
   }
 
   /**
@@ -695,21 +993,40 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
    * Otherwise, returns true if there is an entry for this engine in the
    * meta/global "engines" object.
    *
-   * @param engineName
+   * @param engineName the name to check (e.g., "bookmarks").
+   * @param engineSettings
+   *        if non-null, verify that the server engine settings are congruent
+   *        with this, throwing the appropriate MetaGlobalException if not.
    * @return
    *        true if the engine with the provided name is present in the
-   *        meta/global "engines" object.
+   *        meta/global "engines" object, and verification passed.
    *
    * @throws MetaGlobalException
    */
-  public boolean engineIsEnabled(String engineName) throws MetaGlobalException {
+  public boolean engineIsEnabled(String engineName, EngineSettings engineSettings) throws MetaGlobalException {
     if (this.config.metaGlobal == null) {
       throw new MetaGlobalNotSetException();
     }
-    if (this.config.metaGlobal.engines == null) {
+
+    // This should not occur.
+    if (this.config.enabledEngineNames == null) {
+      Logger.error(LOG_TAG, "No enabled engines in config. Giving up.");
       throw new MetaGlobalMissingEnginesException();
     }
-    return this.config.metaGlobal.engines.get(engineName) != null;
+
+    if (!(this.config.enabledEngineNames.contains(engineName))) {
+      Logger.debug(LOG_TAG, "Engine " + engineName + " not enabled: no meta/global entry.");
+      return false;
+    }
+
+    // If we have a meta/global, check that it's safe for us to sync.
+    // (If we don't, we'll create one later, which is why we return `true` above.)
+    if (engineSettings != null) {
+      // Throws if there's a problem.
+      this.config.metaGlobal.verifyEngineSettings(engineName, engineSettings);
+    }
+
+    return true;
   }
 
   public ClientsDataDelegate getClientsDelegate() {
