@@ -72,8 +72,8 @@ Cu.import("resource://services-sync/engines.js");
 Cu.import("resource://services-sync/record.js");
 Cu.import("resource://services-sync/util.js");
 Cu.import("resource://services-sync/constants.js");
-Cu.import("resource://services-sync/async.js");
-Cu.import("resource://services-sync/ext/Preferences.js");
+Cu.import("resource://services-common/async.js");
+Cu.import("resource://services-common/preferences.js");
 
 Cu.import("resource://gre/modules/AddonManager.jsm");
 Cu.import("resource://gre/modules/AddonRepository.jsm");
