@@ -677,7 +677,7 @@ Connection::initialize(nsIFile *aDatabaseFile,
                                "PRAGMA cache_size = ");
   cacheSizeQuery.AppendInt(NS_MIN(DEFAULT_CACHE_SIZE_PAGES,
                                   PRInt32(MAX_CACHE_SIZE_BYTES / pageSize)));
-  srv = ::sqlite3_exec(mDBConn, cacheSizeQuery.get(), NULL, NULL, NULL);
+  srv = executeSql(cacheSizeQuery.get());
   if (srv != SQLITE_OK) {
     ::sqlite3_close(mDBConn);
     mDBConn = nsnull;
@@ -910,9 +910,16 @@ Connection::stepStatement(sqlite3_stmt *aStatement)
   // Report very slow SQL statements to Telemetry
   TimeDuration duration = TimeStamp::Now() - startTime;
   if (duration.ToMilliseconds() >= Telemetry::kSlowStatementThreshold) {
-    nsDependentCString statementString(::sqlite3_sql(aStatement));
-    Telemetry::RecordSlowSQLStatement(statementString, getFilename(),
-                                      duration.ToMilliseconds());
+    const char *sql = ::sqlite3_sql(aStatement);
+    // FIXME: Try runs have found hard to reproduce crashes where sql is NULL.
+    // It is not clear how can we get a NULL sql statement in here.
+    // sqlite3_prepare_v2 always copies the incoming argument and fails
+    // if it runs out of memory.
+    if (sql) {
+      nsDependentCString statementString(sql);
+      Telemetry::RecordSlowSQLStatement(statementString, getFilename(),
+                                        duration.ToMilliseconds(), false);
+    }
   }
 
   (void)::sqlite3_extended_result_codes(mDBConn, 0);
@@ -965,6 +972,27 @@ Connection::prepareStatement(const nsCString &aSQL,
   return srv & 0xFF;
 }
 
+
+int
+Connection::executeSql(const char *aSqlString)
+{
+  if (!mDBConn)
+    return SQLITE_MISUSE;
+
+  TimeStamp startTime = TimeStamp::Now();
+  int srv = ::sqlite3_exec(mDBConn, aSqlString, NULL, NULL, NULL);
+
+  // Report very slow SQL statements to Telemetry
+  TimeDuration duration = TimeStamp::Now() - startTime;
+  if (duration.ToMilliseconds() >= Telemetry::kSlowStatementThreshold) {
+    nsDependentCString statementString(aSqlString);
+    Telemetry::RecordSlowSQLStatement(statementString, getFilename(),
+                                      duration.ToMilliseconds(), true);
+  }
+
+  return srv;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //// nsIInterfaceRequestor
 
@@ -1226,8 +1254,7 @@ Connection::ExecuteSimpleSQL(const nsACString &aSQLStatement)
 {
   if (!mDBConn) return NS_ERROR_NOT_INITIALIZED;
 
-  int srv = ::sqlite3_exec(mDBConn, PromiseFlatCString(aSQLStatement).get(),
-                           NULL, NULL, NULL);
+  int srv = executeSql(PromiseFlatCString(aSQLStatement).get());
   return convertResultCode(srv);
 }
 
@@ -1357,7 +1384,7 @@ Connection::CreateTable(const char *aTableName,
   if (!buf)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  int srv = ::sqlite3_exec(mDBConn, buf, NULL, NULL, NULL);
+  int srv = executeSql(buf);
   ::PR_smprintf_free(buf);
 
   return convertResultCode(srv);
