@@ -96,34 +96,6 @@ Tester.prototype = {
     this._globalProperties = Object.keys(window);
     this._globalPropertyWhitelist = ["navigator", "constructor", "Application",
       "__SS_tabsToRestore", "__SSi", "webConsoleCommandController",
-
-      // Temporarily added to whitelist for Fennec tests:
-      "AddonUpdateChecker",
-      "PlacesAggregatedTransaction",
-      "PlacesCreateFolderTransaction",
-      "PlacesCreateBookmarkTransaction",
-      "PlacesCreateSeparatorTransaction",
-      "PlacesCreateLivemarkTransaction",
-      "PlacesMoveItemTransaction",
-      "PlacesRemoveItemTransaction",
-      "PlacesEditItemTitleTransaction",
-      "PlacesEditBookmarkURITransaction",
-      "PlacesSetItemAnnotationTransaction",
-      "PlacesSetPageAnnotationTransaction",
-      "PlacesEditBookmarkKeywordTransaction",
-      "PlacesEditBookmarkPostDataTransaction",
-      "PlacesEditLivemarkSiteURITransaction",
-      "PlacesEditLivemarkFeedURITransaction",
-      "PlacesEditItemDateAddedTransaction",
-      "PlacesEditItemLastModifiedTransaction",
-      "PlacesSortFolderByNameTransaction",
-      "PlacesTagURITransaction",
-      "PlacesUntagURITransaction",
-      "DownloadUtils",
-      "AddonRepository",
-      "LocaleRepository",
-      "Contacts",
-      "VKBstateHasChanged"
     ];
 
     if (this.tests.length)
