@@ -505,23 +505,6 @@ const queryOptionSwitches = [
       }
     ]
   },
-  // redirectsMode
-  {
-    property: "redirectsMode",
-    desc:     "nsINavHistoryQueryOptions.redirectsMode",
-    matches:  simplePropertyMatches,
-    runs:     [
-      function (aQuery, aQueryOptions) {
-        aQueryOptions.redirectsMode = aQueryOptions.REDIRECTS_MODE_ALL;
-      },
-      function (aQuery, aQueryOptions) {
-        aQueryOptions.redirectsMode = aQueryOptions.REDIRECTS_MODE_TARGET;
-      },
-      function (aQuery, aQueryOptions) {
-        aQueryOptions.redirectsMode = aQueryOptions.REDIRECTS_MODE_SOURCE;
-      }
-    ]
-  },
 ];
 
 ///////////////////////////////////////////////////////////////////////////////
