@@ -121,7 +121,7 @@ FormHistory.prototype = {
     },
 
 
-    log : function (message) {
+    log : function log(message) {
         if (!this.debug)
             return;
         dump("FormHistory: " + message + "\n");
@@ -129,7 +129,7 @@ FormHistory.prototype = {
     },
 
 
-    init : function() {
+    init : function init() {
         Services.prefs.addObserver("browser.formfill.", this, true);
 
         this.updatePrefs();
@@ -175,7 +175,7 @@ FormHistory.prototype = {
     },
 
 
-    addEntry : function (name, value) {
+    addEntry : function addEntry(name, value) {
         if (!this.enabled ||
             this.privBrowsingSvc && this.privBrowsingSvc.privateBrowsingEnabled)
             return;
@@ -188,7 +188,7 @@ FormHistory.prototype = {
         let stmt;
 
         if (id != -1) {
-            // Update existing entry
+            // Update existing entry.
             let query = "UPDATE moz_formhistory SET timesUsed = timesUsed + 1, lastUsed = :lastUsed WHERE id = :id";
             let params = {
                             lastUsed : now,
@@ -209,7 +209,7 @@ FormHistory.prototype = {
             }
 
         } else {
-            // Add new entry
+            // Add new entry.
             guid = this.generateGUID();
 
             let query = "INSERT INTO moz_formhistory (fieldname, value, timesUsed, firstUsed, lastUsed, guid) " +
@@ -239,7 +239,7 @@ FormHistory.prototype = {
     },
 
 
-    removeEntry : function (name, value) {
+    removeEntry : function removeEntry(name, value) {
         this.log("removeEntry for " + name + "=" + value);
 
         let [id, guid] = this.getExistingEntryID(name, value);
@@ -279,7 +279,7 @@ FormHistory.prototype = {
     },
 
 
-    removeEntriesForName : function (name) {
+    removeEntriesForName : function removeEntriesForName(name) {
         this.log("removeEntriesForName with name=" + name);
 
         this.sendStringNotification("before-removeEntriesForName", name);
@@ -319,7 +319,7 @@ FormHistory.prototype = {
     },
 
 
-    removeAllEntries : function () {
+    removeAllEntries : function removeAllEntries() {
         this.log("removeAllEntries");
 
         this.sendNotification("before-removeAllEntries", null);
@@ -333,18 +333,15 @@ FormHistory.prototype = {
             existingTransactionInProgress = this.dbConnection.transactionInProgress;
             if (!existingTransactionInProgress)
                 this.dbConnection.beginTransaction();
-            this.moveToDeletedTable(
-              "SELECT guid, :timeDeleted FROM moz_formhistory", {
-              timeDeleted: Date.now()
-            });
-
+            // TODO: Add these items to the deleted items table once we've sorted
+            //       out the issues from bug 756701
             stmt = this.dbCreateStatement(query);
             stmt.execute();
             this.sendNotification("removeAllEntries", null);
         } catch (e) {
             if (!existingTransactionInProgress)
                 this.dbConnection.rollbackTransaction();
-            this.log("removeEntriesForName failed: " + e);
+            this.log("removeAllEntries failed: " + e);
             throw e;
         } finally {
             if (stmt) {
@@ -356,7 +353,7 @@ FormHistory.prototype = {
     },
 
 
-    nameExists : function (name) {
+    nameExists : function nameExists(name) {
         this.log("nameExists for name=" + name);
         let stmt;
         let query = "SELECT COUNT(1) AS numEntries FROM moz_formhistory WHERE fieldname = :fieldname";
@@ -375,14 +372,14 @@ FormHistory.prototype = {
         }
     },
 
-    entryExists : function (name, value) {
+    entryExists : function entryExists(name, value) {
         this.log("entryExists for " + name + "=" + value);
         let [id, guid] = this.getExistingEntryID(name, value);
         this.log("entryExists: id=" + id);
         return (id != -1);
     },
 
-    removeEntriesByTimeframe : function (beginTime, endTime) {
+    removeEntriesByTimeframe : function removeEntriesByTimeframe(beginTime, endTime) {
         this.log("removeEntriesByTimeframe for " + beginTime + " to " + endTime);
 
         this.sendIntNotification("before-removeEntriesByTimeframe", beginTime, endTime);
@@ -424,20 +421,20 @@ FormHistory.prototype = {
             this.dbConnection.commitTransaction();
     },
 
-    moveToDeletedTable : function (values, params) {
+    moveToDeletedTable : function moveToDeletedTable(values, params) {
 #ifdef ANDROID
-        this.log("move entries to deleted");
+        this.log("Moving entries to deleted table.");
 
         let stmt;
 
         try {
-            // move the entry to the deleted items table
+            // Move the entries to the deleted items table.
             let query = "INSERT INTO moz_deleted_formhistory (guid, timeDeleted) ";
             if (values) query += values;
             stmt = this.dbCreateStatement(query, params);
             stmt.execute();
         } catch (e) {
-            this.log("move entry failed: " + e);
+            this.log("Moving deleted entries failed: " + e);
             throw e;
         } finally {
             if (stmt) {
@@ -481,7 +478,7 @@ FormHistory.prototype = {
     /* ---- nsIObserver interface ---- */
 
 
-    observe : function (subject, topic, data) {
+    observe : function observe(subject, topic, data) {
         switch(topic) {
         case "nsPref:changed":
             this.updatePrefs();
