@@ -8,23 +8,28 @@ Components.utils.import("resource://gre/modules/Services.jsm");
 function onResetProfileLoad() {
 #expand const MOZ_BUILD_APP = "__MOZ_BUILD_APP__";
 #expand const MOZ_APP_NAME = "__MOZ_APP_NAME__";
-  const MAX_MIGRATED_TYPES = 16;
+
+  // From migration.properties
+  const MIGRATED_TYPES = [
+    "4_" + MOZ_APP_NAME + "_history_and_bookmarks",
+    "16_" + MOZ_APP_NAME, // Passwords
+    "8_" + MOZ_APP_NAME,  // Form History
+    "2_" + MOZ_APP_NAME,  // Cookies
+  ];
 
   var migratedItems = document.getElementById("migratedItems");
   var bundle = Services.strings.createBundle("chrome://" + MOZ_BUILD_APP +
                                              "/locale/migration/migration.properties");
 
-  // Loop over possible data to migrate to give the user a list of what will be preserved. This
-  // assumes that if the string for the data exists then it will be migrated since calling
-  // getMigrateData now won't give the correct result.
-  for (var i = 1; i < MAX_MIGRATED_TYPES; ++i) {
-    var itemID = Math.pow(2, i);
+  // Loop over possible data to migrate to give the user a list of what will be preserved.
+  for (var itemStringName of MIGRATED_TYPES) {
     try {
       var checkbox = document.createElement("label");
-      checkbox.setAttribute("value", bundle.GetStringFromName(itemID + "_" + MOZ_APP_NAME));
+      checkbox.setAttribute("value", bundle.GetStringFromName(itemStringName));
       migratedItems.appendChild(checkbox);
     } catch (x) {
-      // Catch exceptions when the string for a data type doesn't exist because it's not migrated
+      // Catch exceptions when the string for a data type doesn't exist.
+      Components.utils.reportError(x);
     }
   }
 }
