@@ -37,11 +37,14 @@
 
 #include <EGL/egl.h>
 #include <EGL/eglext.h>
+#include <fcntl.h>
 
 #include "android/log.h"
 #include "ui/FramebufferNativeWindow.h"
 
 #include "mozilla/Hal.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/FileUtils.h"
 #include "Framebuffer.h"
 #include "gfxContext.h"
 #include "gfxUtils.h"
@@ -74,32 +77,127 @@ static nsWindow *gWindowToRedraw = nsnull;
 static nsWindow *gFocusedWindow = nsnull;
 static android::FramebufferNativeWindow *gNativeWindow = nsnull;
 static bool sFramebufferOpen;
+static bool sUsingOMTC;
+static nsRefPtr<gfxASurface> sOMTCSurface;
+static nsCOMPtr<nsIThread> sFramebufferWatchThread;
+
+namespace {
+
+class ScreenOnOffEvent : public nsRunnable {
+public:
+    ScreenOnOffEvent(bool on)
+        : mIsOn(on)
+    {}
+
+    NS_IMETHOD Run() {
+        nsSizeModeEvent event(true, NS_SIZEMODE, NULL);
+        nsEventStatus status;
+
+        event.time = PR_Now() / 1000;
+        event.mSizeMode = mIsOn ? nsSizeMode_Fullscreen : nsSizeMode_Minimized;
+
+        for (PRUint32 i = 0; i < sTopWindows.Length(); i++) {
+            nsWindow *win = sTopWindows[i];
+            event.widget = win;
+            win->DispatchEvent(&event, status);
+        }
+
+        return NS_OK;
+    }
+
+private:
+    bool mIsOn;
+};
+
+static const char* kSleepFile = "/sys/power/wait_for_fb_sleep";
+static const char* kWakeFile = "/sys/power/wait_for_fb_wake";
+
+class FramebufferWatcher : public nsRunnable {
+public:
+    FramebufferWatcher()
+        : mScreenOnEvent(new ScreenOnOffEvent(true))
+        , mScreenOffEvent(new ScreenOnOffEvent(false))
+    {}
+
+    NS_IMETHOD Run() {
+        int len = 0;
+        char buf;
+
+        // Cannot use epoll here because kSleepFile and kWakeFile are
+        // always ready to read and blocking.
+        {
+            ScopedClose fd(open(kSleepFile, O_RDONLY, 0));
+            do {
+                len = read(fd.get(), &buf, 1);
+            } while (len < 0 && errno == EINTR);
+            NS_WARN_IF_FALSE(len >= 0, "WAIT_FOR_FB_SLEEP failed");
+            NS_DispatchToMainThread(mScreenOffEvent);
+        }
+
+        {
+            ScopedClose fd(open(kWakeFile, O_RDONLY, 0));
+            do {
+                len = read(fd.get(), &buf, 1);
+            } while (len < 0 && errno == EINTR);
+            NS_WARN_IF_FALSE(len >= 0, "WAIT_FOR_FB_WAKE failed");
+            NS_DispatchToMainThread(mScreenOnEvent);
+        }
+
+        // Dispatch to ourself.
+        NS_DispatchToCurrentThread(this);
+
+        return NS_OK;
+    }
+
+private:
+    nsRefPtr<ScreenOnOffEvent> mScreenOnEvent;
+    nsRefPtr<ScreenOnOffEvent> mScreenOffEvent;
+};
+
+} // anonymous namespace
 
 nsWindow::nsWindow()
 {
-    if (!sGLContext && !sFramebufferOpen) {
+    if (!sGLContext && !sFramebufferOpen && !sUsingOMTC) {
         // workaround Bug 725143
         hal::SetScreenEnabled(true);
 
+        // Watching screen on/off state
+        NS_NewThread(getter_AddRefs(sFramebufferWatchThread), new FramebufferWatcher());
+
+        sUsingOMTC = Preferences::GetBool("layers.offmainthreadcomposition.enabled", false);
+
         // We (apparently) don't have a way to tell if allocating the
         // fbs succeeded or failed.
         gNativeWindow = new android::FramebufferNativeWindow();
-        sGLContext = GLContextProvider::CreateForWindow(this);
-        // CreateForWindow sets up gScreenBounds
-        if (!sGLContext) {
-            LOG("Failed to create GL context for fb, trying /dev/graphics/fb0");
-
-            // We can't delete gNativeWindow.
-
+        if (sUsingOMTC) {
             nsIntSize screenSize;
-            sFramebufferOpen = Framebuffer::Open(&screenSize);
+            bool gotFB = Framebuffer::GetSize(&screenSize);
+            MOZ_ASSERT(gotFB);
             gScreenBounds = nsIntRect(nsIntPoint(0, 0), screenSize);
-            if (!sFramebufferOpen) {
-                LOG("Failed to mmap fb(?!?), aborting ...");
-                NS_RUNTIMEABORT("Can't open GL context and can't fall back on /dev/graphics/fb0 ...");
+
+            sOMTCSurface = new gfxImageSurface(gfxIntSize(1, 1),
+                gfxASurface::ImageFormatRGB24);
+        } else {
+            sGLContext = GLContextProvider::CreateForWindow(this);
+            // CreateForWindow sets up gScreenBounds
+            if (!sGLContext) {
+                LOG("Failed to create GL context for fb, trying /dev/graphics/fb0");
+
+                // We can't delete gNativeWindow.
+
+                nsIntSize screenSize;
+                sFramebufferOpen = Framebuffer::Open(&screenSize);
+                gScreenBounds = nsIntRect(nsIntPoint(0, 0), screenSize);
+                if (!sFramebufferOpen) {
+                    LOG("Failed to mmap fb(?!?), aborting ...");
+                    NS_RUNTIMEABORT("Can't open GL context and can't fall back on /dev/graphics/fb0 ...");
+                }
             }
         }
         sVirtualBounds = gScreenBounds;
+
+        nsAppShell::NotifyScreenInitialized();
     }
 }
 
@@ -131,11 +229,16 @@ nsWindow::DoDraw(void)
         oglm->SetWorldTransform(sRotationMatrix);
         gWindowToRedraw->mEventCallback(&event);
     } else if (LayerManager::LAYERS_BASIC == lm->GetBackendType()) {
-        MOZ_ASSERT(sFramebufferOpen);
+        MOZ_ASSERT(sFramebufferOpen || sUsingOMTC);
+        nsRefPtr<gfxASurface> targetSurface;
+
+        if(sUsingOMTC)
+            targetSurface = sOMTCSurface;
+        else
+            targetSurface = Framebuffer::BackBuffer();
 
-        nsRefPtr<gfxASurface> backBuffer = Framebuffer::BackBuffer();
         {
-            nsRefPtr<gfxContext> ctx = new gfxContext(backBuffer);
+            nsRefPtr<gfxContext> ctx = new gfxContext(targetSurface);
             gfxUtils::PathFromRegion(ctx, event.region);
             ctx->Clip();
 
@@ -144,9 +247,11 @@ nsWindow::DoDraw(void)
                 gWindowToRedraw, ctx, BasicLayerManager::BUFFER_NONE);
             gWindowToRedraw->mEventCallback(&event);
         }
-        backBuffer->Flush();
 
-        Framebuffer::Present(event.region);
+        if (!sUsingOMTC) {
+            targetSurface->Flush();
+            Framebuffer::Present(event.region);
+        }
     } else {
         NS_RUNTIMEABORT("Unexpected layer manager type");
     }
@@ -409,6 +514,12 @@ nsWindow::GetLayerManager(PLayersChild* aShadowManager,
         return nsnull;
     }
 
+    if (sUsingOMTC) {
+        CreateCompositor();
+        if (mLayerManager)
+            return mLayerManager;
+    }
+
     if (sGLContext) {
         nsRefPtr<LayerManagerOGL> layerManager = new LayerManagerOGL(this);
 
