@@ -118,8 +118,8 @@ typedef nsEventStatus (* EVENT_CALLBACK)(nsGUIEvent *event);
 #endif
 
 #define NS_IWIDGET_IID \
-  { 0x3fa36ce2, 0x472d, 0x4bff, \
-    { 0xb1, 0xe4, 0xc3, 0xe3, 0x19, 0x24, 0xa1, 0xe4 } }
+  { 0x7c7ff2ff, 0x61f9, 0x4240, \
+    { 0xaa, 0x58, 0x74, 0xb0, 0xcd, 0xa9, 0xe3, 0x05 } }
 
 /*
  * Window shadow styles
@@ -1017,6 +1017,14 @@ class nsIWidget : public nsISupports {
      */
     virtual void SetShowsToolbarButton(bool aShow) = 0;
 
+    /*
+     * On Mac OS X Lion, this method shows or hides the full screen button in
+     * the titlebar that handles native full screen mode.
+     *
+     * Ignored on child widgets, non-Mac platforms, & pre-Lion Mac.
+     */
+    virtual void SetShowsFullScreenButton(bool aShow) = 0;
+
     enum WindowAnimationType {
       eGenericWindowAnimation,
       eDocumentWindowAnimation
@@ -1085,6 +1093,14 @@ class nsIWidget : public nsISupports {
                                           LayerManagerPersistence aPersistence = LAYER_MANAGER_CURRENT,
                                           bool* aAllowRetaining = nsnull) = 0;
 
+    /**
+     * Called before the LayerManager draws the layer tree.
+     *
+     * @param aManager The drawing LayerManager.
+     * @param aWidgetRect The current widget rect that is being drawn.
+     */
+    virtual void DrawWindowUnderlay(LayerManager* aManager, nsIntRect aRect) = 0;
+
     /**
      * Called after the LayerManager draws the layer tree
      *
@@ -1360,6 +1376,40 @@ class nsIWidget : public nsISupports {
                                                 PRUint32 aNativeMessage,
                                                 PRUint32 aModifierFlags) = 0;
 
+    /**
+     * A shortcut to SynthesizeNativeMouseEvent, abstracting away the native message.
+     */
+    virtual nsresult SynthesizeNativeMouseMove(nsIntPoint aPoint) = 0;
+
+    /**
+     * Utility method intended for testing. Dispatching native mouse scroll
+     * events may move the mouse cursor.
+     *
+     * @param aPoint            Mouse cursor position in screen coordinates.
+     *                          In device pixels, the origin at the top left of
+     *                          the primary display.
+     * @param aNativeMessage    Platform native message.
+     * @param aDeltaX           The delta value for X direction.  If the native
+     *                          message doesn't indicate X direction scrolling,
+     *                          this may be ignored.
+     * @param aDeltaY           The delta value for Y direction.  If the native
+     *                          message doesn't indicate Y direction scrolling,
+     *                          this may be ignored.
+     * @param aDeltaZ           The delta value for Z direction.  If the native
+     *                          message doesn't indicate Z direction scrolling,
+     *                          this may be ignored.
+     * @param aModifierFlags    Must be values of Modifiers, or zero.
+     * @param aAdditionalFlags  See nsIDOMWidnowUtils' consts and their
+     *                          document.
+     */
+    virtual nsresult SynthesizeNativeMouseScrollEvent(nsIntPoint aPoint,
+                                                      PRUint32 aNativeMessage,
+                                                      double aDeltaX,
+                                                      double aDeltaY,
+                                                      double aDeltaZ,
+                                                      PRUint32 aModifierFlags,
+                                                      PRUint32 aAdditionalFlags) = 0;
+
     /**
      * Activates a native menu item at the position specified by the index
      * string. The index string is a string of positive integers separated
@@ -1539,6 +1589,19 @@ class nsIWidget : public nsISupports {
      * widget.
      */
     virtual PRUint32 GetGLFrameBufferFormat() { return 0; /*GL_NONE*/ }
+
+    /**
+     * Return true if widget has it's own GL context
+     */
+    virtual bool HasGLContext() { return false; }
+
+    /**
+     * Returns true to indicate that this widget paints an opaque background
+     * that we want to be visible under the page, so layout should not force
+     * a default background.
+     */
+    virtual bool WidgetPaintsBackground() { return false; }
+
 protected:
 
     // keep the list of children.  We also keep track of our siblings.
