@@ -42,6 +42,7 @@
 #define __nsWindow_h__
 
 #include <QKeyEvent>
+#include <QGestureEvent>
 #include <qgraphicswidget.h>
 #include <QTime>
 
@@ -195,6 +196,7 @@ public:
     NS_IMETHOD_(void) SetInputContext(const InputContext& aContext,
                                       const InputContextAction& aAction);
     NS_IMETHOD_(InputContext) GetInputContext();
+    NS_IMETHOD_(bool)  HasGLContext();
 
     //
     // utility methods
