@@ -151,7 +151,8 @@ StackWalkInitCriticalAddress()
   // we force a situation where new_sem_from_pool is on the stack and
   // use dladdr to check the addresses.
 
-  MOZ_ASSERT(malloc_logger == NULL);
+  // malloc_logger can be set by external tools like 'Instruments' or 'leaks'
+  malloc_logger_t *old_malloc_logger = malloc_logger;
   malloc_logger = my_malloc_logger;
 
   pthread_cond_t cond;
@@ -164,7 +165,9 @@ StackWalkInitCriticalAddress()
   MOZ_ASSERT(r == 0);
   struct timespec abstime = {0, 1};
   r = pthread_cond_timedwait_relative_np(&cond, &mutex, &abstime);
-  malloc_logger = NULL;
+
+  // restore the previous malloc logger
+  malloc_logger = old_malloc_logger;
 
   // On Lion, malloc is no longer called from pthread_cond_*wait*. This prevents
   // us from finding the address, but that is fine, since with no call to malloc
