@@ -1413,6 +1413,12 @@ void nsXULWindow::SyncAttributesToWidget()
     mWindow->SetShowsToolbarButton(attr.LowerCaseEqualsLiteral("true"));
   }
 
+  // "fullscreenbutton" attribute
+  rv = windowElement->GetAttribute(NS_LITERAL_STRING("fullscreenbutton"), attr);
+  if (NS_SUCCEEDED(rv)) {
+    mWindow->SetShowsFullScreenButton(attr.LowerCaseEqualsLiteral("true"));
+  }
+
   // "macanimationtype" attribute
   rv = windowElement->GetAttribute(NS_LITERAL_STRING("macanimationtype"), attr);
   if (NS_SUCCEEDED(rv) && attr.EqualsLiteral("document")) {
